@@ -2,7 +2,7 @@
  *  Author: Viktor Dukhovni
  *  License: THIS CODE IS IN THE PUBLIC DOMAIN.
  *
- * Copyright (c) The Exim Maintainers 2014 - 2025
+ * Copyright (c) The Exim Maintainers 2014 - 2018
  */
 #include <stdio.h>
 #include <string.h>
@@ -22,45 +22,13 @@
 # error "OpenSSL 1.0.0 or higher required"
 #endif
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
 # define X509_up_ref(x) CRYPTO_add(&((x)->references), 1, CRYPTO_LOCK_X509)
 #endif
-#if defined(LIBRESSL_VERSION_NUMBER) && LIBRESSL_VERSION_NUMBER < 0x3050000fL
-/* Exact version number uncertain */
-# define X509_up_ref(x) CRYPTO_add(&((x)->references), 1, CRYPTO_LOCK_X509)
-#endif
-
-#ifndef LIBRESSL_VERSION_NUMBER		/* OpenSSL */
-# if OPENSSL_VERSION_NUMBER >= 0x10100000L
-					/* Newer OpenSSL */
-#  define EXIM_HAVE_ASN1_MACROS
-#  define EXIM_OPAQUE_X509
-# else
-					/* Older OpenSSL */
-#  define EXIM_TRANSPARENT_CTX
-#  define EXIM_NEED_CRYPTO_INIT
-# endif
-
-#else					/* LibreSSL */
-# if LIBRESSL_VERSION_NUMBER >= 0x3050000fL
-#  define EXIM_OPAQUE_X509		/* Exact version number uncertain */
-#  define EXIM_NO_NEED_SHA2_REGISTER
-#  define EXIM_HAVE_ASN1_MACROS
-# endif
-
-# if LIBRESSL_VERSION_NUMBER >= 0x2090000fL
-    /* LibreSSL 2.9.0 and later - 2.9.0 has removed a number of macros ... */
-#  define EXIM_HAVE_ASN1_MACROS
-
-# else
-#  define EXIM_TRANSPARENT_CTX
-# endif
-
-# define EXIM_NEED_CRYPTO_INIT
-#endif
-
-#ifdef EXIM_TRANSPARENT_CTX
-/* Older OpenSSL and LibreSSL */
+#if OPENSSL_VERSION_NUMBER >= 0x10100000L && !defined(LIBRESSL_VERSION_NUMBER)
+# define EXIM_HAVE_ASN1_MACROS
+# define EXIM_OPAQUE_X509
+#else
 # define X509_STORE_CTX_get_verify(ctx)		(ctx)->verify
 # define X509_STORE_CTX_get_verify_cb(ctx)	(ctx)->verify_cb
 # define X509_STORE_CTX_get0_cert(ctx)		(ctx)->cert
@@ -75,9 +43,7 @@
 # define ASN1_STRING_get0_data	ASN1_STRING_data
 # define X509_getm_notBefore	X509_get_notBefore
 # define X509_getm_notAfter	X509_get_notAfter
-#endif
 
-#ifdef EXIM_NEED_CRYPTO_INIT
 # define CRYPTO_ONCE_STATIC_INIT 0
 # define CRYPTO_THREAD_run_once	 run_once
 typedef int CRYPTO_ONCE;
@@ -154,7 +120,7 @@ static ERR_STRING_DATA dane_str_reasons[] = {
 };
 #endif
 
-#define DANEerr(f, r) ERR_PUT_error(err_lib_dane, (f), (r), __FUNCTION__, __LINE__)
+#define DANEerr(f, r) ERR_PUT_error(err_lib_dane, (f), (r), __FILE__, __LINE__)
 
 static int err_lib_dane = -1;
 static int dane_idx = -1;
@@ -274,6 +240,7 @@ int matched;
  */
 for (matched = 0; !matched && slist; slist = slist->next)
   {
+  dane_mtype_list m;
   unsigned char mdbuf[EVP_MAX_MD_SIZE];
   unsigned char *buf = NULL;
   unsigned char *buf2;
@@ -306,8 +273,9 @@ for (matched = 0; !matched && slist; slist = slist->next)
   /*
    * Loop over each mtype and data element
    */
-  for (dane_mtype_list m = slist->value->mtype; !matched && m; m = m->next)
+  for (m = slist->value->mtype; !matched && m; m = m->next)
     {
+    dane_data_list d;
     unsigned char *cmpbuf = buf;
     unsigned int cmplen = len;
 
@@ -321,7 +289,7 @@ for (matched = 0; !matched && slist; slist = slist->next)
       if (!EVP_Digest(buf, len, cmpbuf, &cmplen, m->value->md, 0))
 	  matched = -1;
       }
-    for (dane_data_list d = m->value->data; !matched && d; d = d->next)
+    for (d = m->value->data; !matched && d; d = d->next)
 	if (  cmplen == d->value->datalen
 	   && memcmp(cmpbuf, d->value->data, cmplen) == 0)
 	    matched = slist->value->selector + 1;
@@ -386,12 +354,13 @@ ASN1_OCTET_STRING *id;
 unsigned char c = 0;
 int ret = 0;
 
-/* 0 will never be our subject keyid from a SHA-1 hash, but it could be
-our subject keyid if forced from child's akid.  If so, set our
-authority keyid to 1.  This way we are never self-signed, and thus
-exempt from any potential (off by default for now in OpenSSL)
-self-signature checks! */
-
+/*
+ * 0 will never be our subject keyid from a SHA-1 hash, but it could be
+ * our subject keyid if forced from child's akid.  If so, set our
+ * authority keyid to 1.  This way we are never self-signed, and thus
+ * exempt from any potential (off by default for now in OpenSSL)
+ * self-signature checks!
+ */
 id =  akid && akid->keyid ? akid->keyid : 0;
 if (id && ASN1_STRING_length(id) == 1 && *ASN1_STRING_get0_data(id) == c)
   c = 1;
@@ -425,9 +394,10 @@ akid_issuer_name(AUTHORITY_KEYID *akid)
 {
 if (akid && akid->issuer)
   {
+  int     i;
   GENERAL_NAMES *gens = akid->issuer;
 
-  for (int i = 0; i < sk_GENERAL_NAME_num(gens); ++i)
+  for (i = 0; i < sk_GENERAL_NAME_num(gens); ++i)
     {
     GENERAL_NAME *gn = sk_GENERAL_NAME_value(gens, i);
 
@@ -444,7 +414,7 @@ set_issuer_name(X509 *cert, AUTHORITY_KEYID *akid, X509_NAME *subj)
 X509_NAME *name = akid_issuer_name(akid);
 
 /*
- * If subject's akid specifies an authority key identifier issuer name, we
+ * If subject's akid specifies an authority key identifer issuer name, we
  * must use that.
  */
 return X509_set_issuer_name(cert,
@@ -575,6 +545,8 @@ return 0;
 static int
 ta_signed(ssl_dane *dane, X509 *cert, int depth)
 {
+dane_cert_list x;
+dane_pkey_list k;
 EVP_PKEY *pk;
 int done = 0;
 
@@ -585,7 +557,7 @@ int done = 0;
  * first (name comparisons), before we bother with signature checks
  * (public key operations).
  */
-for (dane_cert_list x = dane->certs; !done && x; x = x->next)
+for (x = dane->certs; !done && x; x = x->next)
   {
   if (X509_check_issued(x->value, cert) == X509_V_OK)
     {
@@ -625,7 +597,7 @@ for (dane_cert_list x = dane->certs; !done && x; x = x->next)
  * This may push errors onto the stack when the certificate signature is
  * not of the right type or length, throw these away,
  */
-for (dane_pkey_list k = dane->pkeys; !done && k; k = k->next)
+for (k = dane->pkeys; !done && k; k = k->next)
   if (X509_verify(cert, k->value) > 0)
     done = wrap_issuer(dane, k->value, cert, depth, WRAP_MID) ? 1 : -1;
   else
@@ -635,9 +607,11 @@ return done;
 }
 
 static int
-set_trust_anchor(const X509_STORE_CTX *ctx, ssl_dane *dane, X509 *cert)
+set_trust_anchor(X509_STORE_CTX *ctx, ssl_dane *dane, X509 *cert)
 {
 int matched = 0;
+int n;
+int i;
 int depth = 0;
 EVP_PKEY *takey;
 X509 *ca;
@@ -673,9 +647,8 @@ if (!(in = sk_X509_dup(in)))
  *
  * Caller ensures that the initial certificate is not self-signed.
  */
-for (int n = sk_X509_num(in); n > 0; --n, ++depth)
+for (n = sk_X509_num(in); n > 0; --n, ++depth)
   {
-  int i;
   for (i = 0; i < n; ++i)
     if (X509_check_issued(sk_X509_value(in, i), cert) == X509_V_OK)
       break;
@@ -776,8 +749,9 @@ static int
 match_name(const char *certid, ssl_dane *dane)
 {
 int multi = dane->multi;
+dane_host_list hosts;
 
-for (dane_host_list hosts = dane->hosts; hosts; hosts = hosts->next)
+for (hosts = dane->hosts; hosts; hosts = hosts->next)
   {
   int match_subdomain = 0;
   const char *domain = hosts->value;
@@ -893,8 +867,9 @@ gens = X509_get_ext_d2i(cert, NID_subject_alt_name, 0, 0);
 if (gens)
   {
   int n = sk_GENERAL_NAME_num(gens);
+  int i;
 
-  for (int i = 0; i < n; ++i)
+  for (i = 0; i < n; ++i)
     {
     const GENERAL_NAME *gn = sk_GENERAL_NAME_value(gens, i);
     const char *certid;
@@ -1109,52 +1084,52 @@ if (dane->selectors[DANESSL_USAGE_DANE_EE])
     }
   }
 
-if (dane->selectors[DANESSL_USAGE_DANE_TA])
-  {
-  if ((matched = set_trust_anchor(ctx, dane, cert)) < 0)
+  if (dane->selectors[DANESSL_USAGE_DANE_TA])
     {
-    X509_STORE_CTX_set_error(ctx, X509_V_ERR_OUT_OF_MEM);
-    return -1;
+    if ((matched = set_trust_anchor(ctx, dane, cert)) < 0)
+      {
+      X509_STORE_CTX_set_error(ctx, X509_V_ERR_OUT_OF_MEM);
+      return -1;
+      }
+    if (matched)
+      {
+      /*
+       * Check that setting the untrusted chain updates the expected
+       * structure member at the expected offset.
+       */
+      X509_STORE_CTX_trusted_stack(ctx, dane->roots);
+      X509_STORE_CTX_set_chain(ctx, dane->chain);
+      OPENSSL_assert(dane->chain == X509_STORE_CTX_get0_untrusted(ctx));
+      }
     }
-  if (matched)
+
+  /*
+   * Name checks and usage 0/1 constraint enforcement are delayed until
+   * X509_verify_cert() builds the full chain and calls our verify_chain()
+   * wrapper.
+   */
+  dane->verify = X509_STORE_CTX_get_verify(ctx);
+  X509_STORE_CTX_set_verify(ctx, verify_chain);
+
+  if (X509_verify_cert(ctx))
+    return 1;
+
+  /*
+   * If the chain is invalid, clear any matching cert or hostname, to
+   * protect callers that might erroneously rely on these alone without
+   * checking the validation status.
+   */
+  if (dane->match)
     {
-    /*
-     * Check that setting the untrusted chain updates the expected
-     * structure member at the expected offset.
-     */
-    X509_STORE_CTX_trusted_stack(ctx, dane->roots);
-    X509_STORE_CTX_set_chain(ctx, dane->chain);
-    OPENSSL_assert(dane->chain == X509_STORE_CTX_get0_untrusted(ctx));
+    X509_free(dane->match);
+    dane->match = 0;
     }
-  }
-
-/*
- * Name checks and usage 0/1 constraint enforcement are delayed until
- * X509_verify_cert() builds the full chain and calls our verify_chain()
- * wrapper.
- */
-dane->verify = X509_STORE_CTX_get_verify(ctx);
-X509_STORE_CTX_set_verify(ctx, verify_chain);
-
-if (X509_verify_cert(ctx))
-  return 1;
-
-/*
- * If the chain is invalid, clear any matching cert or hostname, to
- * protect callers that might erroneously rely on these alone without
- * checking the validation status.
- */
-if (dane->match)
-  {
-  X509_free(dane->match);
-  dane->match = 0;
-  }
-if (dane->mhost)
-  {
-  OPENSSL_free(dane->mhost);
-  dane->mhost = 0;
-  }
- return 0;
+  if (dane->mhost)
+    {
+    OPENSSL_free(dane->mhost);
+    dane->mhost = 0;
+    }
+   return 0;
 }
 
 static dane_list
@@ -1182,9 +1157,10 @@ return l;
 static void
 list_free(void *list, void (*f)(void *))
 {
+dane_list head;
 dane_list next;
 
-for (dane_list head = (dane_list) list; head; head = next)
+for (head = (dane_list) list; head; head = next)
   {
   next = head->next;
   if (f && head->value)
@@ -1233,6 +1209,7 @@ void
 DANESSL_cleanup(SSL *ssl)
 {
 ssl_dane *dane;
+int u;
 
 DEBUG(D_tls) debug_printf("Dane lib-cleanup\n");
 
@@ -1243,7 +1220,7 @@ if (dane_idx < 0 || !(dane = SSL_get_ex_data(ssl, dane_idx)))
 dane_reset(dane);
 if (dane->hosts)
   list_free(dane->hosts, ossl_free);
-for (int u = 0; u <= DANESSL_USAGE_LAST; ++u)
+for (u = 0; u <= DANESSL_USAGE_LAST; ++u)
   if (dane->selectors[u])
     list_free(dane->selectors[u], dane_selector_free);
 if (dane->pkeys)
@@ -1373,7 +1350,7 @@ dane_cert_list xlist = 0;
 dane_pkey_list klist = 0;
 const EVP_MD *md = 0;
 
-DEBUG(D_tls) debug_printf("Dane add-tlsa: usage %u sel %u mdname %q\n",
+DEBUG(D_tls) debug_printf("Dane add-tlsa: usage %u sel %u mdname \"%s\"\n",
 			  usage, selector, mdname);
 
 if(dane_idx < 0 || !(dane = SSL_get_ex_data(ssl, dane_idx)))
@@ -1559,6 +1536,7 @@ int
 DANESSL_init(SSL *ssl, const char *sni_domain, const char **hostnames)
 {
 ssl_dane *dane;
+int i;
 
 DEBUG(D_tls) debug_printf("Dane ssl_init\n");
 if (dane_idx < 0)
@@ -1597,7 +1575,7 @@ dane->multi = 0;			/* Future SSL control interface */
 dane->count = 0;
 dane->hosts = 0;
 
-for (int i = 0; i <= DANESSL_USAGE_LAST; ++i)
+for (i = 0; i <= DANESSL_USAGE_LAST; ++i)
   dane->selectors[i] = 0;
 
 if (hostnames && (dane->hosts = host_list_init(hostnames)) == 0)
@@ -1645,8 +1623,10 @@ return -1;
 static void
 dane_init(void)
 {
-/* Store library id in zeroth function slot, used to locate the library
-name.  This must be done before we load the error strings.  */
+/*
+ * Store library id in zeroth function slot, used to locate the library
+ * name.  This must be done before we load the error strings.
+ */
 err_lib_dane = ERR_get_next_error_library();
 
 #ifndef OPENSSL_NO_ERR
@@ -1658,23 +1638,24 @@ if (err_lib_dane > 0)
   }
 #endif
 
-#ifndef EXIM_NO_NEED_SHA2_REGISTER
-/* Register SHA-2 digests, if implemented and not already registered.  */
-
-# if defined(LN_sha256) && defined(NID_sha256) && !defined(OPENSSL_NO_SHA256)
+/*
+ * Register SHA-2 digests, if implemented and not already registered.
+ */
+#if defined(LN_sha256) && defined(NID_sha256) && !defined(OPENSSL_NO_SHA256)
 if (!EVP_get_digestbyname(LN_sha224)) EVP_add_digest(EVP_sha224());
 if (!EVP_get_digestbyname(LN_sha256)) EVP_add_digest(EVP_sha256());
-# endif
-# if defined(LN_sha512) && defined(NID_sha512) && !defined(OPENSSL_NO_SHA512)
+#endif
+#if defined(LN_sha512) && defined(NID_sha512) && !defined(OPENSSL_NO_SHA512)
 if (!EVP_get_digestbyname(LN_sha384)) EVP_add_digest(EVP_sha384());
 if (!EVP_get_digestbyname(LN_sha512)) EVP_add_digest(EVP_sha512());
-# endif
 #endif
 
-/* Register an SSL index for the connection-specific ssl_dane structure.
-Using a separate index makes it possible to add DANE support to existing OpenSSL
-releases that don't have a suitable pointer in the SSL structure. */
-
+/*
+ * Register an SSL index for the connection-specific ssl_dane structure.
+ * Using a separate index makes it possible to add DANE support to
+ * existing OpenSSL releases that don't have a suitable pointer in the
+ * SSL structure.
+ */
 dane_idx = SSL_get_ex_new_index(0, 0, 0, 0, 0);
 }
 
