@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2017 */
+/* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* All the global variables are defined together in this one module, so
@@ -101,38 +101,38 @@ cluttered in several places (e.g. during logging) if we can always refer to
 them. Also, the tls_ variables are now always visible. */
 
 tls_support tls_in = {
- -1,   /* tls_active */
- 0,    /* tls_bits */
- FALSE,/* tls_certificate_verified */
-#ifdef EXPERIMENTAL_DANE
- FALSE,/* dane_verified */
- 0,    /* tlsa_usage */
+ .active =		-1,
+ .bits =		0,
+ .certificate_verified = FALSE,
+#ifdef SUPPORT_DANE
+ .dane_verified =	FALSE,
+ .tlsa_usage =		0,
 #endif
- NULL, /* tls_cipher */
- FALSE,/* tls_on_connect */
- NULL, /* tls_on_connect_ports */
- NULL, /* tls_ourcert */
- NULL, /* tls_peercert */
- NULL, /* tls_peerdn */
- NULL, /* tls_sni */
- 0     /* tls_ocsp */
+ .cipher =		NULL,
+ .on_connect =		FALSE,
+ .on_connect_ports =	NULL,
+ .ourcert =		NULL,
+ .peercert =		NULL,
+ .peerdn =		NULL,
+ .sni =			NULL,
+ .ocsp =		OCSP_NOT_REQ
 };
 tls_support tls_out = {
- -1,   /* tls_active */
- 0,    /* tls_bits */
- FALSE,/* tls_certificate_verified */
-#ifdef EXPERIMENTAL_DANE
- FALSE,/* dane_verified */
- 0,    /* tlsa_usage */
+ .active =		-1,
+ .bits =		0,
+ .certificate_verified = FALSE,
+#ifdef SUPPORT_DANE
+ .dane_verified =	FALSE,
+ .tlsa_usage =		0,
 #endif
- NULL, /* tls_cipher */
- FALSE,/* tls_on_connect */
- NULL, /* tls_on_connect_ports */
- NULL, /* tls_ourcert */
- NULL, /* tls_peercert */
- NULL, /* tls_peerdn */
- NULL, /* tls_sni */
- 0     /* tls_ocsp */
+ .cipher =		NULL,
+ .on_connect =		FALSE,
+ .on_connect_ports =	NULL,
+ .ourcert =		NULL,
+ .peercert =		NULL,
+ .peerdn =		NULL,
+ .sni =			NULL,
+ .ocsp =		OCSP_NOT_REQ
 };
 
 uschar *dsn_envid              = NULL;
@@ -182,10 +182,12 @@ const pcre *regex_UTF8         = NULL;
 incoming TCP/IP. The defaults use stdin. We never need these for any
 stand-alone tests. */
 
-#ifndef STAND_ALONE
+#if !defined(STAND_ALONE) && !defined(MACRO_PREDEF)
 int (*lwr_receive_getc)(unsigned) = stdin_getc;
+uschar * (*lwr_receive_getbuf)(unsigned *) = NULL;
 int (*lwr_receive_ungetc)(int) = stdin_ungetc;
 int (*receive_getc)(unsigned)  = stdin_getc;
+uschar * (*receive_getbuf)(unsigned *)  = NULL;
 void (*receive_get_cache)(void)= NULL;
 int (*receive_ungetc)(int)     = stdin_ungetc;
 int (*receive_feof)(void)      = stdin_feof;
@@ -326,78 +328,82 @@ uschar *add_environment        = NULL;
 address_item  *addr_duplicate  = NULL;
 
 address_item address_defaults = {
-  NULL,                 /* next */
-  NULL,                 /* parent */
-  NULL,                 /* first */
-  NULL,                 /* dupof */
-  NULL,                 /* start_router */
-  NULL,                 /* router */
-  NULL,                 /* transport */
-  NULL,                 /* host_list */
-  NULL,                 /* host_used */
-  NULL,                 /* fallback_hosts */
-  NULL,                 /* reply */
-  NULL,                 /* retries */
-  NULL,                 /* address */
-  NULL,                 /* unique */
-  NULL,                 /* cc_local_part */
-  NULL,                 /* lc_local_part */
-  NULL,                 /* local_part */
-  NULL,                 /* prefix */
-  NULL,                 /* suffix */
-  NULL,                 /* domain */
-  NULL,                 /* address_retry_key */
-  NULL,                 /* domain_retry_key */
-  NULL,                 /* current_dir */
-  NULL,                 /* home_dir */
-  NULL,                 /* message */
-  NULL,                 /* user_message */
-  NULL,                 /* onetime_parent */
-  NULL,                 /* pipe_expandn */
-  NULL,                 /* return_filename */
-  NULL,                 /* self_hostname */
-  NULL,                 /* shadow_message */
+  .next =		NULL,
+  .parent =		NULL,
+  .first =		NULL,
+  .dupof =		NULL,
+  .start_router =	NULL,
+  .router =		NULL,
+  .transport =		NULL,
+  .host_list =		NULL,
+  .host_used =		NULL,
+  .fallback_hosts =	NULL,
+  .reply =		NULL,
+  .retries =		NULL,
+  .address =		NULL,
+  .unique =		NULL,
+  .cc_local_part =	NULL,
+  .lc_local_part =	NULL,
+  .local_part =		NULL,
+  .prefix =		NULL,
+  .suffix =		NULL,
+  .domain =		NULL,
+  .address_retry_key =	NULL,
+  .domain_retry_key =	NULL,
+  .current_dir =	NULL,
+  .home_dir =		NULL,
+  .message =		NULL,
+  .user_message =	NULL,
+  .onetime_parent =	NULL,
+  .pipe_expandn =	NULL,
+  .return_filename =	NULL,
+  .self_hostname =	NULL,
+  .shadow_message =	NULL,
 #ifdef SUPPORT_TLS
-  NULL,                 /* cipher */
-  NULL,			/* ourcert */
-  NULL,			/* peercert */
-  NULL,                 /* peerdn */
-  OCSP_NOT_REQ,         /* ocsp */
+  .cipher =		NULL,
+  .ourcert =		NULL,
+  .peercert =		NULL,
+  .peerdn =		NULL,
+  .ocsp =		OCSP_NOT_REQ,
 #endif
 #ifdef EXPERIMENTAL_DSN_INFO
-  NULL,			/* smtp_greeting */
-  NULL,			/* helo_response */
+  .smtp_greeting =	NULL,
+  .helo_response =	NULL,
 #endif
-  NULL,			/* authenticator */
-  NULL,			/* auth_id */
-  NULL,			/* auth_sndr */
-  NULL,                 /* dsn_orcpt */
-  0,                    /* dsn_flags */
-  0,                    /* dsn_aware */
-  (uid_t)(-1),          /* uid */
-  (gid_t)(-1),          /* gid */
-  0,                    /* flags */
-  { 0 },                /* domain_cache - any larger array should be zeroed */
-  { 0 },                /* localpart_cache - ditto */
-  -1,                   /* mode */
-  0,                    /* more_errno */
-  ERRNO_UNKNOWNERROR,   /* basic_errno */
-  0,                    /* child_count */
-  -1,                   /* return_file */
-  SPECIAL_NONE,         /* special_action */
-  DEFER,                /* transport_return */
-  {                     /* fields that are propagated to children */
-    NULL,               /* address_data */
-    NULL,               /* domain_data */
-    NULL,               /* localpart_data */
-    NULL,               /* errors_address */
-    NULL,               /* extra_headers */
-    NULL,               /* remove_headers */
+  .authenticator =	NULL,
+  .auth_id =		NULL,
+  .auth_sndr =		NULL,
+  .dsn_orcpt =		NULL,
+  .dsn_flags =		0,
+  .dsn_aware =		0,
+  .uid =		(uid_t)(-1),
+  .gid =		(gid_t)(-1),
+  .flags =		{ 0 },
+  .domain_cache =	{ 0 },                /* domain_cache - any larger array should be zeroed */
+  .localpart_cache =	{ 0 },                /* localpart_cache - ditto */
+  .mode =		-1,
+  .more_errno =		0,
+  .delivery_usec =	0,
+  .basic_errno =	ERRNO_UNKNOWNERROR,
+  .child_count =	0,
+  .return_file =	-1,
+  .special_action =	SPECIAL_NONE,
+  .transport_return =	DEFER,
+  .prop = {					/* fields that are propagated to children */
+    .address_data =	NULL,
+    .domain_data =	NULL,
+    .localpart_data =	NULL,
+    .errors_address =	NULL,
+    .extra_headers =	NULL,
+    .remove_headers =	NULL,
 #ifdef EXPERIMENTAL_SRS
-    NULL,               /* srs_sender */
+    .srs_sender =	NULL,
 #endif
+    .ignore_error =	FALSE,
 #ifdef SUPPORT_I18N
-    FALSE,		/* utf8 */
+    .utf8_msg =		FALSE,
+    .utf8_downcvt =	FALSE,
+    .utf8_downcvt_maybe = FALSE
 #endif
   }
 };
@@ -414,30 +420,40 @@ BOOL    allow_domain_literals  = FALSE;
 BOOL    allow_mx_to_ip         = FALSE;
 BOOL    allow_unqualified_recipient = TRUE;    /* For local messages */
 BOOL    allow_unqualified_sender = TRUE;       /* Reset for SMTP */
-BOOL    allow_utf8_domains     = FALSE;
+BOOL    allow_utf8_domains	= FALSE;
+
+#ifdef EXPERIMENTAL_ARC
+struct arc_set *arc_received	= NULL;
+int     arc_received_instance	= 0;
+int     arc_oldest_pass		= 0;
+const uschar *arc_state		= NULL;
+const uschar *arc_state_reason	= NULL;
+#endif
+
 uschar *authenticated_fail_id  = NULL;
 uschar *authenticated_id       = NULL;
 uschar *authenticated_sender   = NULL;
 BOOL    authentication_failed  = FALSE;
+BOOL    authentication_local   = FALSE;
 auth_instance  *auths          = NULL;
 uschar *auth_advertise_hosts   = US"*";
 auth_instance auth_defaults    = {
-    NULL,                      /* chain pointer */
-    NULL,                      /* name */
-    NULL,                      /* info */
-    NULL,                      /* private options block pointer */
-    NULL,                      /* driver_name */
-    NULL,                      /* advertise_condition */
-    NULL,                      /* client_condition */
-    NULL,                      /* public_name */
-    NULL,                      /* set_id */
-    NULL,                      /* set_client_id */
-    NULL,                      /* server_mail_auth_condition */
-    NULL,                      /* server_debug_string */
-    NULL,                      /* server_condition */
-    FALSE,                     /* client */
-    FALSE,                     /* server */
-    FALSE                      /* advertised */
+    .next =		NULL,
+    .name =		NULL,
+    .info =		NULL,
+    .options_block =	NULL,
+    .driver_name =	NULL,
+    .advertise_condition = NULL,
+    .client_condition =	NULL,
+    .public_name =	NULL,
+    .set_id =		NULL,
+    .set_client_id =	NULL,
+    .mail_auth_condition = NULL,
+    .server_debug_string = NULL,
+    .server_condition =	NULL,
+    .client =		FALSE,
+    .server =		FALSE,
+    .advertised =	FALSE
 };
 
 uschar *auth_defer_msg         = US"reason not recorded";
@@ -470,6 +486,7 @@ int     bmi_deliver            = 1;
 int     bmi_run                = 0;
 uschar *bmi_verdicts           = NULL;
 #endif
+int     bsmtp_transaction_linecount = 0;
 int     body_8bitmime          = 0;
 int     body_linecount         = 0;
 int     body_zerocount         = 0;
@@ -481,7 +498,6 @@ int     bounce_return_linesize_limit = 998;
 BOOL    bounce_return_message  = TRUE;
 int     bounce_return_size_limit = 100*1024;
 uschar *bounce_sender_authentication = NULL;
-int     bsmtp_transaction_linecount = 0;
 
 uschar *callout_address        = NULL;
 int     callout_cache_domain_positive_expire = 7*24*60*60;
@@ -508,6 +524,7 @@ uschar *client_authenticated_id = NULL;
 uschar *client_authenticated_sender = NULL;
 int     clmacro_count          = 0;
 uschar *clmacros[MAX_CLMACROS];
+BOOL    commandline_checks_require_admin = FALSE;
 BOOL    config_changed         = FALSE;
 FILE   *config_file            = NULL;
 const uschar *config_filename  = NULL;
@@ -529,6 +546,7 @@ uid_t   config_uid             = 0;
 #endif
 
 int     connection_max_messages= -1;
+uschar *continue_proxy_cipher  = NULL;
 uschar *continue_hostname      = NULL;
 uschar *continue_host_address  = NULL;
 BOOL    continue_more          = FALSE;
@@ -537,10 +555,12 @@ uschar *continue_transport     = NULL;
 
 uschar *csa_status             = NULL;
 cut_t   cutthrough = {
-  FALSE,				/* delivery: when to attempt */
-  FALSE,				/* on defer: spool locally */
-  -1,					/* fd: open connection */
-  0,					/* nrcpt: number of addresses */
+  .callout_hold_only =	FALSE,				/* verify-only: normal delivery */
+  .delivery =		FALSE,				/* when to attempt */
+  .defer_pass =		FALSE,				/* on defer: spool locally */
+  .is_tls =		FALSE,				/* not a TLS conn yet */
+  .fd =			-1,				/* open connection */
+  .nrcpt =		0,				/* number of addresses */
 };
 
 BOOL    daemon_listen          = FALSE;
@@ -656,11 +676,13 @@ int     dkim_key_length          = 0;
 uschar *dkim_signers             = NULL;
 uschar *dkim_signing_domain      = NULL;
 uschar *dkim_signing_selector    = NULL;
+uschar *dkim_verify_overall      = NULL;
 uschar *dkim_verify_signers      = US"$dkim_signers";
+uschar *dkim_verify_status	 = NULL;
+uschar *dkim_verify_reason	 = NULL;
 #endif
 #ifdef EXPERIMENTAL_DMARC
 BOOL    dmarc_has_been_checked  = FALSE;
-uschar *dmarc_ar_header         = NULL;
 uschar *dmarc_domain_policy     = NULL;
 uschar *dmarc_forensic_sender   = NULL;
 uschar *dmarc_history_file      = NULL;
@@ -675,7 +697,7 @@ BOOL    dmarc_enable_forensic   = FALSE;
 uschar *dns_again_means_nonexist = NULL;
 int     dns_csa_search_limit   = 5;
 BOOL    dns_csa_use_reverse    = TRUE;
-#ifdef EXPERIMENTAL_DANE
+#ifdef SUPPORT_DANE
 int     dns_dane_ok            = -1;
 #endif
 uschar *dns_ipv4_lookup        = NULL;
@@ -756,22 +778,23 @@ int     header_maxsize         = HEADER_MAXSIZE;
 int     header_line_maxsize    = 0;
 
 header_name header_names[] = {
-  { US"bcc",            3, TRUE,  htype_bcc },
-  { US"cc",             2, TRUE,  htype_cc },
-  { US"date",           4, TRUE,  htype_date },
-  { US"delivery-date", 13, FALSE, htype_delivery_date },
-  { US"envelope-to",   11, FALSE, htype_envelope_to },
-  { US"from",           4, TRUE,  htype_from },
-  { US"message-id",    10, TRUE,  htype_id },
-  { US"received",       8, FALSE, htype_received },
-  { US"reply-to",       8, FALSE, htype_reply_to },
-  { US"return-path",   11, FALSE, htype_return_path },
-  { US"sender",         6, TRUE,  htype_sender },
-  { US"subject",        7, FALSE, htype_subject },
-  { US"to",             2, TRUE,  htype_to }
+  /* name		len	allow_resent	htype */
+  { US"bcc",            3,	TRUE,		htype_bcc },
+  { US"cc",             2,	TRUE,		htype_cc },
+  { US"date",           4,	TRUE,		htype_date },
+  { US"delivery-date", 13,	FALSE,		htype_delivery_date },
+  { US"envelope-to",   11,	FALSE,		htype_envelope_to },
+  { US"from",           4,	TRUE,		htype_from },
+  { US"message-id",    10,	TRUE,		htype_id },
+  { US"received",       8,	FALSE,		htype_received },
+  { US"reply-to",       8,	FALSE,		htype_reply_to },
+  { US"return-path",   11,	FALSE,		htype_return_path },
+  { US"sender",         6,	TRUE,		htype_sender },
+  { US"subject",        7,	FALSE,		htype_subject },
+  { US"to",             2,	TRUE,		htype_to }
 };
 
-int header_names_size          = sizeof(header_names)/sizeof(header_name);
+int header_names_size          = nelem(header_names);
 
 BOOL    header_rewritten       = FALSE;
 uschar *helo_accept_junk_hosts = NULL;
@@ -844,6 +867,7 @@ int     log_default[]          = { /* for initializing log_selector */
   Li_acl_warn_skipped,
   Li_connection_reject,
   Li_delay_delivery,
+  Li_dkim,
   Li_dnslist_defer,
   Li_etrn,
   Li_host_lookup_failed,
@@ -878,6 +902,10 @@ bit_table log_options[]        = { /* must be in alphabetical order */
   BIT_TABLE(L, delay_delivery),
   BIT_TABLE(L, deliver_time),
   BIT_TABLE(L, delivery_size),
+#ifndef DISABLE_DKIM
+  BIT_TABLE(L, dkim),
+  BIT_TABLE(L, dkim_verbose),
+#endif
   BIT_TABLE(L, dnslist_defer),
   BIT_TABLE(L, dnssec),
   BIT_TABLE(L, etrn),
@@ -886,15 +914,17 @@ bit_table log_options[]        = { /* must be in alphabetical order */
   BIT_TABLE(L, incoming_interface),
   BIT_TABLE(L, incoming_port),
   BIT_TABLE(L, lost_incoming_connection),
+  BIT_TABLE(L, millisec),
   BIT_TABLE(L, outgoing_interface),
   BIT_TABLE(L, outgoing_port),
   BIT_TABLE(L, pid),
-#if defined(SUPPORT_PROXY) || defined (SUPPORT_SOCKS)
+#if defined(SUPPORT_PROXY) || defined(SUPPORT_SOCKS)
   BIT_TABLE(L, proxy),
 #endif
   BIT_TABLE(L, queue_run),
   BIT_TABLE(L, queue_time),
   BIT_TABLE(L, queue_time_overall),
+  BIT_TABLE(L, receive_time),
   BIT_TABLE(L, received_recipients),
   BIT_TABLE(L, received_sender),
   BIT_TABLE(L, rejected_header),
@@ -932,9 +962,7 @@ uschar *lookup_dnssec_authenticated = NULL;
 int     lookup_open_max        = 25;
 uschar *lookup_value           = NULL;
 
-macro_item  *macros            = NULL;
-macro_item  *mlast             = NULL;
-BOOL    macros_builtin_created = FALSE;
+macro_item *macros_user        = NULL;
 uschar *mailstore_basename     = NULL;
 #ifdef WITH_CONTENT_SCAN
 uschar *malware_name           = NULL;  /* Virus Name */
@@ -1102,7 +1130,8 @@ uschar *received_header_text   = US
 
 int     received_headers_max   = 30;
 uschar *received_protocol      = NULL;
-int     received_time          = 0;
+struct timeval received_time   = { 0, 0 };
+struct timeval received_time_taken = { 0, 0 };
 uschar *recipient_data         = NULL;
 uschar *recipient_unqualified_hosts = NULL;
 uschar *recipient_verify_failure = NULL;
@@ -1145,83 +1174,83 @@ uid_t   root_uid               = ROOT_UID;
 
 router_instance  *routers  = NULL;
 router_instance  router_defaults = {
-    NULL,                      /* chain pointer */
-    NULL,                      /* name */
-    NULL,                      /* info */
-    NULL,                      /* private options block pointer */
-    NULL,                      /* driver name */
+    .next =			NULL,
+    .name =			NULL,
+    .info =			NULL,
+    .options_block =		NULL,
+    .driver_name =		NULL,
 
-    NULL,                      /* address_data */
+    .address_data =		NULL,
 #ifdef EXPERIMENTAL_BRIGHTMAIL
-    NULL,                      /* bmi_rule */
+    .bmi_rule =			NULL,
 #endif
-    NULL,                      /* cannot_route_message */
-    NULL,                      /* condition */
-    NULL,                      /* current_directory */
-    NULL,                      /* debug_string */
-    NULL,                      /* domains */
-    NULL,                      /* errors_to */
-    NULL,                      /* expand_gid */
-    NULL,                      /* expand_uid */
-    NULL,                      /* expand_more */
-    NULL,                      /* expand_unseen */
-    NULL,                      /* extra_headers */
-    NULL,                      /* fallback_hosts */
-    NULL,                      /* home_directory */
-    NULL,                      /* ignore_target_hosts */
-    NULL,                      /* local_parts */
-    NULL,                      /* pass_router_name */
-    NULL,                      /* prefix */
-    NULL,                      /* redirect_router_name */
-    NULL,                      /* remove_headers */
-    NULL,                      /* require_files */
-    NULL,                      /* router_home_directory */
-    US"freeze",                /* self */
-    NULL,                      /* senders */
-    NULL,                      /* suffix */
-    NULL,                      /* translate_ip_address */
-    NULL,                      /* transport_name */
+    .cannot_route_message =	NULL,
+    .condition =		NULL,
+    .current_directory =	NULL,
+    .debug_string =		NULL,
+    .domains =			NULL,
+    .errors_to =		NULL,
+    .expand_gid =		NULL,
+    .expand_uid =		NULL,
+    .expand_more =		NULL,
+    .expand_unseen =		NULL,
+    .extra_headers =		NULL,
+    .fallback_hosts =		NULL,
+    .home_directory =		NULL,
+    .ignore_target_hosts =	NULL,
+    .local_parts =		NULL,
+    .pass_router_name =		NULL,
+    .prefix =			NULL,
+    .redirect_router_name =	NULL,
+    .remove_headers =		NULL,
+    .require_files =		NULL,
+    .router_home_directory =	NULL,
+    .self =			US"freeze",
+    .senders =			NULL,
+    .suffix =			NULL,
+    .translate_ip_address =	NULL,
+    .transport_name =		NULL,
 
-    TRUE,                      /* address_test */
+    .address_test =		TRUE,
 #ifdef EXPERIMENTAL_BRIGHTMAIL
-    FALSE,                     /* bmi_deliver_alternate */
-    FALSE,                     /* bmi_deliver_default */
-    FALSE,                     /* bmi_dont_deliver */
+    .bmi_deliver_alternate =	FALSE,
+    .bmi_deliver_default =	FALSE,
+    .bmi_dont_deliver =		FALSE,
 #endif
-    TRUE,                      /* expn */
-    FALSE,                     /* caseful_local_part */
-    FALSE,                     /* check_local_user */
-    FALSE,                     /* disable_logging */
-    FALSE,                     /* fail_verify_recipient */
-    FALSE,                     /* fail_verify_sender */
-    FALSE,                     /* gid_set */
-    FALSE,                     /* initgroups */
-    TRUE_UNSET,                /* log_as_local */
-    TRUE,                      /* more */
-    FALSE,                     /* pass_on_timeout */
-    FALSE,                     /* prefix_optional */
-    TRUE,                      /* repeat_use */
-    TRUE_UNSET,                /* retry_use_local_part - fudge "unset" */
-    FALSE,                     /* same_domain_copy_routing */
-    FALSE,                     /* self_rewrite */
-    FALSE,                     /* suffix_optional */
-    FALSE,                     /* verify_only */
-    TRUE,                      /* verify_recipient */
-    TRUE,                      /* verify_sender */
-    FALSE,                     /* uid_set */
-    FALSE,                     /* unseen */
-    FALSE,                     /* dsn_lasthop */
+    .expn =			TRUE,
+    .caseful_local_part =	FALSE,
+    .check_local_user =		FALSE,
+    .disable_logging =		FALSE,
+    .fail_verify_recipient =	FALSE,
+    .fail_verify_sender =	FALSE,
+    .gid_set =			FALSE,
+    .initgroups =		FALSE,
+    .log_as_local =		TRUE_UNSET,
+    .more =			TRUE,
+    .pass_on_timeout =		FALSE,
+    .prefix_optional =		FALSE,
+    .repeat_use =		TRUE,
+    .retry_use_local_part =	TRUE_UNSET,
+    .same_domain_copy_routing =	FALSE,
+    .self_rewrite =		FALSE,
+    .suffix_optional =		FALSE,
+    .verify_only =		FALSE,
+    .verify_recipient =		TRUE,
+    .verify_sender =		TRUE,
+    .uid_set =			FALSE,
+    .unseen =			FALSE,
+    .dsn_lasthop =		FALSE,
 
-    self_freeze,               /* self_code */
-    (uid_t)(-1),               /* uid */
-    (gid_t)(-1),               /* gid */
+    .self_code =		self_freeze,
+    .uid =			(uid_t)(-1),
+    .gid =			(gid_t)(-1),
 
-    NULL,                      /* fallback_hostlist */
-    NULL,                      /* transport instance */
-    NULL,                      /* pass_router */
-    NULL,                      /* redirect_router */
+    .fallback_hostlist =	NULL,
+    .transport =		NULL,
+    .pass_router =		NULL,
+    .redirect_router =		NULL,
 
-    { NULL, NULL },            /* dnssec_domains {require,request} */
+    .dnssec =			{ NULL, NULL },            /* dnssec_domains {require,request} */
 };
 
 uschar *router_name            = NULL;
@@ -1257,6 +1286,7 @@ uschar *sender_helo_name       = NULL;
 uschar **sender_host_aliases   = &no_aliases;
 uschar *sender_host_address    = NULL;
 uschar *sender_host_authenticated = NULL;
+uschar *sender_host_auth_pubname  = NULL;
 unsigned int sender_host_cache[(MAX_NAMED_LIST * 2)/32];
 BOOL    sender_host_dnssec     = FALSE;
 uschar *sender_host_name       = NULL;
@@ -1302,7 +1332,7 @@ BOOL    smtp_check_spool_space = TRUE;
 int     smtp_ch_index          = 0;
 uschar *smtp_cmd_argument      = NULL;
 uschar *smtp_cmd_buffer        = NULL;
-time_t  smtp_connection_start  = 0;
+struct timeval smtp_connection_start  = {0,0};
 uschar  smtp_connection_had[SMTP_HBUFF_SIZE];
 int     smtp_connect_backlog   = 20;
 double  smtp_delay_mail        = 0.0;
@@ -1348,17 +1378,20 @@ uschar *spam_action            = NULL;
 uschar *spam_score             = NULL;
 uschar *spam_score_int         = NULL;
 #endif
-#ifdef EXPERIMENTAL_SPF
+#ifdef SUPPORT_SPF
 uschar *spf_guess              = US"v=spf1 a/24 mx/24 ptr ?all";
 uschar *spf_header_comment     = NULL;
 uschar *spf_received           = NULL;
 uschar *spf_result             = NULL;
+BOOL    spf_result_guessed     = FALSE;
 uschar *spf_smtp_comment       = NULL;
 #endif
 
 BOOL    split_spool_directory  = FALSE;
 uschar *spool_directory        = US SPOOL_DIRECTORY
                            "\0<--------------Space to patch spool_directory->";
+BOOL    spool_file_wireformat  = FALSE;
+BOOL    spool_wireformat       = FALSE;
 #ifdef EXPERIMENTAL_SRS
 uschar *srs_config             = NULL;
 uschar *srs_db_address         = NULL;
@@ -1405,7 +1438,12 @@ BOOL    system_filter_uid_set  = FALSE;
 BOOL    system_filtering       = FALSE;
 
 BOOL    tcp_fastopen_ok        = FALSE;
+blob	tcp_fastopen_nodata    = { .data = NULL, .len = 0 };
+BOOL    tcp_in_fastopen        = FALSE;
+BOOL    tcp_in_fastopen_logged = FALSE;
 BOOL    tcp_nodelay            = TRUE;
+int     tcp_out_fastopen       = 0;
+BOOL    tcp_out_fastopen_logged= FALSE;
 #ifdef USE_TCP_WRAPPERS
 uschar *tcp_wrappers_daemon_name = US TCP_WRAPPERS_DAEMON_NAME;
 #endif
@@ -1417,59 +1455,59 @@ BOOL    timestamps_utc         = FALSE;
 transport_instance  *transports = NULL;
 
 transport_instance  transport_defaults = {
-    NULL,                     /* chain pointer */
-    NULL,                     /* name */
-    NULL,                     /* info */
-    NULL,                     /* private options block pointer */
-    NULL,                     /* driver name */
-    NULL,                     /* setup entry point */
-    1,                        /* batch_max */
-    NULL,                     /* batch_id */
-    NULL,                     /* home_dir */
-    NULL,                     /* current_dir */
-    NULL,                     /* expand-multi-domain */
-    TRUE,                     /* multi-domain */
-    FALSE,                    /* overrides_hosts */
-    100,                      /* max_addresses */
-    500,                      /* connection_max_messages */
-    FALSE,                    /* deliver_as_creator */
-    FALSE,                    /* disable_logging */
-    FALSE,                    /* initgroups */
-    FALSE,                    /* uid_set */
-    FALSE,                    /* gid_set */
-    (uid_t)(-1),              /* uid */
-    (gid_t)(-1),              /* gid */
-    NULL,                     /* expand_uid */
-    NULL,                     /* expand_gid */
-    NULL,                     /* warn_message */
-    NULL,                     /* shadow */
-    NULL,                     /* shadow_condition */
-    NULL,                     /* filter_command */
-    NULL,                     /* add_headers */
-    NULL,                     /* remove_headers */
-    NULL,                     /* return_path */
-    NULL,                     /* debug_string */
-    NULL,                     /* max_parallel */
-    NULL,                     /* message_size_limit */
-    NULL,                     /* headers_rewrite */
-    NULL,                     /* rewrite_rules */
-    0,                        /* rewrite_existflags */
-    300,                      /* filter_timeout */
-    FALSE,                    /* body_only */
-    FALSE,                    /* delivery_date_add */
-    FALSE,                    /* envelope_to_add */
-    FALSE,                    /* headers_only */
-    FALSE,                    /* rcpt_include_affixes */
-    FALSE,                    /* return_path_add */
-    FALSE,                    /* return_output */
-    FALSE,                    /* return_fail_output */
-    FALSE,                    /* log_output */
-    FALSE,                    /* log_fail_output */
-    FALSE,                    /* log_defer_output */
-    TRUE_UNSET                /* retry_use_local_part: BOOL, but set neither
-                                 1 nor 0 so can detect unset */
+    .next =			NULL,
+    .name =			NULL,
+    .info =			NULL,
+    .options_block =		NULL,
+    .driver_name =		NULL,
+    .setup =			NULL,
+    .batch_max =		1,
+    .batch_id =			NULL,
+    .home_dir =			NULL,
+    .current_dir =		NULL,
+    .expand_multi_domain =	NULL,
+    .multi_domain =		TRUE,
+    .overrides_hosts =		FALSE,
+    .max_addresses =		100,
+    .connection_max_messages =	500,
+    .deliver_as_creator =	FALSE,
+    .disable_logging =		FALSE,
+    .initgroups =		FALSE,
+    .uid_set =			FALSE,
+    .gid_set =			FALSE,
+    .uid =			(uid_t)(-1),
+    .gid =			(gid_t)(-1),
+    .expand_uid =		NULL,
+    .expand_gid =		NULL,
+    .warn_message =		NULL,
+    .shadow =			NULL,
+    .shadow_condition =		NULL,
+    .filter_command =		NULL,
+    .add_headers =		NULL,
+    .remove_headers =		NULL,
+    .return_path =		NULL,
+    .debug_string =		NULL,
+    .max_parallel =		NULL,
+    .message_size_limit =	NULL,
+    .headers_rewrite =		NULL,
+    .rewrite_rules =		NULL,
+    .rewrite_existflags =	0,
+    .filter_timeout =		300,
+    .body_only =		FALSE,
+    .delivery_date_add =	FALSE,
+    .envelope_to_add =		FALSE,
+    .headers_only =		FALSE,
+    .rcpt_include_affixes =	FALSE,
+    .return_path_add =		FALSE,
+    .return_output =		FALSE,
+    .return_fail_output =	FALSE,
+    .log_output =		FALSE,
+    .log_fail_output =		FALSE,
+    .log_defer_output =		FALSE,
+    .retry_use_local_part =	TRUE_UNSET,	/* retry_use_local_part: BOOL, but set neither
+						 1 nor 0 so can detect unset */
 #ifndef DISABLE_EVENT
-   ,NULL		      /* event_action */
+   .event_action =		NULL
 #endif
 };
 
@@ -1523,8 +1561,8 @@ uschar *uucp_from_sender       = US"$1";
 
 uschar *verify_mode	       = NULL;
 uschar *version_copyright      =
- US"Copyright (c) University of Cambridge, 1995 - 2017\n"
-   "(c) The Exim Maintainers and contributors in ACKNOWLEDGMENTS file, 2007 - 2017";
+ US"Copyright (c) University of Cambridge, 1995 - 2018\n"
+   "(c) The Exim Maintainers and contributors in ACKNOWLEDGMENTS file, 2007 - 2018";
 uschar *version_date           = US"?";
 uschar *version_cnumber        = US"????";
 uschar *version_string         = US"?";
