@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Functions for doing things with sockets. With the advent of IPv6 this has
 got messier, so that it's worth pulling out the code into separate functions
@@ -16,12 +14,6 @@ different places in the code where sockets are used. */
 #include "exim.h"
 
 
-#if defined(TCP_FASTOPEN)
-# if defined(MSG_FASTOPEN) || defined(EXIM_TFO_CONNECTX) || defined(EXIM_TFO_FREEBSD)
-#  define EXIM_SUPPORT_TFO
-# endif
-#endif
-
 /*************************************************
 *             Create a socket                    *
 *************************************************/
@@ -77,7 +69,7 @@ ip_addrinfo(const uschar *address, struct sockaddr_in6 *saddr)
 #ifdef IPV6_USE_INET_PTON
 
   if (inet_pton(AF_INET6, CCS address, &saddr->sin6_addr) != 1)
-    log_write_die(0, LOG_MAIN, "unable to parse %q as an "
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to parse \"%s\" as an "
       "IP address", address);
   saddr->sin6_family = AF_INET6;
 
@@ -90,11 +82,11 @@ ip_addrinfo(const uschar *address, struct sockaddr_in6 *saddr)
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_flags = AI_NUMERICHOST;
   if ((rc = getaddrinfo(CCS address, NULL, &hints, &res)) != 0 || res == NULL)
-    log_write_die(0, LOG_MAIN, "unable to parse %q as an "
+    log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to parse \"%s\" as an "
       "IP address: %s", address,
-      rc == 0 ? "NULL result returned" : gai_strerror(rc));
-  else
-    { memcpy(saddr, res->ai_addr, res->ai_addrlen); freeaddrinfo(res); }
+      (rc == 0)? "NULL result returned" : gai_strerror(rc));
+  memcpy(saddr, res->ai_addr, res->ai_addrlen);
+  freeaddrinfo(res);
 
 #endif
 }
@@ -128,6 +120,8 @@ if (af == AF_INET6)
   return sizeof(sin->v6);
   }
 else
+#else     /* HAVE_IPv6 */
+af = af;  /* Avoid compiler warning */
 #endif    /* HAVE_IPV6 */
 
 /* Setup code when using IPv4 socket. The wildcard address is "". */
@@ -157,18 +151,35 @@ Returns:         the result of bind()
 */
 
 int
-ip_bind(int sock, int af, const uschar * address, int port)
+ip_bind(int sock, int af, uschar *address, int port)
 {
 union sockaddr_46 sin;
 int s_len = ip_addr(&sin, af, address, port);
-int rc = bind(sock, (struct sockaddr *)&sin, s_len);
-if (rc < 0)
-  log_write(0, LOG_MAIN, "bind of [%s]:%d failed", address, port);
-return rc;
+return bind(sock, (struct sockaddr *)&sin, s_len);
 }
 
 
 
+/*************************************************
+*************************************************/
+
+#ifdef EXIM_TFO_PROBE
+void
+tfo_probe(void)
+{
+# ifdef TCP_FASTOPEN
+int sock, backlog = 5;
+
+if (  (sock = socket(SOCK_STREAM, AF_INET, 0)) < 0
+   && setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, &backlog, sizeof(backlog))
+   )
+  tcp_fastopen_ok = TRUE;
+close(sock);
+# endif
+}
+#endif
+
+
 /*************************************************
 *        Connect socket to remote host           *
 *************************************************/
@@ -185,7 +196,7 @@ Arguments:
   port        the remote port
   timeout     a timeout (zero for indefinite timeout)
   fastopen_blob    non-null iff TCP_FASTOPEN can be used; may indicate early-data to
-		be sent in SYN segment.  Any such data must be idempotent.
+		be sent in SYN segment
 
 Returns:      0 on success; -1 on failure, with errno set
 */
@@ -211,6 +222,8 @@ if (af == AF_INET6)
   s_len = sizeof(s_in6);
   }
 else
+#else     /* HAVE_IPV6 */
+af = af;  /* Avoid compiler warning */
 #endif    /* HAVE_IPV6 */
 
 /* For an IPv4 address, use an IPv4 sockaddr structure, even on a system with
@@ -230,123 +243,57 @@ timer, thereby allowing the inbuilt OS timeout to operate. */
 
 callout_address = string_sprintf("[%s]:%d", address, port);
 sigalrm_seen = FALSE;
-if (timeout > 0) ALARM(timeout);
+if (timeout > 0) alarm(timeout);
 
-#ifdef EXIM_SUPPORT_TFO
+#if defined(TCP_FASTOPEN) && defined(MSG_FASTOPEN)
 /* TCP Fast Open, if the system has a cookie from a previous call to
 this peer, can send data in the SYN packet.  The peer can send data
 before it gets our ACK of its SYN,ACK - the latter is useful for
 the SMTP banner.  Other (than SMTP) cases of TCP connections can
-possibly use the data-on-syn, so support that too. */
+possibly use the data-on-syn, so support that too.  */
 
-if (fastopen_blob && f.tcp_fastopen_ok)
+if (fastopen_blob && tcp_fastopen_ok)
   {
-# ifdef MSG_FASTOPEN
-  /* This is a Linux implementation. */
-
   if ((rc = sendto(sock, fastopen_blob->data, fastopen_blob->len,
 		    MSG_FASTOPEN | MSG_DONTWAIT, s_ptr, s_len)) >= 0)
 	/* seen for with-data, experimental TFO option, with-cookie case */
 	/* seen for with-data, proper TFO opt, with-cookie case */
     {
     DEBUG(D_transport|D_v)
-      debug_printf(" TFO mode connection attempt to %s, %lu data\n",
+      debug_printf("non-TFO mode connection attempt to %s, %lu data\n",
 	address, (unsigned long)fastopen_blob->len);
-    /*XXX also seen on successful TFO, sigh */
-    tcp_out_fastopen = fastopen_blob->len > 0 ?  TFO_ATTEMPTED_DATA : TFO_ATTEMPTED_NODATA;
+    tcp_out_fastopen = fastopen_blob->len > 0 ?  2 : 1;
     }
-  else switch (errno)
-    {
-    case EINPROGRESS:	/* expected if we had no cookie for peer */
+  else if (errno == EINPROGRESS)	/* expected if we had no cookie for peer */
 	/* seen for no-data, proper TFO option, both cookie-request and with-cookie cases */
 	/*  apparently no visibility of the diffference at this point */
 	/* seen for with-data, proper TFO opt, cookie-req */
 	/*   with netwk delay, post-conn tcp_info sees unacked 1 for R, 2 for C; code in smtp_out.c */
 	/* ? older Experimental TFO option behaviour ? */
-      DEBUG(D_transport|D_v) debug_printf(" TFO mode sendto, %s data: EINPROGRESS\n",
-	fastopen_blob->len > 0 ? "with"  : "no");
-      if (!fastopen_blob->data)
-	{
-	tcp_out_fastopen = TFO_ATTEMPTED_NODATA;		/* we tried; unknown if useful yet */
-	rc = 0;
-	}
-      else					/* queue unsent data */
-	rc = send(sock, fastopen_blob->data, fastopen_blob->len, 0);
-      break;
-
-    case EOPNOTSUPP:
-      DEBUG(D_transport)
-	debug_printf("Tried TCP Fast Open but apparently not enabled by sysctl\n");
-      goto legacy_connect;
-
-    case EPIPE:
-      DEBUG(D_transport)
-	debug_printf("Tried TCP Fast Open but kernel too old to support it\n");
-      goto legacy_connect;
+    {					/* queue unsent data */
+    DEBUG(D_transport|D_v) debug_printf("TFO mode sendto, %s data: EINPROGRESS\n",
+      fastopen_blob->len > 0 ? "with"  : "no");
+    if (!fastopen_blob->data)
+      {
+      tcp_out_fastopen = 1;		/* we tried; unknown if useful yet */
+      rc = 0;
+      }
+    else
+      rc = send(sock, fastopen_blob->data, fastopen_blob->len, 0);
     }
-
-# elif defined(EXIM_TFO_FREEBSD)
-  /* Re: https://people.freebsd.org/~pkelsey/tfo-tools/tfo-client.c */
-
-  if (setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, &on, sizeof(on)) < 0)
+  else if(errno == EOPNOTSUPP)
     {
     DEBUG(D_transport)
       debug_printf("Tried TCP Fast Open but apparently not enabled by sysctl\n");
     goto legacy_connect;
     }
-  if ((rc = sendto(sock, fastopen_blob->data, fastopen_blob->len, 0,
-		    s_ptr, s_len)) >= 0)
-    {
-    DEBUG(D_transport|D_v)
-      debug_printf(" TFO mode connection attempt to %s, %lu data\n",
-	address, (unsigned long)fastopen_blob->len);
-    tcp_out_fastopen = fastopen_blob->len > 0 ?  TFO_ATTEMPTED_DATA : TFO_ATTEMPTED_NODATA;
-    }
-
-# elif defined(EXIM_TFO_CONNECTX)
-  /* MacOS */
-  sa_endpoints_t ends = {
-    .sae_srcif = 0, .sae_srcaddr = NULL, .sae_srcaddrlen = 0,
-    .sae_dstaddr = s_ptr, .sae_dstaddrlen = s_len };
-  struct iovec iov = {
-    .iov_base = fastopen_blob->data, .iov_len = fastopen_blob->len };
-  size_t len;
-
-  if ((rc = connectx(sock, &ends, SAE_ASSOCID_ANY,
-	     CONNECT_DATA_IDEMPOTENT, &iov, 1, &len, NULL)) == 0)
-    {
-    DEBUG(D_transport|D_v)
-      debug_printf(" TFO mode connection attempt to %s, %lu data\n",
-	address, (unsigned long)fastopen_blob->len);
-    tcp_out_fastopen = fastopen_blob->len > 0 ?  TFO_ATTEMPTED_DATA : TFO_ATTEMPTED_NODATA;
-
-    if (len != fastopen_blob->len)
-      DEBUG(D_transport|D_v)
-	debug_printf(" only queued %lu data!\n", (unsigned long)len);
-    }
-  else if (errno == EINPROGRESS)
-    {
-    DEBUG(D_transport|D_v) debug_printf(" TFO mode connectx, %s data: EINPROGRESS\n",
-      fastopen_blob->len > 0 ? "with"  : "no");
-    if (!fastopen_blob->data)
-      {
-      tcp_out_fastopen = TFO_ATTEMPTED_NODATA;		/* we tried; unknown if useful yet */
-      rc = 0;
-      }
-    else	/* assume that no data was queued; block in send */
-      rc = send(sock, fastopen_blob->data, fastopen_blob->len, 0);
-    }
-# endif
   }
 else
-#endif	/*EXIM_SUPPORT_TFO*/
-  {
-#if defined(EXIM_SUPPORT_TFO) && !defined(EXIM_TFO_CONNECTX)
-legacy_connect:
 #endif
-
+  {
+legacy_connect:
   DEBUG(D_transport|D_v) if (fastopen_blob)
-    debug_printf(" non-TFO mode connection attempt to %s, %lu data\n",
+    debug_printf("non-TFO mode connection attempt to %s, %lu data\n",
       address, (unsigned long)fastopen_blob->len);
   if ((rc = connect(sock, s_ptr, s_len)) >= 0)
     if (  fastopen_blob && fastopen_blob->data && fastopen_blob->len
@@ -355,13 +302,13 @@ legacy_connect:
   }
 
 save_errno = errno;
-ALARM_CLR(0);
+alarm(0);
 
 /* There is a testing facility for simulating a connection timeout, as I
 can't think of any other way of doing this. It converts a connection refused
 into a timeout if the timeout is set to 999999. */
 
-if (f.running_in_test_harness  && save_errno == ECONNREFUSED && timeout == 999999)
+if (running_in_test_harness  && save_errno == ECONNREFUSED && timeout == 999999)
   {
   rc = -1;
   save_errno = EINTR;
@@ -392,13 +339,13 @@ return -1;
 Arguments:
   type          SOCK_DGRAM or SOCK_STREAM
   af            AF_INET6 or AF_INET for the socket type
-  hostname	host name, or ip address (as text)
+  address       the remote address, in text form
   portlo,porthi the remote port range
   timeout       a timeout
   connhost	if not NULL, host_item to be filled in with connection details
   errstr        pointer for allocated string on error
   fastopen_blob	with SOCK_STREAM, if non-null, request TCP Fast Open.
-		Additionally, optional idempotent early-data to send
+		Additionally, optional early-data to send
 
 Return:
   socket fd, or -1 on failure (having allocated an error string)
@@ -407,8 +354,9 @@ int
 ip_connectedsocket(int type, const uschar * hostname, int portlo, int porthi,
       int timeout, host_item * connhost, uschar ** errstr, const blob * fastopen_blob)
 {
-int namelen;
+int namelen, port;
 host_item shost;
+host_item *h;
 int af = 0, fd, fd4 = -1, fd6 = -1;
 
 shost.next = NULL;
@@ -426,7 +374,7 @@ if (hostname[0] == '[' &&
   uschar * host = string_copyn(hostname+1, namelen-2);
   if (string_is_ip_address(host, NULL) == 0)
     {
-    *errstr = string_sprintf("malformed IP address %q", hostname);
+    *errstr = string_sprintf("malformed IP address \"%s\"", hostname);
     return -1;
     }
   shost.name = shost.address = host;
@@ -452,7 +400,7 @@ else
 
 /* Try to connect to the server - test each IP till one works */
 
-for (host_item * h = &shost; h; h = h->next)
+for (h = &shost; h; h = h->next)
   {
   fd = Ustrchr(h->address, ':') != 0
     ? fd6 < 0 ? (fd6 = ip_socket(type, af = AF_INET6)) : fd6
@@ -464,11 +412,11 @@ for (host_item * h = &shost; h; h = h->next)
     goto bad;
     }
 
-  for (int port = portlo; port <= porthi; port++)
+  for(port = portlo; port <= porthi; port++)
     if (ip_connect(fd, af, h->address, port, timeout, fastopen_blob) == 0)
       {
-      if (fd6 >= 0 && fd != fd6) close(fd6);
-      if (fd4 >= 0 && fd != fd4) close(fd4);
+      if (fd != fd6) close(fd6);
+      if (fd != fd4) close(fd4);
       if (connhost)
 	{
 	h->port = port;
@@ -489,8 +437,7 @@ bad:
 
 /*XXX TFO? */
 int
-ip_tcpsocket(const uschar * hostport, uschar ** errstr, int tmo,
-  host_item * connhost)
+ip_tcpsocket(const uschar * hostport, uschar ** errstr, int tmo)
 {
 int scan;
 uschar hostname[256];
@@ -509,7 +456,7 @@ if (scan != 3)
   }
 
 return ip_connectedsocket(SOCK_STREAM, hostname, portlow, porthigh,
-			  tmo, connhost, errstr, NULL);
+			  tmo, NULL, errstr, NULL);
 }
 
 int
@@ -526,7 +473,7 @@ if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)
 
 callout_address = string_copy(path);
 server.sun_family = AF_UNIX;
-Ustrncpy(US server.sun_path, path, sizeof(server.sun_path)-1);
+Ustrncpy(server.sun_path, path, sizeof(server.sun_path)-1);
 server.sun_path[sizeof(server.sun_path)-1] = '\0';
 if (connect(sock, (struct sockaddr *) &server, sizeof(server)) < 0)
   {
@@ -539,18 +486,11 @@ if (connect(sock, (struct sockaddr *) &server, sizeof(server)) < 0)
 return sock;
 }
 
-/* spec is either an absolute path (with a leading /), or
-a host (name or IP) and port (whitespace-separated).
-The port can be a range, dash-separated, or a single number.
-
-For a TCP socket, optionally fill in a  host_item.
-*/
 int
-ip_streamsocket(const uschar * spec, uschar ** errstr, int tmo,
-  host_item * connhost)
+ip_streamsocket(const uschar * spec, uschar ** errstr, int tmo)
 {
 return *spec == '/'
-  ? ip_unixsocket(spec, errstr) : ip_tcpsocket(spec, errstr, tmo, connhost);
+  ? ip_unixsocket(spec, errstr) : ip_tcpsocket(spec, errstr, tmo);
 }
 
 /*************************************************
@@ -586,14 +526,17 @@ if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE,
 /*
 Arguments:
   fd          the file descriptor
-  timelimit   the timeout endpoint, seconds-since-epoch
+  timeout     the timeout, seconds
 Returns:      TRUE => ready for i/o
               FALSE => timed out, or other error
 */
 BOOL
-fd_ready(int fd, time_t timelimit)
+fd_ready(int fd, int timeout)
 {
-int rc, time_left = timelimit - time(NULL);
+fd_set select_inset;
+time_t start_recv = time(NULL);
+int time_left = timeout;
+int rc;
 
 if (time_left <= 0)
   {
@@ -604,8 +547,12 @@ if (time_left <= 0)
 
 do
   {
+  struct timeval tv = { .tv_sec = time_left, .tv_usec = 0 };
+  FD_ZERO (&select_inset);
+  FD_SET (fd, &select_inset);
+
   /*DEBUG(D_transport) debug_printf("waiting for data on fd\n");*/
-  rc = poll_one_fd(fd, POLLIN, time_left * 1000);
+  rc = select(fd + 1, (SELECT_ARG2_TYPE *)&select_inset, NULL, NULL, &tv);
 
   /* If some interrupt arrived, just retry. We presume this to be rare,
   but it can happen (e.g. the SIGUSR1 signal sent by exiwhat causes
@@ -622,7 +569,8 @@ do
     DEBUG(D_transport) debug_printf("EINTR while waiting for socket data\n");
 
     /* Watch out, 'continue' jumps to the condition, not to the loops top */
-    if ((time_left = timelimit - time(NULL)) > 0) continue;
+    time_left = timeout - (time(NULL) - start_recv);
+    if (time_left > 0) continue;
     }
 
   if (rc <= 0)
@@ -634,7 +582,7 @@ do
   /* Checking the FD_ISSET is not enough, if we're interrupted, the
   select_inset may still contain the 'input'. */
   }
-while (rc < 0);
+while (rc < 0 || !FD_ISSET(fd, &select_inset));
 return TRUE;
 }
 
@@ -643,36 +591,34 @@ getting interrupted, and the possibility of select() returning with a positive
 result but no ready descriptor. Is this in fact possible?
 
 Arguments:
-  cctx        the connection context (socket fd, possibly TLS context)
+  sock        the socket
   buffer      to read into
   bufsize     the buffer size
-  timelimit   the timeout endpoint, seconds-since-epoch
+  timeout     the timeout
 
 Returns:      > 0 => that much data read
-              <= 0 on error or EOF; errno set - zero for EOF or zero buffer
+              <= 0 on error or EOF; errno set - zero for EOF
 */
 
 int
-ip_recv(client_conn_ctx * cctx, uschar * buffer, int buffsize, time_t timelimit)
+ip_recv(int sock, uschar *buffer, int buffsize, int timeout)
 {
 int rc;
 
-if (buffsize == 0)
-  { errno = 0; return 0; }
-if (!fd_ready(cctx->sock, timelimit))
+if (!fd_ready(sock, timeout))
   return -1;
 
 /* The socket is ready, read from it (via TLS if it's active). On EOF (i.e.
 close down of the connection), set errno to zero; otherwise leave it alone. */
 
-#ifndef DISABLE_TLS
-if (cctx->tls_ctx)					/* client TLS */
-  rc = tls_read(cctx->tls_ctx, buffer, buffsize);
-else if (tls_in.active.sock == cctx->sock)		/* server TLS */
-  rc = tls_read(NULL, buffer, buffsize);
+#ifdef SUPPORT_TLS
+if (tls_out.active == sock)
+  rc = tls_read(FALSE, buffer, buffsize);
+else if (tls_in.active == sock)
+  rc = tls_read(TRUE, buffer, buffsize);
 else
 #endif
-  rc = recv(cctx->sock, buffer, buffsize, 0);
+  rc = recv(sock, buffer, buffsize, 0);
 
 if (rc > 0) return rc;
 if (rc == 0) errno = 0;
@@ -849,7 +795,8 @@ return FALSE;
 void
 dscp_list_to_stream(FILE *stream)
 {
-for (int i = 0; i < dscp_table_size; ++i)
+int i;
+for (i=0; i < dscp_table_size; ++i)
   fprintf(stream, "%s\n", dscp_table[i].name);
 }
 
