@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "../exim.h"
 
@@ -30,7 +28,7 @@ static redis_connection *redis_connections = NULL;
 
 
 static void *
-redis_open(const uschar * filename, uschar ** errmsg)
+redis_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);
 }
@@ -49,7 +47,7 @@ redis_connection *cn;
 while ((cn = redis_connections))
   {
   redis_connections = cn->next;
-  DEBUG(D_lookup) debug_printf_indent("close REDIS connection: %s\n", cn->server);
+  DEBUG(D_lookup) debug_printf("close REDIS connection: %s\n", cn->server);
   redisFree(cn->handle);
   }
 }
@@ -65,7 +63,6 @@ single server.
       errmsg       where to point an error message
       defer_break  TRUE if no more servers are to be tried after DEFER
       do_cache     set false if data is changed
-      opts	   options
 
     The server string is of the form "host/dbnumber/password". The host can be
     host:port. This string is in a nextinlist temporary buffer, so can be
@@ -76,17 +73,18 @@ single server.
 
 static int
 perform_redis_search(const uschar *command, uschar *server, uschar **resultptr,
-  uschar **errmsg, BOOL *defer_break, uint *do_cache, const uschar * opts)
+  uschar **errmsg, BOOL *defer_break, uint *do_cache)
 {
-redisContext * redis_handle = NULL;        /* Keep compilers happy */
-redisReply * redis_reply = NULL;
-redisReply * entry = NULL;
-redisReply * tentry = NULL;
+redisContext *redis_handle = NULL;        /* Keep compilers happy */
+redisReply *redis_reply = NULL;
+redisReply *entry = NULL;
+redisReply *tentry = NULL;
 redis_connection *cn;
 int yield = DEFER;
+int i, j;
 gstring * result = NULL;
-uschar * server_copy = NULL;
-uschar * sdata[3];
+uschar *server_copy = NULL;
+uschar *sdata[3];
 
 /* Disaggregate the parameters from the server argument.
 The order is host:port(socket)
@@ -94,20 +92,20 @@ We can write to the string, since it is in a nextinlist temporary buffer.
 This copy is also used for debugging output.  */
 
 memset(sdata, 0, sizeof(sdata)) /* Set all to NULL */;
-for (int j = 2; j > 0; j--)
+for (i = 2; i > 0; i--)
   {
-  uschar * pp = Ustrrchr(server, '/');
+  uschar *pp = Ustrrchr(server, '/');
 
   if (!pp)
     {
     *errmsg = string_sprintf("incomplete Redis server data: %s",
-      j == 2 ? server : server_copy);
+      i == 2 ? server : server_copy);
     *defer_break = TRUE;
     return DEFER;
     }
-  *pp++ = '\0';
-  sdata[j] = pp;
-  if (j == 2) server_copy = string_copy(server);  /* sans password */
+  *pp++ = 0;
+  sdata[i] = pp;
+  if (i == 2) server_copy = string_copy(server);  /* sans password */
   }
 sdata[0] = server;   /* What's left at the start */
 
@@ -156,7 +154,7 @@ if (!cn)
     }
 
   DEBUG(D_lookup)
-    debug_printf_indent("REDIS new connection: host=%s port=%d socket=%s database=%s\n",
+    debug_printf("REDIS new connection: host=%s port=%d socket=%s database=%s\n",
       sdata[0], port, socket, sdata[1]);
 
   /* Get store for a new handle, initialize it, and connect to the server */
@@ -165,20 +163,23 @@ if (!cn)
     socket ? redisConnectUnix(CCS socket) : redisConnect(CCS server, port);
   if (!redis_handle)
     {
-    *errmsg = US"REDIS connection failed";
+    *errmsg = string_sprintf("REDIS connection failed");
     *defer_break = FALSE;
     goto REDIS_EXIT;
     }
 
   /* Add the connection to the cache */
-  cn = store_get(sizeof(redis_connection), GET_UNTAINTED);
+  cn = store_get(sizeof(redis_connection));
   cn->server = server_copy;
   cn->handle = redis_handle;
   cn->next = redis_connections;
   redis_connections = cn;
   }
-else DEBUG(D_lookup)
-  debug_printf_indent("REDIS using cached connection for %s\n", server_copy);
+else
+  {
+  DEBUG(D_lookup)
+    debug_printf("REDIS using cached connection for %s\n", server_copy);
+}
 
 /* Authenticate if there is a password */
 if(sdata[2])
@@ -198,17 +199,18 @@ if(sdata[1])
     *defer_break = FALSE;
     goto REDIS_EXIT;
     }
-  DEBUG(D_lookup) debug_printf_indent("REDIS: Selecting database=%s\n", sdata[1]);
+  DEBUG(D_lookup) debug_printf("REDIS: Selecting database=%s\n", sdata[1]);
   }
 
 /* split string on whitespace into argv */
   {
   uschar * argv[32];
-  const uschar * s = command;
   int i;
+  const uschar * s = command;
+  int siz, ptr;
   uschar c;
 
-  Uskip_whitespace(&s);
+  while (isspace(*s)) s++;
 
   for (i = 0; *s && i < nele(argv); i++)
     {
@@ -219,14 +221,15 @@ if(sdata[1])
 	g = string_catn(g, s, 1);
     argv[i] = string_from_gstring(g);
 
-    DEBUG(D_lookup) debug_printf_indent("REDIS: argv[%d] '%s'\n", i, argv[i]);
-    Uskip_whitespace(&s);
+    DEBUG(D_lookup) debug_printf("REDIS: argv[%d] '%s'\n", i, argv[i]);
+    while (isspace(*s)) s++;
     }
 
   /* Run the command. We use the argv form rather than plain as that parses
   into args by whitespace yet has no escaping mechanism. */
 
-  if (!(redis_reply = redisCommandArgv(redis_handle, i, CCSS argv, NULL)))
+  redis_reply = redisCommandArgv(redis_handle, i, (const char **) argv, NULL);
+  if (!redis_reply)
     {
     *errmsg = string_sprintf("REDIS: query failed: %s\n", redis_handle->errstr);
     *defer_break = FALSE;
@@ -243,7 +246,7 @@ switch (redis_reply->type)
     if (Ustrncmp(redis_reply->str, "MOVED", 5) == 0)
       {
       DEBUG(D_lookup)
-        debug_printf_indent("REDIS: cluster redirect %s\n", redis_reply->str);
+        debug_printf("REDIS: cluster redirect %s\n", redis_reply->str);
       /* follow redirect
       This is cheating, we simply set defer_break = FALSE to move on to
       the next server in the redis_servers list */
@@ -258,7 +261,7 @@ switch (redis_reply->type)
 
   case REDIS_REPLY_NIL:
     DEBUG(D_lookup)
-      debug_printf_indent("REDIS: query was not one that returned any data\n");
+      debug_printf("REDIS: query was not one that returned any data\n");
     result = string_catn(result, US"", 1);
     *do_cache = 0;
     goto REDIS_EXIT;
@@ -278,9 +281,9 @@ switch (redis_reply->type)
     /* NOTE: For now support 1 nested array result. If needed a limitless
     result can be parsed */
 
-    for (int k = 0; k < redis_reply->elements; k++)
+    for (i = 0; i < redis_reply->elements; i++)
       {
-      entry = redis_reply->element[k];
+      entry = redis_reply->element[i];
 
       if (result)
 	result = string_catn(result, US"\n", 1);
@@ -288,15 +291,15 @@ switch (redis_reply->type)
       switch (entry->type)
 	{
 	case REDIS_REPLY_INTEGER:
-	  result = string_fmt_append(result, "%d", entry->integer);
+	  result = string_cat(result, string_sprintf("%d", entry->integer));
 	  break;
 	case REDIS_REPLY_STRING:
 	  result = string_catn(result, US entry->str, entry->len);
 	  break;
 	case REDIS_REPLY_ARRAY:
-	  for (int n = 0; n < entry->elements; n++)
+	  for (j = 0; j < entry->elements; j++)
 	    {
-	    tentry = entry->element[n];
+	    tentry = entry->element[j];
 
 	    if (result)
 	      result = string_catn(result, US"\n", 1);
@@ -304,25 +307,25 @@ switch (redis_reply->type)
 	    switch (tentry->type)
 	      {
 	      case REDIS_REPLY_INTEGER:
-		result = string_fmt_append(result, "%d", tentry->integer);
+		result = string_cat(result, string_sprintf("%d", tentry->integer));
 		break;
 	      case REDIS_REPLY_STRING:
 		result = string_catn(result, US tentry->str, tentry->len);
 		break;
 	      case REDIS_REPLY_ARRAY:
 		DEBUG(D_lookup)
-		  debug_printf_indent("REDIS: result has nesting of arrays which"
+		  debug_printf("REDIS: result has nesting of arrays which"
 		    " is not supported. Ignoring!\n");
 		break;
 	      default:
-		DEBUG(D_lookup) debug_printf_indent(
+		DEBUG(D_lookup) debug_printf(
 			  "REDIS: result has unsupported type. Ignoring!\n");
 		break;
 	      }
 	    }
 	    break;
 	  default:
-	    DEBUG(D_lookup) debug_printf_indent("REDIS: query returned unsupported type\n");
+	    DEBUG(D_lookup) debug_printf("REDIS: query returned unsupported type\n");
 	    break;
 	  }
 	}
@@ -331,7 +334,7 @@ switch (redis_reply->type)
 
 
 if (result)
-  gstring_release_unused(result);
+  store_reset(result->s + result->ptr + 1);
 else
   {
   yield = FAIL;
@@ -354,7 +357,7 @@ if (result)
   }
 else
   {
-  DEBUG(D_lookup) debug_printf_indent("%s\n", *errmsg);
+  DEBUG(D_lookup) debug_printf("%s\n", *errmsg);
   /* NOTE: Required to close connection since it needs to be reopened */
   return yield;      /* FAIL or DEFER */
   }
@@ -373,13 +376,13 @@ else
  */
 
 static int
-redis_find(void * handle __attribute__((unused)),
-  const uschar * filename __attribute__((unused)),
-  const uschar * command, int length, uschar ** result, uschar ** errmsg,
-  uint * do_cache, const uschar * opts)
+redis_find(void *handle __attribute__((unused)),
+  uschar *filename __attribute__((unused)),
+  const uschar *command, int length, uschar **result, uschar **errmsg,
+  uint *do_cache)
 {
 return lf_sqlperform(US"Redis", US"redis_servers", redis_servers, command,
-  result, errmsg, do_cache, opts, perform_redis_search);
+  result, errmsg, do_cache, perform_redis_search);
 }
 
 
@@ -396,25 +399,27 @@ whitespace into an argument.
 Arguments:
   s          the string to be quoted
   opt        additional option text or NULL if none
-  idx	     lookup type index
 
 Returns:     the processed string or NULL for a bad option
 */
 
 static uschar *
-redis_quote(uschar * s, uschar * opt, unsigned idx)
+redis_quote(uschar *s, uschar *opt)
 {
-int c, count = 0;
-uschar * t = s, * quoted;
+register int c;
+int count = 0;
+uschar *t = s;
+uschar *quoted;
 
 if (opt) return NULL;     /* No options recognized */
 
-while ((c = *t++))
+while ((c = *t++) != 0)
   if (isspace(c) || c == '\\') count++;
 
-t = quoted = store_get_quoted(Ustrlen(s) + count + 1, s, idx, US"redis");
+if (count == 0) return s;
+t = quoted = store_get(Ustrlen(s) + count + 1);
 
-while ((c = *s++))
+while ((c = *s++) != 0)
   {
   if (isspace(c) || c == '\\') *t++ = '\\';
   *t++ = c;
@@ -430,31 +435,29 @@ return quoted;
 *************************************************/
 #include "../version.h"
 
-gstring *
-redis_version_report(gstring * g)
+void
+redis_version_report(FILE *f)
 {
-g = string_fmt_append(g,
-  "Library version: REDIS: Compile: %d [%d]\n", HIREDIS_MAJOR, HIREDIS_MINOR);
+fprintf(f, "Library version: REDIS: Compile: %d [%d]\n",
+               HIREDIS_MAJOR, HIREDIS_MINOR);
 #ifdef DYNLOOKUP
-g = string_fmt_append(g,
-  "                        Exim version %s\n", EXIM_VERSION_STR);
+fprintf(f, "                        Exim version %s\n", EXIM_VERSION_STR);
 #endif
-return g;
 }
 
 
 
 /* These are the lookup_info blocks for this driver */
 static lookup_info redis_lookup_info = {
-  .name = US"redis",			/* lookup name */
-  .type = lookup_querystyle,		/* query-style lookup */
-  .open = redis_open,			/* open function */
-  .check = NULL,			/* no check function */
-  .find = redis_find,			/* find function */
-  .close = NULL,			/* no close function */
-  .tidy = redis_tidy,			/* tidy function */
-  .quote = redis_quote,			/* quoting function */
-  .version_report = redis_version_report           /* version reporting */
+  US"redis",                     /* lookup name */
+  lookup_querystyle,             /* query-style lookup */
+  redis_open,                    /* open function */
+  NULL,                          /* no check function */
+  redis_find,                    /* find function */
+  NULL,                          /* no close function */
+  redis_tidy,                    /* tidy function */
+  redis_quote,                   /* quoting function */
+  redis_version_report           /* version reporting */
 };
 
 #ifdef DYNLOOKUP
