@@ -1,7 +1,7 @@
 /*
  *  PDKIM - a RFC4871 (DKIM) implementation
- *  Copyright (c) The Exim Maintainers 1995 - 2024
- *  SPDX-License-Identifier: GPL-2.0-or-later
+ *
+ *  Copyright (C) 1995 - 2018  Exim maintainers
  *
  *  signing/verification interface
  */
@@ -28,8 +28,8 @@ features_crypto(void)
 
 #ifndef DISABLE_DKIM	/* rest of file */
 
-#ifdef DISABLE_TLS
-# error Must no DISABLE_TLS, for DKIM
+#ifndef SUPPORT_TLS
+# error Need SUPPORT_TLS for DKIM
 #endif
 
 
@@ -37,10 +37,6 @@ features_crypto(void)
 #ifdef SIGN_GNUTLS
 # define EXIM_GNUTLS_LIBRARY_LOG_LEVEL 3
 
-# ifndef GNUTLS_VERIFY_ALLOW_BROKEN
-#  define GNUTLS_VERIFY_ALLOW_BROKEN 0
-# endif
-
 
 /* Logging function which can be registered with
  *   gnutls_global_set_log_function()
@@ -94,17 +90,14 @@ exim_dkim_signing_init(const uschar * privkey_pem, es_ctx * sign_ctx)
 {
 gnutls_datum_t k = { .data = (void *)privkey_pem, .size = Ustrlen(privkey_pem) };
 gnutls_x509_privkey_t x509_key;
-const uschar * where;
 int rc;
 
-if (  (where = US"internal init", rc = gnutls_x509_privkey_init(&x509_key))
+if (  (rc = gnutls_x509_privkey_init(&x509_key))
+   || (rc = gnutls_x509_privkey_import(x509_key, &k, GNUTLS_X509_FMT_PEM))
    || (rc = gnutls_privkey_init(&sign_ctx->key))
-   || (where = US"privkey PEM-block import",
-       rc = gnutls_x509_privkey_import(x509_key, &k, GNUTLS_X509_FMT_PEM))
-   || (where = US"internal privkey transfer",
-       rc = gnutls_privkey_import_x509(sign_ctx->key, x509_key, 0))
+   || (rc = gnutls_privkey_import_x509(sign_ctx->key, x509_key, 0))
    )
-  return string_sprintf("%s: %s", where, gnutls_strerror(rc));
+  return CUS gnutls_strerror(rc);
 
 switch (rc = gnutls_privkey_get_pk_algorithm(sign_ctx->key, NULL))
   {
@@ -159,8 +152,7 @@ return NULL;
 Return: NULL for success, or an error string */
 
 const uschar *
-exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx,
-  unsigned * bits)
+exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx)
 {
 gnutls_datum_t k;
 int rc;
@@ -174,20 +166,19 @@ switch(fmt)
   {
   case KEYFMT_DER:
     if ((rc = gnutls_pubkey_import(verify_ctx->key, &k, GNUTLS_X509_FMT_DER)))
-      ret = US gnutls_strerror(rc);
+      ret = gnutls_strerror(rc);
     break;
 #ifdef SIGN_HAVE_ED25519
   case KEYFMT_ED25519_BARE:
     if ((rc = gnutls_pubkey_import_ecc_raw(verify_ctx->key,
 					  GNUTLS_ECC_CURVE_ED25519, &k, NULL)))
-      ret = US gnutls_strerror(rc);
+      ret = gnutls_strerror(rc);
     break;
 #endif
   default:
     ret = US"pubkey format not handled";
     break;
   }
-if (!ret && bits) gnutls_pubkey_get_pk_algorithm(verify_ctx->key, bits);
 return ret;
 }
 
@@ -209,7 +200,7 @@ if (verify_ctx->keytype == KEYTYPE_ED25519)
   {
   if ((rc = gnutls_pubkey_verify_data2(verify_ctx->key,
 				      GNUTLS_SIGN_EDDSA_ED25519, 0, &k, &s)) < 0)
-    ret = US gnutls_strerror(rc);
+    ret = gnutls_strerror(rc);
   }
 else
 #endif
@@ -223,9 +214,8 @@ else
     default:		return US"nonhandled hash type";
     }
 
-  if ((rc = gnutls_pubkey_verify_hash2(verify_ctx->key, algo,
-	      GNUTLS_VERIFY_ALLOW_BROKEN, &k, &s)) < 0)
-    ret = US gnutls_strerror(rc);
+  if ((rc = gnutls_pubkey_verify_hash2(verify_ctx->key, algo, 0, &k, &s)) < 0)
+    ret = gnutls_strerror(rc);
   }
 
 gnutls_pubkey_deinit(verify_ctx->key);
@@ -308,7 +298,7 @@ makes sure that important subsystems are initialized. */
 if (!gcry_check_version (GCRYPT_VERSION))
   {
   fputs ("libgcrypt version mismatch\n", stderr);
-  exim_exit(2);
+  exit (2);
   }
 
 /* We don't want to see any warnings, e.g. because we have not yet
@@ -419,9 +409,8 @@ if (  !(s1 = Ustrstr(CS privkey_pem, "-----BEGIN RSA PRIVATE KEY-----"))
 
 *s2 = '\0';
 
-if ((rc = b64decode(s1, &der.data, s1) < 0))
+if ((der.len = b64decode(s1, &der.data)) < 0)
   return US"Bad PEM-DER b64 decode";
-der.len = rc;
 
 /* untangle asn.1 */
 
@@ -507,7 +496,7 @@ switch (hash)
   }
 
 #define SIGSPACE 128
-sig->data = store_get(SIGSPACE, GET_UNTAINTED);
+sig->data = store_get(SIGSPACE);
 
 if (gcry_mpi_cmp (sign_ctx->p, sign_ctx->q) > 0)
   {
@@ -559,8 +548,7 @@ return NULL;
 Return: NULL for success, or an error string */
 
 const uschar *
-exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx,
-  unsigned * bits)
+exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx)
 {
 /*
 in code sequence per b81207d2bfa92 rsa_parse_public_key() and asn1_get_mpi()
@@ -568,7 +556,6 @@ in code sequence per b81207d2bfa92 rsa_parse_public_key() and asn1_get_mpi()
 uschar tag_class;
 int taglen;
 long alen;
-unsigned nbits;
 int rc;
 uschar * errstr;
 gcry_error_t gerr;
@@ -617,10 +604,10 @@ if ((rc = as_tag(pubkey, ASN1_CLASS_STRUCTURED, ASN1_TAG_SEQUENCE, NULL))
 
 /* read two integers */
 DEBUG(D_acl) stage = US"MPI";
-nbits = pubkey->len;
-if ((errstr = as_mpi(pubkey, &verify_ctx->n))) return errstr;
-nbits = (nbits - pubkey->len) * 8;
-if ((errstr = as_mpi(pubkey, &verify_ctx->e))) return errstr;
+if (  (errstr = as_mpi(pubkey, &verify_ctx->n))
+   || (errstr = as_mpi(pubkey, &verify_ctx->e))
+   )
+  return errstr;
 
 #ifdef extreme_debug
 DEBUG(D_acl) debug_printf_indent("rsa_verify_init:\n");
@@ -633,7 +620,6 @@ DEBUG(D_acl) debug_printf_indent("rsa_verify_init:\n");
 	}
 
 #endif
-if (bits) *bits = nbits;
 return NULL;
 
 asn_err:
@@ -723,11 +709,10 @@ Return: NULL for success, or an error string */
 const uschar *
 exim_dkim_signing_init(const uschar * privkey_pem, es_ctx * sign_ctx)
 {
-BIO * bp = BIO_new_mem_buf((void *)privkey_pem, -1);
+BIO * bp = BIO_new_mem_buf(privkey_pem, -1);
 
 if (!(sign_ctx->key = PEM_read_bio_PrivateKey(bp, NULL, NULL, NULL)))
-  return string_sprintf("privkey PEM-block import: %s",
-		       	ERR_error_string(ERR_get_error(), NULL));
+  return US ERR_error_string(ERR_get_error(), NULL);
 
 sign_ctx->keytype =
 #ifdef SIGN_HAVE_ED25519
@@ -766,7 +751,7 @@ switch (hash)
 if (  (ctx = EVP_MD_CTX_new())
    && EVP_DigestSignInit(ctx, NULL, md, NULL, sign_ctx->key) > 0
    && EVP_DigestSign(ctx, NULL, &siglen, NULL, 0) > 0
-   && (sig->data = store_get(siglen, GET_UNTAINTED))
+   && (sig->data = store_get(siglen))
 
    /* Obtain the signature (slen could change here!) */
    && EVP_DigestSign(ctx, sig->data, &siglen, data->data, data->len) > 0
@@ -782,7 +767,7 @@ if (  (ctx = EVP_MD_CTX_create())
    && EVP_DigestSignInit(ctx, NULL, md, NULL, sign_ctx->key) > 0
    && EVP_DigestSignUpdate(ctx, data->data, data->len) > 0
    && EVP_DigestSignFinal(ctx, NULL, &siglen) > 0
-   && (sig->data = store_get(siglen, GET_UNTAINTED))
+   && (sig->data = store_get(siglen))
  
    /* Obtain the signature (slen could change here!) */
    && EVP_DigestSignFinal(ctx, sig->data, &siglen) > 0
@@ -804,8 +789,7 @@ return US ERR_error_string(ERR_get_error(), NULL);
 Return: NULL for success, or an error string */
 
 const uschar *
-exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx,
-  unsigned * bits)
+exim_dkim_verify_init(blob * pubkey, keyformat fmt, ev_ctx * verify_ctx)
 {
 const uschar * s = pubkey->data;
 uschar * ret = NULL;
@@ -829,7 +813,6 @@ switch(fmt)
     break;
   }
 
-if (!ret && bits) *bits = EVP_PKEY_bits(verify_ctx->key);
 return ret;
 }
 
@@ -859,38 +842,30 @@ if (!md)
   {
   EVP_MD_CTX * ctx;
 
-  if ((ctx = EVP_MD_CTX_new()))
-    {
-    if (  EVP_DigestVerifyInit(ctx, NULL, md, NULL, verify_ctx->key) > 0
-       && EVP_DigestVerify(ctx, sig->data, sig->len, data->data, data->len) > 0
-       )
-      { EVP_MD_CTX_free(ctx); return NULL; }
-    EVP_MD_CTX_free(ctx);
-    }
+  if (  (ctx = EVP_MD_CTX_new())
+     && EVP_DigestVerifyInit(ctx, NULL, md, NULL, verify_ctx->key) > 0
+     && EVP_DigestVerify(ctx, sig->data, sig->len, data->data, data->len) > 0
+     )
+    { EVP_MD_CTX_free(ctx); return NULL; }
+
+  if (ctx) EVP_MD_CTX_free(ctx);
   }
 else
 #endif
   {
   EVP_PKEY_CTX * ctx;
 
-  if ((ctx = EVP_PKEY_CTX_new(verify_ctx->key, NULL)))
-    {
-    if (  EVP_PKEY_verify_init(ctx) > 0
-       && EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING) > 0
-       && EVP_PKEY_CTX_set_signature_md(ctx, md) > 0
-       && EVP_PKEY_verify(ctx, sig->data, sig->len,
-				      data->data, data->len) == 1
-       )
-      { EVP_PKEY_CTX_free(ctx); return NULL; }
-    EVP_PKEY_CTX_free(ctx);
+  if (  (ctx = EVP_PKEY_CTX_new(verify_ctx->key, NULL))
+     && EVP_PKEY_verify_init(ctx) > 0
+     && EVP_PKEY_CTX_set_rsa_padding(ctx, RSA_PKCS1_PADDING) > 0
+     && EVP_PKEY_CTX_set_signature_md(ctx, md) > 0
+     && EVP_PKEY_verify(ctx, sig->data, sig->len,
+	  data->data, data->len) == 1
+     )
+    { EVP_PKEY_CTX_free(ctx); return NULL; }
 
-    DEBUG(D_tls)
-      if (Ustrcmp(ERR_reason_error_string(ERR_peek_error()), "wrong signature length") == 0)
-	debug_printf("sig len (from msg hdr): %d, expected (from dns pubkey) %d\n",
-	 (int) sig->len, EVP_PKEY_size(verify_ctx->key));
-    }
+  if (ctx) EVP_PKEY_CTX_free(ctx);
   }
-
 return US ERR_error_string(ERR_get_error(), NULL);
 }
 
