@@ -2,36 +2,44 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "../exim.h"
-
-#ifdef ROUTER_DNSLOOKUP		/* Remainder of file */
 #include "rf_functions.h"
 #include "dnslookup.h"
 
 
 
 /* Options specific to the dnslookup router. */
-#define LOFF(field) OPT_OFF(dnslookup_router_options_block, field)
 
 optionlist dnslookup_router_options[] = {
-  { "check_secondary_mx", opt_bool,		LOFF(check_secondary_mx) },
-  { "check_srv",          opt_stringptr,	LOFF(check_srv) },
-  { "fail_defer_domains", opt_stringptr,	LOFF(fail_defer_domains) },
-  { "ipv4_only",          opt_stringptr,	LOFF(ipv4_only) },
-  { "ipv4_prefer",        opt_stringptr,	LOFF(ipv4_prefer) },
-  { "mx_domains",         opt_stringptr,	LOFF(mx_domains) },
-  { "mx_fail_domains",    opt_stringptr,	LOFF(mx_fail_domains) },
-  { "qualify_single",     opt_bool,		LOFF(qualify_single) },
-  { "rewrite_headers",    opt_bool,		LOFF(rewrite_headers) },
-  { "same_domain_copy_routing", opt_bool|opt_public, OPT_OFF(router_instance, same_domain_copy_routing) },
-  { "search_parents",     opt_bool,		LOFF(search_parents) },
-  { "srv_fail_domains",   opt_stringptr,	LOFF(srv_fail_domains) },
-  { "widen_domains",      opt_stringptr,	LOFF(widen_domains) }
+  { "check_secondary_mx", opt_bool,
+      (void *)(offsetof(dnslookup_router_options_block, check_secondary_mx)) },
+  { "check_srv",          opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, check_srv)) },
+  { "fail_defer_domains", opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, fail_defer_domains)) },
+  { "ipv4_only",          opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, ipv4_only)) },
+  { "ipv4_prefer",        opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, ipv4_prefer)) },
+  { "mx_domains",         opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, mx_domains)) },
+  { "mx_fail_domains",    opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, mx_fail_domains)) },
+  { "qualify_single",     opt_bool,
+      (void *)(offsetof(dnslookup_router_options_block, qualify_single)) },
+  { "rewrite_headers",    opt_bool,
+      (void *)(offsetof(dnslookup_router_options_block, rewrite_headers)) },
+  { "same_domain_copy_routing", opt_bool|opt_public,
+      (void *)(offsetof(router_instance, same_domain_copy_routing)) },
+  { "search_parents",     opt_bool,
+      (void *)(offsetof(dnslookup_router_options_block, search_parents)) },
+  { "srv_fail_domains",   opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, srv_fail_domains)) },
+  { "widen_domains",      opt_stringptr,
+      (void *)(offsetof(dnslookup_router_options_block, widen_domains)) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -45,7 +53,7 @@ int dnslookup_router_options_count =
 
 /* Dummy entries */
 dnslookup_router_options_block dnslookup_router_option_defaults = {0};
-void dnslookup_router_init(driver_instance *rblock) {}
+void dnslookup_router_init(router_instance *rblock) {}
 int dnslookup_router_entry(router_instance *rblock, address_item *addr,
   struct passwd *pw, int verify, address_item **addr_local,
   address_item **addr_remote, address_item **addr_new,
@@ -56,12 +64,21 @@ int dnslookup_router_entry(router_instance *rblock, address_item *addr,
 
 
 
-/* Default private options block for the dnslookup router.
-All non-mentioned are NULL/0/FALSE. */
+/* Default private options block for the dnslookup router. */
 
 dnslookup_router_options_block dnslookup_router_option_defaults = {
+  .check_secondary_mx =	FALSE,
   .qualify_single =	TRUE,
+  .search_parents =	FALSE,
   .rewrite_headers =	TRUE,
+  .widen_domains =	NULL,
+  .mx_domains =		NULL,
+  .mx_fail_domains =	NULL,
+  .srv_fail_domains =	NULL,
+  .check_srv =		NULL,
+  .fail_defer_domains =	NULL,
+  .ipv4_only =		NULL,
+  .ipv4_prefer =	NULL,
 };
 
 
@@ -74,7 +91,7 @@ dnslookup_router_options_block dnslookup_router_option_defaults = {
 consistency checks to be done, or anything else that needs to be set up. */
 
 void
-dnslookup_router_init(driver_instance *rblock)
+dnslookup_router_init(router_instance *rblock)
 {
 /*
 dnslookup_router_options_block *ob =
@@ -144,31 +161,36 @@ host_item h;
 int rc;
 int widen_sep = 0;
 int whichrrs = HOST_FIND_BY_MX | HOST_FIND_BY_A | HOST_FIND_BY_AAAA;
-dnslookup_router_options_block * ob =
-  (dnslookup_router_options_block *)(rblock->drinst.options_block);
-const uschar * srv_service = NULL;
-uschar * widen = NULL;
-const uschar * pre_widen = addr->domain, * post_widen = NULL;
-const uschar * fully_qualified_name, * listptr;
+dnslookup_router_options_block *ob =
+  (dnslookup_router_options_block *)(rblock->options_block);
+uschar *srv_service = NULL;
+uschar *widen = NULL;
+const uschar *pre_widen = addr->domain;
+const uschar *post_widen = NULL;
+const uschar *fully_qualified_name;
+const uschar *listptr;
 uschar widen_buffer[256];
 
+addr_new = addr_new;          /* Keep picky compilers happy */
+addr_succeed = addr_succeed;
+
 DEBUG(D_route)
-  debug_printf_indent("%s router called for %s\n  domain = %s\n",
-    rblock->drinst.name, addr->address, addr->domain);
+  debug_printf("%s router called for %s\n  domain = %s\n",
+    rblock->name, addr->address, addr->domain);
 
 /* If an SRV check is required, expand the service name */
 
-GET_OPTION("check_srv");
 if (ob->check_srv)
+  {
   if (  !(srv_service = expand_string(ob->check_srv))
-     && !f.expand_string_forcedfail)
+     && !expand_string_forcedfail)
     {
-    addr->message = string_sprintf("%s router: failed to expand %q: %s",
-      rblock->drinst.name, ob->check_srv, expand_string_message);
+    addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
+      rblock->name, ob->check_srv, expand_string_message);
     return DEFER;
     }
-  else
-    whichrrs |= HOST_FIND_BY_SRV;
+  else whichrrs |= HOST_FIND_BY_SRV;
+  }
 
 /* Set up the first of any widening domains. The code further down copes with
 either pre- or post-widening, but at present there is no way to turn on
@@ -196,7 +218,6 @@ if (  ob->widen_domains
    && (verify != v_sender || !ob->rewrite_headers || addr->parent))
   {
   listptr = ob->widen_domains;
-  /* not expanded so should never be tainted */
   widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
     sizeof(widen_buffer));
 
@@ -226,32 +247,31 @@ for (;;)
   else if (widen)
     {
     h.name = string_sprintf("%s.%s", addr->domain, widen);
-    /* not expanded so should never be tainted */
     widen = string_nextinlist(&listptr, &widen_sep, widen_buffer,
       sizeof(widen_buffer));
-    DEBUG(D_route) debug_printf("%s router widened %s to %s\n",
-      rblock->drinst.name, addr->domain, h.name);
+    DEBUG(D_route) debug_printf("%s router widened %s to %s\n", rblock->name,
+      addr->domain, h.name);
     }
   else if (post_widen)
     {
     h.name = post_widen;
     post_widen = NULL;
     DEBUG(D_route) debug_printf("%s router trying %s after widening failed\n",
-      rblock->drinst.name, h.name);
+      rblock->name, h.name);
     }
   else return DECLINE;
 
   /* Check if we must request only. or prefer, ipv4 */
 
   if (  ob->ipv4_only
-     && expand_check_condition(ob->ipv4_only, rblock->drinst.name, US"router"))
+     && expand_check_condition(ob->ipv4_only, rblock->name, US"router"))
     flags = flags & ~HOST_FIND_BY_AAAA | HOST_FIND_IPV4_ONLY;
-  else if (f.search_find_defer)
+  else if (search_find_defer)
     return DEFER;
   if (  ob->ipv4_prefer
-     && expand_check_condition(ob->ipv4_prefer, rblock->drinst.name, US"router"))
+     && expand_check_condition(ob->ipv4_prefer, rblock->name, US"router"))
     flags |= HOST_FIND_IPV4_FIRST;
-  else if (f.search_find_defer)
+  else if (search_find_defer)
     return DEFER;
 
   /* Set up the rest of the initial host item. Others may get chained on if
@@ -265,12 +285,11 @@ for (;;)
   h.mx = MX_NONE;
   h.status = hstatus_unknown;
   h.why = hwhy_unknown;
-  h.dnssec_used = DS_UNK;
   h.last_try = 0;
 
   /* Unfortunately, we cannot set the mx_only option in advance, because the
   DNS lookup may extend an unqualified name. Therefore, we must do the test
-  subsequently. We use the same logic as that for widen_domains above to avoid
+  stoubsequently. We use the same logic as that for widen_domains above to avoid
   requesting a header rewrite that cannot work. */
 
   if (verify != v_sender || !ob->rewrite_headers || addr->parent)
@@ -279,15 +298,9 @@ for (;;)
     if (ob->search_parents) flags |= HOST_FIND_SEARCH_PARENTS;
     }
 
-  DEBUG(D_route) debug_printf_indent("main lookup for domain\n");
-   {
-    expand_level++;
-    rc = host_find_bydns(&h, CUS rblock->ignore_target_hosts, flags,
-      srv_service, ob->srv_fail_domains, ob->mx_fail_domains,
-      &rblock->dnssec, &fully_qualified_name, &removed);
-    expand_level--;
-   }
-
+  rc = host_find_bydns(&h, CUS rblock->ignore_target_hosts, flags,
+    srv_service, ob->srv_fail_domains, ob->mx_fail_domains,
+    &rblock->dnssec, &fully_qualified_name, &removed);
   if (removed) setflag(addr, af_local_host_removed);
 
   /* If host found with only address records, test for the domain's being in
@@ -306,7 +319,7 @@ for (;;)
 
       case OK:
       DEBUG(D_route) debug_printf("%s router rejected %s: no MX record(s)\n",
-        rblock->drinst.name, fully_qualified_name);
+        rblock->name, fully_qualified_name);
       continue;
       }
 
@@ -323,7 +336,7 @@ for (;;)
     if (rblock->pass_on_timeout)
       {
       DEBUG(D_route) debug_printf("%s router timed out, and pass_on_timeout is set\n",
-        rblock->drinst.name);
+        rblock->name);
       return PASS;
       }
     addr->message = US"host lookup did not complete";
@@ -343,7 +356,7 @@ for (;;)
 
       case OK:
 	DEBUG(D_route) debug_printf("%s router: matched fail_defer_domains\n",
-	  rblock->drinst.name);
+	  rblock->name);
 	addr->message = US"missing MX, or all MXs point to missing A records,"
 	  " and defer requested";
 	return DEFER;
@@ -359,7 +372,7 @@ for (;;)
   As a common cause of this problem is MX records with IP addresses on the
   RHS, give a special message in this case. */
 
-  if (h.mx >= 0 && !h.address)
+  if (h.mx >= 0 && h.address == NULL)
     {
     setflag(addr, af_pass_message);   /* This is not a security risk */
     if (h.name[0] == 0)
@@ -384,9 +397,9 @@ for (;;)
   /* If there's a syntax error, do not continue with any widening, and note
   the error. */
 
-  if (f.host_find_failed_syntax)
+  if (host_find_failed_syntax)
     {
-    addr->message = string_sprintf("mail domain %q is syntactically "
+    addr->message = string_sprintf("mail domain \"%s\" is syntactically "
       "invalid", h.name);
     return DECLINE;
     }
@@ -444,7 +457,7 @@ else if (ob->check_secondary_mx && !testflag(addr, af_local_host_removed))
 rc = rf_get_errors_address(addr, rblock, verify, &addr->prop.errors_address);
 if (rc != OK) return rc;
 
-/* Set up the additional and removable headers for this address. */
+/* Set up the additional and removeable headers for this address. */
 
 rc = rf_get_munge_headers(addr, rblock, &addr->prop.extra_headers,
   &addr->prop.remove_headers);
@@ -453,47 +466,22 @@ if (rc != OK) return rc;
 /* Get store in which to preserve the original host item, chained on
 to the address. */
 
-addr->host_list = store_get(sizeof(host_item), GET_UNTAINTED);
+addr->host_list = store_get(sizeof(host_item));
 addr->host_list[0] = h;
 
 /* Fill in the transport and queue the address for delivery. */
 
 if (!rf_get_transport(rblock->transport_name, &(rblock->transport),
-      addr, rblock->drinst.name, NULL))
+      addr, rblock->name, NULL))
   return DEFER;
 
 addr->transport = rblock->transport;
 
-return rf_queue_add(addr, addr_local, addr_remote, rblock, pw) ?  OK : DEFER;
+return rf_queue_add(addr, addr_local, addr_remote, rblock, pw)?
+  OK : DEFER;
 }
 
-
-
-
-# ifdef DYNLOOKUP
-#  define dnslookup_router_info _router_info
-# endif
-#endif	/*!MACRO_PREDEF*/
-
-router_info dnslookup_router_info =
-{
-.drinfo = {
-  .driver_name =	US"dnslookup",
-  .options =		dnslookup_router_options,
-  .options_count =	&dnslookup_router_options_count,
-  .options_block =	&dnslookup_router_option_defaults,
-  .options_len =	sizeof(dnslookup_router_options_block),
-  .init =		dnslookup_router_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		ROUTER_MAGIC,
-# endif
-  },
-.code =			dnslookup_router_entry,
-.tidyup =		NULL,     /* no tidyup entry */
-.ri_flags =		ri_yestransport
-};
-
-#endif	/*ROUTER_DNSLOOKUP*/
+#endif   /*!MACRO_PREDEF*/
 /* End of routers/dnslookup.c */
 /* vi: aw ai sw=2
 */
