@@ -2,15 +2,11 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
-
-#ifdef ROUTER_MANUALROUTE
 #include "rf_functions.h"
 #include "manualroute.h"
 
@@ -18,21 +14,18 @@
 /* Options specific to the manualroute router. */
 
 optionlist manualroute_router_options[] = {
-  { "*expand_hosts_randomize", opt_stringptr | opt_hidden,
-    OPT_OFF(manualroute_router_options_block, expand_hosts_randomize) },
-
   { "host_all_ignored", opt_stringptr,
-      OPT_OFF(manualroute_router_options_block, host_all_ignored) },
+      (void *)(offsetof(manualroute_router_options_block, host_all_ignored)) },
   { "host_find_failed", opt_stringptr,
-      OPT_OFF(manualroute_router_options_block, host_find_failed) },
-  { "hosts_randomize",  opt_expand_bool,
-      OPT_OFF(manualroute_router_options_block, hosts_randomize) },
+      (void *)(offsetof(manualroute_router_options_block, host_find_failed)) },
+  { "hosts_randomize",  opt_bool,
+      (void *)(offsetof(manualroute_router_options_block, hosts_randomize)) },
   { "route_data",       opt_stringptr,
-      OPT_OFF(manualroute_router_options_block, route_data) },
+      (void *)(offsetof(manualroute_router_options_block, route_data)) },
   { "route_list",       opt_stringptr,
-      OPT_OFF(manualroute_router_options_block, route_list) },
+      (void *)(offsetof(manualroute_router_options_block, route_list)) },
   { "same_domain_copy_routing", opt_bool|opt_public,
-      OPT_OFF(router_instance, same_domain_copy_routing) }
+      (void *)(offsetof(router_instance, same_domain_copy_routing)) }
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -46,7 +39,7 @@ int manualroute_router_options_count =
 
 /* Dummy entries */
 manualroute_router_options_block manualroute_router_option_defaults = {0};
-void manualroute_router_init(driver_instance *rblock) {}
+void manualroute_router_init(router_instance *rblock) {}
 int manualroute_router_entry(router_instance *rblock, address_item *addr,
   struct passwd *pw, int verify, address_item **addr_local,
   address_item **addr_remote, address_item **addr_new,
@@ -56,14 +49,16 @@ int manualroute_router_entry(router_instance *rblock, address_item *addr,
 
 
 
-/* Default private options block for the manualroute router.
-All other entries 0/FALSE/NULL */
+/* Default private options block for the manualroute router. */
 
 manualroute_router_options_block manualroute_router_option_defaults = {
-  .hai_code = -1,			/* host_all_ignored code (unset) */
-  .hff_code = -1,			/* host_find_failed code (unset) */
-  .host_all_ignored =	US"defer",
-  .host_find_failed =	US"freeze",
+  -1,           /* host_all_ignored code (unset) */
+  -1,           /* host_find_failed code (unset) */
+  FALSE,        /* hosts_randomize */
+  US"defer",    /* host_all_ignored */
+  US"freeze",   /* host_find_failed */
+  NULL,         /* route_data */
+  NULL          /* route_list */
 };
 
 
@@ -92,38 +87,43 @@ static int hff_count= sizeof(hff_codes)/sizeof(int);
 consistency checks to be done, or anything else that needs to be set up. */
 
 void
-manualroute_router_init(driver_instance * rblock)
+manualroute_router_init(router_instance *rblock)
 {
-manualroute_router_options_block * ob =
-  (manualroute_router_options_block *) rblock->options_block;
+manualroute_router_options_block *ob =
+  (manualroute_router_options_block *)(rblock->options_block);
+int i;
 
 /* Host_find_failed must be a recognized word */
 
-for (int i = 0; i < hff_count; i++)
+for (i = 0; i < hff_count; i++)
+  {
   if (Ustrcmp(ob->host_find_failed, hff_names[i]) == 0)
     {
     ob->hff_code = hff_codes[i];
     break;
     }
+  }
 if (ob->hff_code < 0)
-  log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
     "unrecognized setting for host_find_failed option", rblock->name);
 
-for (int i = 1; i < hff_count; i++)   /* NB starts at 1 to skip "ignore" */
+for (i = 1; i < hff_count; i++)   /* NB starts at 1 to skip "ignore" */
+  {
   if (Ustrcmp(ob->host_all_ignored, hff_names[i]) == 0)
     {
     ob->hai_code = hff_codes[i];
     break;
     }
+  }
 if (ob->hai_code < 0)
-  log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
     "unrecognized setting for host_all_ignored option", rblock->name);
 
 /* One of route_list or route_data must be specified */
 
-if (  !ob->route_list && !ob->route_data
-   || ob->route_list && ob->route_data)
-  log_write_die(0, LOG_CONFIG_FOR, "%s router:\n  "
+if ((ob->route_list == NULL && ob->route_data == NULL) ||
+    (ob->route_list != NULL && ob->route_data != NULL))
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
     "route_list or route_data (but not both) must be specified",
     rblock->name);
 }
@@ -157,17 +157,17 @@ static BOOL
 parse_route_item(const uschar *s, const uschar **domain, const uschar **hostlist,
   const uschar **options)
 {
-Uskip_whitespace(&s);
+while (*s != 0 && isspace(*s)) s++;
 
-if (domain)
+if (domain != NULL)
   {
-  if (!*s) return FALSE;            /* missing data */
+  if (*s == 0) return FALSE;            /* missing data */
   *domain = string_dequote(&s);
-  Uskip_whitespace(&s);
+  while (*s != 0 && isspace(*s)) s++;
   }
 
 *hostlist = string_dequote(&s);
-Uskip_whitespace(&s);
+while (*s != 0 && isspace(*s)) s++;
 *options = s;
 return TRUE;
 }
@@ -241,25 +241,32 @@ manualroute_router_entry(
 {
 int rc, lookup_type;
 uschar *route_item = NULL;
-const uschar * options = NULL, * hostlist = NULL, * domain, * listptr;
-uschar * newhostlist;
-manualroute_router_options_block * ob =
-  (manualroute_router_options_block *)(rblock->drinst.options_block);
-transport_instance * transport = NULL;
-BOOL individual_transport_set = FALSE, randomize;
+const uschar *options = NULL;
+const uschar *hostlist = NULL;
+const uschar *domain;
+uschar *newhostlist;
+const uschar *listptr;
+manualroute_router_options_block *ob =
+  (manualroute_router_options_block *)(rblock->options_block);
+transport_instance *transport = NULL;
+BOOL individual_transport_set = FALSE;
+BOOL randomize = ob->hosts_randomize;
 
-DEBUG(D_route) debug_printf_indent("%s router called for %s\n  domain = %s\n",
-  rblock->drinst.name, addr->address, addr->domain);
+addr_new = addr_new;           /* Keep picky compilers happy */
+addr_succeed = addr_succeed;
+
+DEBUG(D_route) debug_printf("%s router called for %s\n  domain = %s\n",
+  rblock->name, addr->address, addr->domain);
 
 /* The initialization check ensures that either route_list or route_data is
 set. */
 
-if (ob->route_list)
+if (ob->route_list != NULL)
   {
   int sep = -(';');             /* Default is semicolon */
   listptr = ob->route_list;
 
-  while ((route_item = string_nextinlist(&listptr, &sep, NULL, 0)))
+  while ((route_item = string_nextinlist(&listptr, &sep, NULL, 0)) != NULL)
     {
     int rc;
 
@@ -282,7 +289,7 @@ if (ob->route_list)
       }
     }
 
-  if (!route_item) return DECLINE;  /* No pattern in the list matched */
+  if (route_item == NULL) return DECLINE;  /* No pattern in the list matched */
   }
 
 /* Handle a single routing item in route_data. If it expands to an empty
@@ -290,18 +297,17 @@ string, decline. */
 
 else
   {
-  GET_OPTION("route_data");
-  if (!(route_item = rf_expand_data(addr, ob->route_data, &rc)))
-    return rc;
+  route_item = rf_expand_data(addr, ob->route_data, &rc);
+  if (route_item == NULL) return rc;
   (void) parse_route_item(route_item, NULL, &hostlist, &options);
-  if (!hostlist[0]) return DECLINE;
+  if (hostlist[0] == 0) return DECLINE;
   }
 
 /* Expand the hostlist item. It may then pointing to an empty string, or to a
 single host or a list of hosts; options is pointing to the rest of the
 routelist item, which is either empty or contains various option words. */
 
-DEBUG(D_route) debug_printf_indent("original list of hosts = '%s' options = '%s'\n",
+DEBUG(D_route) debug_printf("original list of hosts = \"%s\" options = %s\n",
   hostlist, options);
 
 newhostlist = expand_string_copy(hostlist);
@@ -311,36 +317,28 @@ expand_nmax = -1;
 /* If the expansion was forced to fail, just decline. Otherwise there is a
 configuration problem. */
 
-if (!newhostlist)
+if (newhostlist == NULL)
   {
-  if (f.expand_string_forcedfail) return DECLINE;
-  addr->message = string_sprintf("%s router: failed to expand %q: %s",
-    rblock->drinst.name, hostlist, expand_string_message);
+  if (expand_string_forcedfail) return DECLINE;
+  addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
+    rblock->name, hostlist, expand_string_message);
   return DEFER;
   }
 else hostlist = newhostlist;
 
-DEBUG(D_route) debug_printf_indent("expanded list of hosts = '%s' options = '%s'\n",
+DEBUG(D_route) debug_printf("expanded list of hosts = \"%s\" options = %s\n",
   hostlist, options);
 
-/* Get the hosts_randomize router option, expanding if needed */
-
-if (exp_bool(addr, US"router", rblock->drinst.name, D_route,
-	  US"hosts_randomize", ob->hosts_randomize, ob->expand_hosts_randomize,
-	  &randomize) != OK)
-  return DEFER;
-
-/* Set default lookup type and scan the route-data options */
+/* Set default lookup type and scan the options */
 
 lookup_type = LK_DEFAULT;
 
-while (*options)
+while (*options != 0)
   {
   unsigned n;
-  const uschar * s = options;
-
-  Uskip_nonwhite(&options);
-  n = options - s;
+  const uschar *s = options;
+  while (*options != 0 && !isspace(*options)) options++;
+  n = options-s;
 
   if (Ustrncmp(s, "randomize", n) == 0) randomize = TRUE;
   else if (Ustrncmp(s, "no_randomize", n) == 0) randomize = FALSE;
@@ -352,9 +350,9 @@ while (*options)
   else if (Ustrncmp(s, "ipv4_only",   n) == 0) lookup_type |= LK_IPV4_ONLY;
   else
     {
-    transport_instance * t;
-    for (t = transports; t; t = t->drinst.next)
-      if (Ustrncmp(t->drinst.name, s, n) == 0)
+    transport_instance *t;
+    for (t = transports; t; t = t->next)
+      if (Ustrncmp(t->name, s, n) == 0)
         {
         transport = t;
         individual_transport_set = TRUE;
@@ -363,8 +361,8 @@ while (*options)
 
     if (!t)
       {
-      s = string_sprintf("unknown routing option or transport name %q", s);
-      log_write(0, LOG_MAIN, "Error in %s router: %s", rblock->drinst.name, s);
+      s = string_sprintf("unknown routing option or transport name \"%s\"", s);
+      log_write(0, LOG_MAIN, "Error in %s router: %s", rblock->name, s);
       addr->message = string_sprintf("error in router: %s", s);
       return DEFER;
       }
@@ -373,7 +371,7 @@ while (*options)
   if (*options)
     {
     options++;
-    Uskip_whitespace(&options);
+    while (*options != 0 && isspace(*options)) options++;
     }
   }
 
@@ -382,7 +380,7 @@ while (*options)
 rc = rf_get_errors_address(addr, rblock, verify, &addr->prop.errors_address);
 if (rc != OK) return rc;
 
-/* Set up the additional and removable headers for this address. */
+/* Set up the additional and removeable headers for this address. */
 
 rc = rf_get_munge_headers(addr, rblock, &addr->prop.extra_headers,
   &addr->prop.remove_headers);
@@ -394,8 +392,8 @@ set. */
 
 if (!individual_transport_set)
   {
-  if (!rf_get_transport(rblock->transport_name, &rblock->transport, addr,
-      rblock->drinst.name, NULL))
+  if (!rf_get_transport(rblock->transport_name, &(rblock->transport), addr,
+      rblock->name, NULL))
     return DEFER;
   transport = rblock->transport;
   }
@@ -403,44 +401,40 @@ if (!individual_transport_set)
 /* Deal with the case of a local transport. The host list is passed over as a
 single text string that ends up in $host. */
 
-if (transport)
+if (transport && transport->info->local)
   {
-  const transport_info * ti = transport->drinst.info;
-  if (ti->local)
+  if (hostlist[0] != 0)
     {
-    if (hostlist[0])
-      {
-      host_item * h = store_get(sizeof(host_item), GET_UNTAINTED);
-      h->name = string_copy(hostlist);
-      h->address = NULL;
-      h->port = PORT_NONE;
-      h->mx = MX_NONE;
-      h->status = hstatus_unknown;
-      h->why = hwhy_unknown;
-      h->dnssec_used = DS_UNK;
-      h->last_try = 0;
-      h->next = NULL;
-      addr->host_list = h;
-      }
-
-    /* There is nothing more to do other than to queue the address for the
-    local transport, filling in any uid/gid. This can be done by the common
-    rf_queue_add() function. */
-
-    addr->transport = transport;
-    return rf_queue_add(addr, addr_local, addr_remote, rblock, pw) ? OK : DEFER;
+    host_item *h;
+    addr->host_list = h = store_get(sizeof(host_item));
+    h->name = string_copy(hostlist);
+    h->address = NULL;
+    h->port = PORT_NONE;
+    h->mx = MX_NONE;
+    h->status = hstatus_unknown;
+    h->why = hwhy_unknown;
+    h->last_try = 0;
+    h->next = NULL;
     }
+
+  /* There is nothing more to do other than to queue the address for the
+  local transport, filling in any uid/gid. This can be done by the common
+  rf_queue_add() function. */
+
+  addr->transport = transport;
+  return rf_queue_add(addr, addr_local, addr_remote, rblock, pw)?
+    OK : DEFER;
   }
 
 /* There is either no transport (verify_only) or a remote transport. A host
 list is mandatory in either case, except when verifying, in which case the
 address is just accepted. */
 
-if (!hostlist[0])
+if (hostlist[0] == 0)
   {
   if (verify != v_none) goto ROUTED;
   addr->message = string_sprintf("error in %s router: no host(s) specified "
-    "for domain %s", rblock->drinst.name, addr->domain);
+    "for domain %s", rblock->name, addr->domain);
   log_write(0, LOG_MAIN, "%s", addr->message);
   return DEFER;
   }
@@ -448,7 +442,7 @@ if (!hostlist[0])
 /* Otherwise we finish the routing here by building a chain of host items
 for the list of configured hosts, and then finding their addresses. */
 
-host_build_hostlist(&addr->host_list, hostlist, randomize);
+host_build_hostlist(&(addr->host_list), hostlist, randomize);
 rc = rf_lookup_hostlist(rblock, addr, rblock->ignore_target_hosts, lookup_type,
   ob->hff_code, addr_new);
 if (rc != OK) return rc;
@@ -468,8 +462,7 @@ if (!addr->host_list)
     if (ob->hai_code == hff_codes[i]) break;
 
   addr->message = string_sprintf("lookup failed for all hosts in %s router: "
-    "host_find_failed=ignore host_all_ignored=%s",
-    rblock->drinst.name, hff_names[i]);
+    "host_find_failed=ignore host_all_ignored=%s", rblock->name, hff_names[i]);
 
   if (ob->hai_code == hff_defer) return DEFER;
   if (ob->hai_code == hff_fail) return FAIL;
@@ -482,10 +475,10 @@ if (!addr->host_list)
 defined for these hosts. It will be a remote one, as a local transport is
 dealt with above. However, we don't need one if verifying only. */
 
-if (!transport && verify == v_none)
+if (transport == NULL && verify == v_none)
     {
     log_write(0, LOG_MAIN, "Error in %s router: no transport defined",
-      rblock->drinst.name);
+      rblock->name);
     addr->message = US"error in router: transport missing";
     return DEFER;
     }
@@ -500,31 +493,5 @@ addr->transport = transport;
 return OK;
 }
 
-
-
-
-# ifdef DYNLOOKUP
-#  define manualroute_router_info _router_info
-# endif
-#endif	/*!MACRO_PREDEF*/
-
-router_info manualroute_router_info =
-{
-.drinfo = {
-  .driver_name =	US"manualroute",
-  .options =		manualroute_router_options,
-  .options_count =	&manualroute_router_options_count,
-  .options_block =	&manualroute_router_option_defaults,
-  .options_len =	sizeof(manualroute_router_options_block),
-  .init =		manualroute_router_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		ROUTER_MAGIC,
-# endif
-  },
-.code =			manualroute_router_entry,
-.tidyup =		NULL,     /* no tidyup entry */
-.ri_flags =		0
-};
-
-#endif	/*ROUTER_MANUALROUTE*/
+#endif   /*!MACRO_PREDEF*/
 /* End of routers/manualroute.c */
