@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2021 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "../exim.h"
 #include "rf_functions.h"
@@ -36,22 +34,21 @@ rf_get_munge_headers(address_item *addr, router_instance *rblock,
 /* Default is to retain existing headers */
 *extra_headers = addr->prop.extra_headers;
 
-GET_OPTION("headers_add");
 if (rblock->extra_headers)
   {
   const uschar * list = rblock->extra_headers;
   int sep = '\n';
-  uschar * s, * t;
+  uschar * s;
   int slen;
 
   while ((s = string_nextinlist(&list, &sep, NULL, 0)))
-    if (!(s = expand_string(t = s)))
+    if (!(s = expand_string(s)))
       {
-      if (!f.expand_string_forcedfail)
+      if (!expand_string_forcedfail)
 	{
 	addr->message = string_sprintf(
-	  "%s router failed to expand add_headers item %q: %s",
-	  rblock->drinst.name, t, expand_string_message);
+	  "%s router failed to expand add_headers item \"%s\": %s",
+	  rblock->name, s, expand_string_message);
 	return DEFER;
 	}
       }
@@ -62,7 +59,7 @@ if (rblock->extra_headers)
       shared with other addresses. The output function outputs them in reverse
       order. */
 
-      header_line *  h = store_get(sizeof(header_line), GET_UNTAINTED);
+      header_line *  h = store_get(sizeof(header_line));
 
       /* We used to use string_sprintf() to add the newline if needed, but that
       causes problems if the header line is exceedingly long (e.g. adding
@@ -72,7 +69,7 @@ if (rblock->extra_headers)
 	h->text = s;
       else
 	{
-	h->text = store_get(slen+2, s);
+	h->text = store_get(slen+2);
 	memcpy(h->text, s, slen);
 	h->text[slen++] = '\n';
 	h->text[slen] = 0;
@@ -89,33 +86,31 @@ if (rblock->extra_headers)
 *remove_headers = addr->prop.remove_headers;
 
 /* Expand items from colon-sep list separately, then build new list */
-GET_OPTION("headers_remove");
 if (rblock->remove_headers)
   {
   const uschar * list = rblock->remove_headers;
   int sep = ':';
-  uschar * s, * t;
+  uschar * s;
   gstring * g = NULL;
 
   if (*remove_headers)
     g = string_cat(NULL, *remove_headers);
 
   while ((s = string_nextinlist(&list, &sep, NULL, 0)))
-    if (!(s = expand_string(t = s)))
+    if (!(s = expand_string(s)))
       {
-      if (!f.expand_string_forcedfail)
+      if (!expand_string_forcedfail)
 	{
 	addr->message = string_sprintf(
-	  "%s router failed to expand remove_headers item %q: %s",
-	  rblock->drinst.name, t, expand_string_message);
+	  "%s router failed to expand remove_headers item \"%s\": %s",
+	  rblock->name, s, expand_string_message);
 	return DEFER;
 	}
       }
     else if (*s)
       g = string_append_listele(g, ':', s);
-
-  if (g)
-    *remove_headers = g->s;
+    if (g)
+      *remove_headers = g->s;
   }
 
 return OK;
