@@ -2,10 +2,8 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2024 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Functions for reading spool files. When compiling for a utility (eximon),
 not all are needed, and some functionality can be cut out. */
@@ -36,50 +34,44 @@ Side effect: message_subdir is set for the (possibly split) spool directory
 */
 
 int
-spool_open_datafile(const uschar * id)
+spool_open_datafile(uschar *id)
 {
+int i;
 struct stat statbuf;
 flock_t lock_data;
 int fd;
 
-/* If split_spool_directory is set (handled by set_subdir_str()), first look for
-the file in the appropriate sub-directory of the input directory. If it is not
-found there, try the input directory itself, to pick up leftovers from before
-the splitting. If split_ spool_directory is not set, first look in the main
-input directory. If it is not found there, try the split sub-directory, in case
-it is left over from a splitting state. */
+/* If split_spool_directory is set, first look for the file in the appropriate
+sub-directory of the input directory. If it is not found there, try the input
+directory itself, to pick up leftovers from before the splitting. If split_
+spool_directory is not set, first look in the main input directory. If it is
+not found there, try the split sub-directory, in case it is left over from a
+splitting state. */
 
-for (int i = 0; i < 2; i++)
+for (i = 0; i < 2; i++)
   {
   uschar * fname;
   int save_errno;
 
-  set_subdir_str(message_subdir, id, i);
+  message_subdir[0] = split_spool_directory == i ? '\0' : id[5];
   fname = spool_fname(US"input", message_subdir, id, US"-D");
-  DEBUG(D_deliver) debug_printf_indent("Trying spool file %s\n", fname);
-
-  /* We protect against symlink attacks both in not propagating the
-  file-descriptor to other processes as we exec, and also ensuring that we
-  don't even open symlinks.
-  No -D file inside the spool area should be a symlink.  */
+  DEBUG(D_deliver) debug_printf("Trying spool file %s\n", fname);
 
   if ((fd = Uopen(fname,
-		  EXIM_CLOEXEC | EXIM_NOFOLLOW | O_RDWR | O_APPEND, 0)) >= 0)
+#ifdef O_CLOEXEC
+		      O_CLOEXEC |
+#endif
+		      O_RDWR | O_APPEND, 0)) >= 0)
     break;
   save_errno = errno;
   if (errno == ENOENT)
     {
     if (i == 0) continue;
-    if (!f.queue_running)
+    if (!queue_running)
       log_write(0, LOG_MAIN, "Spool%s%s file %s-D not found",
 	*queue_name ? US" Q=" : US"",
 	*queue_name ? queue_name : US"",
 	id);
-    else DEBUG(D_deliver)
-      debug_printf("Spool%s%s file %s-D not found\n",
-	*queue_name ? US" Q=" : US"",
-	*queue_name ? queue_name : US"",
-	id);
     }
   else
     log_write(0, LOG_MAIN, "Spool error for %s: %s", fname, strerror(errno));
@@ -102,13 +94,13 @@ what it locks. */
 lock_data.l_type = F_WRLCK;
 lock_data.l_whence = SEEK_SET;
 lock_data.l_start = 0;
-lock_data.l_len = spool_data_start_offset(id);
+lock_data.l_len = SPOOL_DATA_START_OFFSET;
 
 if (fcntl(fd, F_SETLK, &lock_data) < 0)
   {
-  log_write(L_skip_delivery, LOG_MAIN,
-      "Spool file for %s is locked (another process is handling this message)",
-      id);
+  log_write(L_skip_delivery,
+            LOG_MAIN,
+            "Spool file is locked (another process is handling this message)");
   (void)close(fd);
   errno = 0;
   return -1;
@@ -119,7 +111,7 @@ in the count, but add one for the newline before the data. */
 
 if (fstat(fd, &statbuf) == 0)
   {
-  message_body_size = statbuf.st_size - spool_data_start_offset(id);
+  message_body_size = statbuf.st_size - SPOOL_DATA_START_OFFSET;
   message_size = message_body_size + 1;
   }
 
@@ -186,7 +178,7 @@ BOOL right = buffer[1] == 'Y';
 
 if (n < 5) return FALSE;    /* malformed line */
 buffer[n-1] = 0;            /* Remove \n */
-node = store_get(sizeof(tree_node) + n - 3, GET_TAINTED);	/* rcpt names tainted */
+node = store_get(sizeof(tree_node) + n - 3);
 *connect = node;
 Ustrcpy(node->name, buffer + 3);
 node->data.ptr = NULL;
@@ -224,24 +216,22 @@ spool_clear_header_globals(void)
 acl_var_c = acl_var_m = NULL;
 authenticated_id = NULL;
 authenticated_sender = NULL;
-f.allow_unqualified_recipient = FALSE;
-f.allow_unqualified_sender = FALSE;
+allow_unqualified_recipient = FALSE;
+allow_unqualified_sender = FALSE;
 body_linecount = 0;
 body_zerocount = 0;
-f.deliver_firsttime = FALSE;
-f.deliver_freeze = FALSE;
+deliver_firsttime = FALSE;
+deliver_freeze = FALSE;
 deliver_frozen_at = 0;
-f.deliver_manual_thaw = FALSE;
-/* f.dont_deliver must NOT be reset */
+deliver_manual_thaw = FALSE;
+/* dont_deliver must NOT be reset */
 header_list = header_last = NULL;
 host_lookup_deferred = FALSE;
 host_lookup_failed = FALSE;
 interface_address = NULL;
 interface_port = 0;
-f.local_error_message = FALSE;
-#ifdef HAVE_LOCAL_SCAN
+local_error_message = FALSE;
 local_scan_data = NULL;
-#endif
 max_received_linelength = 0;
 message_linecount = 0;
 received_protocol = NULL;
@@ -253,13 +243,13 @@ sender_helo_name = NULL;
 sender_host_address = NULL;
 sender_host_name = NULL;
 sender_host_port = 0;
-sender_host_authenticated = sender_host_auth_pubname = NULL;
+sender_host_authenticated = NULL;
 sender_ident = NULL;
-f.sender_local = FALSE;
-f.sender_set_untrusted = FALSE;
+sender_local = FALSE;
+sender_set_untrusted = FALSE;
 smtp_active_hostname = primary_hostname;
 #ifndef COMPILE_UTILITY
-f.spool_file_wireformat = FALSE;
+spool_file_wireformat = FALSE;
 #endif
 tree_nonrecipients = NULL;
 
@@ -270,16 +260,16 @@ bmi_verdicts = NULL;
 
 #ifndef DISABLE_DKIM
 dkim_signers = NULL;
-f.dkim_disable_verify = FALSE;
-dkim_collect_input = 0;
+dkim_disable_verify = FALSE;
+dkim_collect_input = FALSE;
 #endif
 
-#ifndef DISABLE_TLS
+#ifdef SUPPORT_TLS
 tls_in.certificate_verified = FALSE;
 # ifdef SUPPORT_DANE
 tls_in.dane_verified = FALSE;
 # endif
-tls_in.ver = tls_in.cipher = NULL;
+tls_in.cipher = NULL;
 # ifndef COMPILE_UTILITY	/* tls support fns not built in */
 tls_free_cert(&tls_in.ourcert);
 tls_free_cert(&tls_in.peercert);
@@ -300,42 +290,10 @@ message_smtputf8 = FALSE;
 message_utf8_downconvert = 0;
 #endif
 
-#ifndef COMPILE_UTILITY
-debuglog_name[0] = '\0';
-#endif
 dsn_ret = 0;
 dsn_envid = NULL;
 }
 
-static void *
-fgets_big_buffer(FILE *fp)
-{
-int len = 0;
-
-big_buffer[0] = 0;
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) return NULL;
-
-while ((len = Ustrlen(big_buffer)) == big_buffer_size-1
-      && big_buffer[len-1] != '\n')
-  {
-  uschar *newbuffer;
-  int newsize;
-
-  if (big_buffer_size >= BIG_BUFFER_SIZE * 4) return NULL;
-  newsize = big_buffer_size * 2;
-  newbuffer = store_get_perm(newsize, FALSE);
-  memcpy(newbuffer, big_buffer, len);
-
-  big_buffer = newbuffer;
-  big_buffer_size = newsize;
-  if (Ufgets(big_buffer + len, big_buffer_size - len, fp) == NULL) return NULL;
-  }
-
-if (len <= 0 || big_buffer[len-1] != '\n') return NULL;
-return big_buffer;
-}
-
-
 
 /*************************************************
 *             Read spool header file             *
@@ -369,12 +327,12 @@ Returns:        spool_read_OK        success
 int
 spool_read_header(uschar *name, BOOL read_headers, BOOL subdir_set)
 {
-FILE * fp = NULL;
+FILE *f = NULL;
 int n;
 int rcount = 0;
 long int uid, gid;
 BOOL inheader = FALSE;
-const uschar * where;
+uschar *p;
 
 /* Reset all the global variables to their default values. However, there is
 one exception. DO NOT change the default value of dont_deliver, because it may
@@ -386,12 +344,12 @@ spool_clear_header_globals();
 set, just look in the given directory. Otherwise, look in both the split
 and unsplit directories, as for the data file above. */
 
-for (int n = 0; n < 2; n++)
+for (n = 0; n < 2; n++)
   {
   if (!subdir_set)
-    set_subdir_str(message_subdir, name, n);
+    message_subdir[0] = split_spool_directory == (n == 0) ? name[5] : 0;
 
-  if ((fp = Ufopen(spool_fname(US"input", message_subdir, name, US""), "rb")))
+  if ((f = Ufopen(spool_fname(US"input", message_subdir, name, US""), "rb")))
     break;
   if (n != 0 || subdir_set || errno != ENOENT)
     return spool_read_notopen;
@@ -400,21 +358,15 @@ for (int n = 0; n < 2; n++)
 errno = 0;
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_deliver) debug_printf_indent("reading spool file %s\n", name);
+DEBUG(D_deliver) debug_printf("reading spool file %s\n", name);
 #endif  /* COMPILE_UTILITY */
 
 /* The first line of a spool file contains the message id followed by -H (i.e.
 the file name), in order to make the file self-identifying. */
 
-where = US"first line read";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
-where = US"first line length";
-if (  (  Ustrlen(big_buffer) != MESSAGE_ID_LENGTH + 3
-      && Ustrlen(big_buffer) != MESSAGE_ID_LENGTH_OLD + 3
-      )
-   || (  Ustrncmp(big_buffer, name, MESSAGE_ID_LENGTH + 2) != 0
-      && Ustrncmp(big_buffer, name, MESSAGE_ID_LENGTH_OLD + 2) != 0
-   )  )
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
+if (Ustrlen(big_buffer) != MESSAGE_ID_LENGTH + 3 ||
+    Ustrncmp(big_buffer, name, MESSAGE_ID_LENGTH + 2) != 0)
   goto SPOOL_FORMAT_ERROR;
 
 /* The next three lines in the header file are in a fixed format. The first
@@ -424,64 +376,49 @@ negative uids and gids. The second contains the mail address of the message's
 sender, enclosed in <>. The third contains the time the message was received,
 and the number of warning messages for delivery delays that have been sent. */
 
-where = US"2nd line read";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 
- {
-  uschar *p = big_buffer + Ustrlen(big_buffer);
-  while (p > big_buffer && isspace(p[-1])) p--;
-  *p = 0;
-  where = US"2nd line fmt 1";
-  if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
-  while (p > big_buffer && (isdigit(p[-1]) || '-' == p[-1])) p--;
-  gid = Uatoi(p);
-  where = US"2nd line fmt 2";
-  if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
-  *p = 0;
-  if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
-  while (p > big_buffer && (isdigit(p[-1]) || '-' == p[-1])) p--;
-  uid = Uatoi(p);
-  where = US"2nd line fmt 3";
-  if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
-  *p = 0;
- }
+p = big_buffer + Ustrlen(big_buffer);
+while (p > big_buffer && isspace(p[-1])) p--;
+*p = 0;
+if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
+while (p > big_buffer && (isdigit(p[-1]) || '-' == p[-1])) p--;
+gid = Uatoi(p);
+if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
+*p = 0;
+if (!isdigit(p[-1])) goto SPOOL_FORMAT_ERROR;
+while (p > big_buffer && (isdigit(p[-1]) || '-' == p[-1])) p--;
+uid = Uatoi(p);
+if (p <= big_buffer || *(--p) != ' ') goto SPOOL_FORMAT_ERROR;
+*p = 0;
 
 originator_login = string_copy(big_buffer);
 originator_uid = (uid_t)uid;
 originator_gid = (gid_t)gid;
 
-where = US"envelope from";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
+/* envelope from */
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 n = Ustrlen(big_buffer);
 if (n < 3 || big_buffer[0] != '<' || big_buffer[n-2] != '>')
   goto SPOOL_FORMAT_ERROR;
 
- {
-  uschar * s = store_get(n-2, GET_TAINTED);
-  Ustrncpy(s, big_buffer+1, n-3);
-  s[n-3] = '\0';
-  sender_address = s;
- }
+sender_address = store_get(n-2);
+Ustrncpy(sender_address, big_buffer+1, n-3);
+sender_address[n-3] = 0;
 
-where = US"time";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
+/* time */
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (sscanf(CS big_buffer, TIME_T_FMT " %d", &received_time.tv_sec, &warning_count) != 2)
   goto SPOOL_FORMAT_ERROR;
 received_time.tv_usec = 0;
-received_time_complete = received_time;
-
 
 message_age = time(NULL) - received_time.tv_sec;
-#ifndef COMPILE_UTILITY
-if (f.running_in_test_harness)
-  message_age = test_harness_fudged_queue_time(message_age);
-#endif
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_deliver) debug_printf_indent("user=%s uid=%ld gid=%ld sender=%s\n",
+DEBUG(D_deliver) debug_printf("user=%s uid=%ld gid=%ld sender=%s\n",
   originator_login, (long int)originator_uid, (long int)originator_gid,
   sender_address);
-#endif
+#endif  /* COMPILE_UTILITY */
 
 /* Now there may be a number of optional lines, each starting with "-". If you
 add a new setting here, make sure you set the default above.
@@ -494,46 +431,29 @@ by not re-scanning the first two characters.
 To allow new versions of Exim that add additional flags to interwork with older
 versions that do not understand them, just ignore any lines starting with "-"
 that we don't recognize. Otherwise it wouldn't be possible to back off a new
-version that left new-style flags written on the spool.
-
-If the line starts with "--" the content of the variable is tainted.
-If the line start "--(<lookuptype>)" it is also quoted for the given <lookuptype>.
-*/
+version that left new-style flags written on the spool. */
 
+p = big_buffer + 2;
 for (;;)
   {
-  const void * proto_mem;
-  uschar * var;
-  const uschar * p;
-
-  if (fgets_big_buffer(fp) == NULL) goto SPOOL_READ_ERROR;
+  int len;
+  if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
   if (big_buffer[0] != '-') break;
-  big_buffer[Ustrlen(big_buffer)-1] = 0;
-
-  proto_mem = big_buffer[1] == '-' ? GET_TAINTED : GET_UNTAINTED;
-  var =  big_buffer + (proto_mem == GET_UNTAINTED ? 1 : 2);
-  if (*var == '(')				/* marker for quoted value */
-    {
-    uschar * s;
-    for (s = ++var; *s != ')'; ) s++;
-#ifndef COMPILE_UTILITY
-      {
-      int idx;
-      if ((idx = search_findtype(var, s - var)) < 0)
-	{
-	DEBUG(D_any)
-	  debug_printf("Unrecognised quoter %.*s\n", (int)(s - var), var+1);
-	where = NULL;
-	goto SPOOL_FORMAT_ERROR;
-	}
-      proto_mem = store_get_quoted(1, GET_TAINTED, idx);
-      }
-#endif  /* COMPILE_UTILITY */
-    var = s + 1;
+  while (  (len = Ustrlen(big_buffer)) == big_buffer_size-1
+	&& big_buffer[len-1] != '\n'
+	)
+    {	/* buffer not big enough for line; certs make this possible */
+    uschar * buf;
+    if (big_buffer_size >= BIG_BUFFER_SIZE*4) goto SPOOL_READ_ERROR;
+    buf = store_get_perm(big_buffer_size *= 2);
+    memcpy(buf, big_buffer, --len);
+    big_buffer = buf;
+    if (Ufgets(big_buffer+len, big_buffer_size-len, f) == NULL)
+      goto SPOOL_READ_ERROR;
     }
-  p = var + 1;
+  big_buffer[len-1] = 0;
 
-  switch(*var)
+  switch(big_buffer[1])
     {
     case 'a':
 
@@ -548,29 +468,28 @@ for (;;)
       uschar *name, *endptr;
       int count;
       tree_node *node;
-      endptr = Ustrchr(var + 5, ' ');
-      where = US"-aclXn";
-      if (!endptr) goto SPOOL_FORMAT_ERROR;
-      name = string_sprintf("%c%.*s", var[3],
-        (int)(endptr - var - 5), var + 5);
+      endptr = Ustrchr(big_buffer + 6, ' ');
+      if (endptr == NULL) goto SPOOL_FORMAT_ERROR;
+      name = string_sprintf("%c%.*s", big_buffer[4],
+        (int)(endptr - big_buffer - 6), big_buffer + 6);
       if (sscanf(CS endptr, " %d", &count) != 1) goto SPOOL_FORMAT_ERROR;
       node = acl_var_create(name);
-      node->data.ptr = store_get(count + 1, proto_mem);
-      if (fread(node->data.ptr, 1, count+1, fp) < count) goto SPOOL_READ_ERROR;
+      node->data.ptr = store_get(count + 1);
+      if (fread(node->data.ptr, 1, count+1, f) < count) goto SPOOL_READ_ERROR;
       ((uschar*)node->data.ptr)[count] = 0;
       }
 
     else if (Ustrcmp(p, "llow_unqualified_recipient") == 0)
-      f.allow_unqualified_recipient = TRUE;
+      allow_unqualified_recipient = TRUE;
     else if (Ustrcmp(p, "llow_unqualified_sender") == 0)
-      f.allow_unqualified_sender = TRUE;
+      allow_unqualified_sender = TRUE;
 
     else if (Ustrncmp(p, "uth_id", 6) == 0)
-      authenticated_id = string_copy_taint(var + 8, proto_mem);
+      authenticated_id = string_copy(big_buffer + 9);
     else if (Ustrncmp(p, "uth_sender", 10) == 0)
-      authenticated_sender = string_copy_taint(var + 12, proto_mem);
+      authenticated_sender = string_copy(big_buffer + 13);
     else if (Ustrncmp(p, "ctive_hostname", 14) == 0)
-      smtp_active_hostname = string_copy_taint(var + 16, proto_mem);
+      smtp_active_hostname = string_copy(big_buffer + 17);
 
     /* For long-term backward compatibility, we recognize "-acl", which was
     used before the number of ACL variables changed from 10 to 20. This was
@@ -584,8 +503,7 @@ for (;;)
       unsigned index, count;
       uschar name[20];   /* Need plenty of space for %u format */
       tree_node * node;
-      where = US"-acl (old)";
-      if (  sscanf(CS var + 4, "%u %u", &index, &count) != 2
+      if (  sscanf(CS big_buffer + 5, "%u %u", &index, &count) != 2
 	 || index >= 20
 	 || count > 16384	/* arbitrary limit on variable size */
          )
@@ -595,45 +513,40 @@ for (;;)
       else
         (void) string_format(name, sizeof(name), "%c%u", 'm', index - 10);
       node = acl_var_create(name);
-      node->data.ptr = store_get(count + 1, proto_mem);
+      node->data.ptr = store_get(count + 1);
       /* We sanity-checked the count, so disable the Coverity error */
       /* coverity[tainted_data] */
-      if (fread(node->data.ptr, 1, count+1, fp) < count) goto SPOOL_READ_ERROR;
+      if (fread(node->data.ptr, 1, count+1, f) < count) goto SPOOL_READ_ERROR;
       (US node->data.ptr)[count] = '\0';
       }
     break;
 
     case 'b':
     if (Ustrncmp(p, "ody_linecount", 13) == 0)
-      body_linecount = Uatoi(var + 14);
+      body_linecount = Uatoi(big_buffer + 15);
     else if (Ustrncmp(p, "ody_zerocount", 13) == 0)
-      body_zerocount = Uatoi(var + 14);
+      body_zerocount = Uatoi(big_buffer + 15);
 #ifdef EXPERIMENTAL_BRIGHTMAIL
     else if (Ustrncmp(p, "mi_verdicts ", 12) == 0)
-      bmi_verdicts = string_copy_taint(var + 13, proto_mem);
+      bmi_verdicts = string_copy(big_buffer + 14);
 #endif
     break;
 
     case 'd':
     if (Ustrcmp(p, "eliver_firsttime") == 0)
-      f.deliver_firsttime = TRUE;
+      deliver_firsttime = TRUE;
+    /* Check if the dsn flags have been set in the header file */
     else if (Ustrncmp(p, "sn_ret", 6) == 0)
-      dsn_ret= atoi(CS var + 7);
+      dsn_ret= atoi(CS big_buffer + 8);
     else if (Ustrncmp(p, "sn_envid", 8) == 0)
-      dsn_envid = string_copy_taint(var + 10, proto_mem);
-#ifndef COMPILE_UTILITY
-    else if (Ustrncmp(p, "ebug_selector ", 14) == 0)
-      debug_selector = strtol(CS var + 15, NULL, 0);
-    else if (Ustrncmp(p, "ebuglog_name ", 13) == 0)
-      debug_logging_from_spool(var + 14);
-#endif
+      dsn_envid = string_copy(big_buffer + 11);
     break;
 
     case 'f':
     if (Ustrncmp(p, "rozen", 5) == 0)
       {
-      f.deliver_freeze = TRUE;
-      if (sscanf(CS var+6, TIME_T_FMT, &deliver_frozen_at) != 1)
+      deliver_freeze = TRUE;
+      if (sscanf(CS big_buffer+7, TIME_T_FMT, &deliver_frozen_at) != 1)
 	goto SPOOL_READ_ERROR;
       }
     break;
@@ -643,14 +556,12 @@ for (;;)
       host_lookup_deferred = TRUE;
     else if (Ustrcmp(p, "ost_lookup_failed") == 0)
       host_lookup_failed = TRUE;
-    else if (Ustrncmp(p, "ost_auth_pubname", 16) == 0)
-      sender_host_auth_pubname = string_copy_taint(var + 18, proto_mem);
     else if (Ustrncmp(p, "ost_auth", 8) == 0)
-      sender_host_authenticated = string_copy_taint(var + 10, proto_mem);
+      sender_host_authenticated = string_copy(big_buffer + 11);
     else if (Ustrncmp(p, "ost_name", 8) == 0)
-      sender_host_name = string_copy_taint(var + 10, proto_mem);
+      sender_host_name = string_copy(big_buffer + 11);
     else if (Ustrncmp(p, "elo_name", 8) == 0)
-      sender_helo_name = string_copy_taint(var + 10, proto_mem);
+      sender_helo_name = string_copy(big_buffer + 11);
 
     /* We now record the port number after the address, separated by a
     dot. For compatibility during upgrading, do nothing if there
@@ -658,80 +569,65 @@ for (;;)
 
     else if (Ustrncmp(p, "ost_address", 11) == 0)
       {
-      sender_host_port = host_address_extract_port(var + 13);
-      sender_host_address = string_copy_taint(var + 13, proto_mem);
+      sender_host_port = host_address_extract_port(big_buffer + 14);
+      sender_host_address = string_copy(big_buffer + 14);
       }
     break;
 
     case 'i':
     if (Ustrncmp(p, "nterface_address", 16) == 0)
       {
-      interface_port = host_address_extract_port(var + 18);
-      interface_address = string_copy_taint(var + 18, proto_mem);
+      interface_port = host_address_extract_port(big_buffer + 19);
+      interface_address = string_copy(big_buffer + 19);
       }
     else if (Ustrncmp(p, "dent", 4) == 0)
-      sender_ident = string_copy_taint(var + 6, proto_mem);
+      sender_ident = string_copy(big_buffer + 7);
     break;
 
     case 'l':
     if (Ustrcmp(p, "ocal") == 0)
-      f.sender_local = TRUE;
-    else if (Ustrcmp(var, "localerror") == 0)
-      f.local_error_message = TRUE;
-#ifdef HAVE_LOCAL_SCAN
+      sender_local = TRUE;
+    else if (Ustrcmp(big_buffer, "-localerror") == 0)
+      local_error_message = TRUE;
     else if (Ustrncmp(p, "ocal_scan ", 10) == 0)
-      local_scan_data = string_copy_taint(var + 11, proto_mem);
-#endif
+      local_scan_data = string_copy(big_buffer + 12);
     break;
 
     case 'm':
-    if (Ustrcmp(p, "anual_thaw") == 0)
-      f.deliver_manual_thaw = TRUE;
+    if (Ustrcmp(p, "anual_thaw") == 0) deliver_manual_thaw = TRUE;
     else if (Ustrncmp(p, "ax_received_linelength", 22) == 0)
-      max_received_linelength = Uatoi(var + 23);
+      max_received_linelength = Uatoi(big_buffer + 24);
     break;
 
     case 'N':
-    if (*p == 0) f.dont_deliver = TRUE;   /* -N */
+    if (*p == 0) dont_deliver = TRUE;   /* -N */
     break;
 
     case 'r':
     if (Ustrncmp(p, "eceived_protocol", 16) == 0)
-      received_protocol = string_copy_taint(var + 18, proto_mem);
+      received_protocol = string_copy(big_buffer + 19);
     else if (Ustrncmp(p, "eceived_time_usec", 17) == 0)
       {
       unsigned usec;
-      if (sscanf(CS var + 20, "%u", &usec) == 1)
-	{
+      if (sscanf(CS big_buffer + 21, "%u", &usec) == 1)
 	received_time.tv_usec = usec;
-	if (!received_time_complete.tv_sec) received_time_complete.tv_usec = usec;
-	}
-      }
-    else if (Ustrncmp(p, "eceived_time_complete", 21) == 0)
-      {
-      unsigned sec, usec;
-      if (sscanf(CS var + 23, "%u.%u", &sec, &usec) == 2)
-	{
-	received_time_complete.tv_sec = sec;
-	received_time_complete.tv_usec = usec;
-	}
       }
     break;
 
     case 's':
     if (Ustrncmp(p, "ender_set_untrusted", 19) == 0)
-      f.sender_set_untrusted = TRUE;
+      sender_set_untrusted = TRUE;
 #ifdef WITH_CONTENT_SCAN
     else if (Ustrncmp(p, "pam_bar ", 8) == 0)
-      spam_bar = string_copy_taint(var + 9, proto_mem);
+      spam_bar = string_copy(big_buffer + 10);
     else if (Ustrncmp(p, "pam_score ", 10) == 0)
-      spam_score = string_copy_taint(var + 11, proto_mem);
+      spam_score = string_copy(big_buffer + 12);
     else if (Ustrncmp(p, "pam_score_int ", 14) == 0)
-      spam_score_int = string_copy_taint(var + 15, proto_mem);
+      spam_score_int = string_copy(big_buffer + 16);
 #endif
 #ifndef COMPILE_UTILITY
     else if (Ustrncmp(p, "pool_file_wireformat", 20) == 0)
-      f.spool_file_wireformat = TRUE;
+      spool_file_wireformat = TRUE;
 #endif
 #if defined(SUPPORT_I18N) && !defined(COMPILE_UTILITY)
     else if (Ustrncmp(p, "mtputf8", 7) == 0)
@@ -739,34 +635,24 @@ for (;;)
 #endif
     break;
 
-#ifndef DISABLE_TLS
+#ifdef SUPPORT_TLS
     case 't':
-    if (Ustrncmp(p, "ls_", 3) == 0)
-      {
-      const uschar * q = p + 3;
-      if (Ustrncmp(q, "certificate_verified", 20) == 0)
-	tls_in.certificate_verified = TRUE;
-      else if (Ustrncmp(q, "cipher", 6) == 0)
-	tls_in.cipher = string_copy_taint(q+7, proto_mem);
+    if (Ustrncmp(p, "ls_certificate_verified", 23) == 0)
+      tls_in.certificate_verified = TRUE;
+    else if (Ustrncmp(p, "ls_cipher", 9) == 0)
+      tls_in.cipher = string_copy(big_buffer + 12);
 # ifndef COMPILE_UTILITY	/* tls support fns not built in */
-      else if (Ustrncmp(q, "ourcert", 7) == 0)
-	(void) tls_import_cert(q+8, &tls_in.ourcert);
-      else if (Ustrncmp(q, "peercert", 8) == 0)
-	(void) tls_import_cert(q+9, &tls_in.peercert);
+    else if (Ustrncmp(p, "ls_ourcert", 10) == 0)
+      (void) tls_import_cert(big_buffer + 13, &tls_in.ourcert);
+    else if (Ustrncmp(p, "ls_peercert", 11) == 0)
+      (void) tls_import_cert(big_buffer + 14, &tls_in.peercert);
 # endif
-      else if (Ustrncmp(q, "peerdn", 6) == 0)
-	tls_in.peerdn = string_unprinting(string_copy_taint(q+7, proto_mem));
-      else if (Ustrncmp(q, "sni", 3) == 0)
-	tls_in.sni = string_unprinting(string_copy_taint(q+4, proto_mem));
-      else if (Ustrncmp(q, "ocsp", 4) == 0)
-	tls_in.ocsp = q[5] - '0';
-# ifndef DISABLE_TLS_RESUME
-      else if (Ustrncmp(q, "resumption", 10) == 0)
-	tls_in.resumption = q[11] - 'A';
-# endif
-      else if (Ustrncmp(q, "ver", 3) == 0)
-	tls_in.ver = string_copy_taint(q+4, proto_mem);
-      }
+    else if (Ustrncmp(p, "ls_peerdn", 9) == 0)
+      tls_in.peerdn = string_unprinting(string_copy(big_buffer + 12));
+    else if (Ustrncmp(p, "ls_sni", 6) == 0)
+      tls_in.sni = string_unprinting(string_copy(big_buffer + 9));
+    else if (Ustrncmp(p, "ls_ocsp", 7) == 0)
+      tls_in.ocsp = big_buffer[10] - '0';
     break;
 #endif
 
@@ -792,43 +678,44 @@ host_build_sender_fullhost();
 
 #ifndef COMPILE_UTILITY
 DEBUG(D_deliver)
-  debug_printf_indent("sender_local=%d ident=%s\n", f.sender_local,
-    sender_ident ? sender_ident : US"unset");
+  debug_printf("sender_local=%d ident=%s\n", sender_local,
+    (sender_ident == NULL)? US"unset" : sender_ident);
 #endif  /* COMPILE_UTILITY */
 
 /* We now have the tree of addresses NOT to deliver to, or a line
 containing "XX", indicating no tree. */
 
-where = US"nondeliver";
 if (Ustrncmp(big_buffer, "XX\n", 3) != 0 &&
-  !read_nonrecipients_tree(&tree_nonrecipients, fp, big_buffer, big_buffer_size))
+  !read_nonrecipients_tree(&tree_nonrecipients, f, big_buffer, big_buffer_size))
     goto SPOOL_FORMAT_ERROR;
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_deliver) debug_print_tree("Non-recipients", tree_nonrecipients);
+DEBUG(D_deliver)
+  {
+  debug_printf("Non-recipients:\n");
+  debug_print_tree(tree_nonrecipients);
+  }
 #endif  /* COMPILE_UTILITY */
 
 /* After reading the tree, the next line has not yet been read into the
 buffer. It contains the count of recipients which follow on separate lines.
 Apply an arbitrary sanity check.*/
 
-where = US"rcpt cnt";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (sscanf(CS big_buffer, "%d", &rcount) != 1 || rcount > 16384)
   goto SPOOL_FORMAT_ERROR;
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_deliver) debug_printf_indent("recipients_count=%d\n", rcount);
+DEBUG(D_deliver) debug_printf("recipients_count=%d\n", rcount);
 #endif  /* COMPILE_UTILITY */
 
 recipients_list_max = rcount;
-recipients_list = store_get(rcount * sizeof(recipient_item), GET_UNTAINTED);
+recipients_list = store_get(rcount * sizeof(recipient_item));
 
 /* We sanitised the count and know we have enough memory, so disable
 the Coverity error on recipients_count */
 /* coverity[tainted_data] */
 
-where = US"recipient";
 for (recipients_count = 0; recipients_count < rcount; recipients_count++)
   {
   int nn;
@@ -838,7 +725,7 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
   uschar *errors_to = NULL;
   uschar *p;
 
-  if (fgets_big_buffer(fp) == NULL) goto SPOOL_READ_ERROR;
+  if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
   nn = Ustrlen(big_buffer);
   if (nn < 2) goto SPOOL_FORMAT_ERROR;
 
@@ -889,9 +776,6 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
   if (*p == ',')
     {
     int dummy;
-#if !defined (COMPILE_UTILITY)
-    DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - Exim 3 spool file\n");
-#endif
     while (isdigit(*(--p)) || *p == ',');
     if (*p == ' ')
       {
@@ -904,9 +788,6 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
 
   else if (*p == ' ')
     {
-#if !defined (COMPILE_UTILITY)
-    DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - early Exim 4 spool file\n");
-#endif
     *p++ = 0;
     (void)sscanf(CS p, "%d", &pno);
     }
@@ -918,12 +799,12 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
     int flags;
 
 #if !defined (COMPILE_UTILITY)
-    DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - Exim standard format spoolfile\n");
+    DEBUG(D_deliver) debug_printf("**** SPOOL_IN - Exim 4 standard format spoolfile\n");
 #endif
 
     (void)sscanf(CS p+1, "%d", &flags);
 
-    if (flags & 0x01)      /* one_time data exists */
+    if ((flags & 0x01) != 0)      /* one_time data exists */
       {
       int len;
       while (isdigit(*(--p)) || *p == ',' || *p == '-');
@@ -932,12 +813,12 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
       if (len > 0)
         {
         p -= len;
-        errors_to = string_copy_taint(p, GET_TAINTED);
+        errors_to = string_copy(p);
         }
       }
 
-    *--p = 0;   /* Terminate address */
-    if (flags & 0x02)      /* one_time data exists */
+    *(--p) = 0;   /* Terminate address */
+    if ((flags & 0x02) != 0)      /* one_time data exists */
       {
       int len;
       while (isdigit(*(--p)) || *p == ',' || *p == '-');
@@ -946,25 +827,29 @@ for (recipients_count = 0; recipients_count < rcount; recipients_count++)
       if (len > 0)
         {
         p -= len;
-        orcpt = string_copy_taint(p, GET_TAINTED);
+        orcpt = string_copy(p);
         }
       }
 
-    *--p = 0;   /* Terminate address */
+    *(--p) = 0;   /* Terminate address */
     }
 #if !defined(COMPILE_UTILITY)
   else
-    { DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - No additional fields\n"); }
+    { DEBUG(D_deliver) debug_printf("**** SPOOL_IN - No additional fields\n"); }
 
-  if (orcpt || dsn_flags)
-    DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - address: <%s> orcpt: <%s> dsn_flags: 0x%x\n",
+  if ((orcpt != NULL) || (dsn_flags != 0))
+    {
+    DEBUG(D_deliver) debug_printf("**** SPOOL_IN - address: |%s| orcpt: |%s| dsn_flags: %d\n",
       big_buffer, orcpt, dsn_flags);
-  if (errors_to)
-    DEBUG(D_deliver) debug_printf_indent("**** SPOOL_IN - address: <%s> errorsto: <%s>\n",
+    }
+  if (errors_to != NULL)
+    {
+    DEBUG(D_deliver) debug_printf("**** SPOOL_IN - address: |%s| errorsto: |%s|\n",
       big_buffer, errors_to);
+    }
 #endif
 
-  recipients_list[recipients_count].address = string_copy_taint(big_buffer, GET_TAINTED);
+  recipients_list[recipients_count].address = string_copy(big_buffer);
   recipients_list[recipients_count].pno = pno;
   recipients_list[recipients_count].errors_to = errors_to;
   recipients_list[recipients_count].orcpt = orcpt;
@@ -980,38 +865,37 @@ always, in order to check on the format of the file, but only create a header
 list if requested to do so. */
 
 inheader = TRUE;
-where = US"headers";
-if (Ufgets(big_buffer, big_buffer_size, fp) == NULL) goto SPOOL_READ_ERROR;
+if (Ufgets(big_buffer, big_buffer_size, f) == NULL) goto SPOOL_READ_ERROR;
 if (big_buffer[0] != '\n') goto SPOOL_FORMAT_ERROR;
 
-while ((n = fgetc(fp)) != EOF)
+while ((n = fgetc(f)) != EOF)
   {
-  header_line * h;
+  header_line *h;
   uschar flag[4];
   int i;
 
   if (!isdigit(n)) goto SPOOL_FORMAT_ERROR;
-  if(ungetc(n, fp) == EOF  ||  fscanf(fp, "%d%c ", &n, flag) == EOF)
+  if(ungetc(n, f) == EOF  ||  fscanf(f, "%d%c ", &n, flag) == EOF)
     goto SPOOL_READ_ERROR;
   if (flag[0] != '*') message_size += n;  /* Omit non-transmitted headers */
 
   if (read_headers)
     {
-    h = store_get(sizeof(header_line), GET_UNTAINTED);
+    h = store_get(sizeof(header_line));
     h->next = NULL;
     h->type = flag[0];
     h->slen = n;
-    h->text = store_get(n+1, GET_TAINTED);
+    h->text = store_get(n+1);
 
     if (h->type == htype_received) received_count++;
 
-    if (header_list) header_last->next = h;
-    else header_list = h;
+    if (header_list == NULL) header_list = h;
+      else header_last->next = h;
     header_last = h;
 
     for (i = 0; i < n; i++)
       {
-      int c = fgetc(fp);
+      int c = fgetc(f);
       if (c == 0 || c == EOF) goto SPOOL_FORMAT_ERROR;
       if (c == '\n' && h->type != htype_old) message_linecount++;
       h->text[i] = c;
@@ -1023,7 +907,7 @@ while ((n = fgetc(fp)) != EOF)
 
   else for (i = 0; i < n; i++)
     {
-    int c = fgetc(fp);
+    int c = fgetc(f);
     if (c == 0 || c == EOF) goto SPOOL_FORMAT_ERROR;
     }
   }
@@ -1033,13 +917,13 @@ line count by adding the body linecount to the header linecount. Close the file
 and give a positive response. */
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_deliver) debug_printf_indent("body_linecount=%d message_linecount=%d\n",
+DEBUG(D_deliver) debug_printf("body_linecount=%d message_linecount=%d\n",
   body_linecount, message_linecount);
 #endif  /* COMPILE_UTILITY */
 
 message_linecount += body_linecount;
 
-fclose(fp);
+fclose(f);
 return spool_read_OK;
 
 
@@ -1056,63 +940,22 @@ if (errno != 0)
   DEBUG(D_any) debug_printf("Error while reading spool file %s\n", name);
 #endif  /* COMPILE_UTILITY */
 
-  fclose(fp);
+  fclose(f);
   errno = n;
-  return inheader ? spool_read_hdrerror : spool_read_enverror;
+  return inheader? spool_read_hdrerror : spool_read_enverror;
   }
 
 SPOOL_FORMAT_ERROR:
 
 #ifndef COMPILE_UTILITY
-DEBUG(D_any) debug_printf("Format error in spool file %s%s%s\n", name,
-  where ? ": " : "", where ? where : US"");
+DEBUG(D_any) debug_printf("Format error in spool file %s\n", name);
 #endif  /* COMPILE_UTILITY */
 
-fclose(fp);
+fclose(f);
 errno = ERRNO_SPOOLFORMAT;
 return inheader? spool_read_hdrerror : spool_read_enverror;
 }
 
-
-#ifndef COMPILE_UTILITY
-/* Read out just the (envelope) sender string from the spool -H file.
-Remove the <> wrap and return it in allocated store.  Return NULL on error.
-
-We assume that message_subdir is already set.
-*/
-
-uschar *
-spool_sender_from_msgid(const uschar * id)
-{
-FILE * fp;
-int n;
-uschar * yield = NULL;
-
-if (!(fp = Ufopen(spool_fname(US"input", message_subdir, id, US"-H"), "rb")))
-  return NULL;
-
-DEBUG(D_deliver) debug_printf_indent("reading spool file %s-H\n", id);
-
-/* Skip the line with the copy of the filename, then the line with login/uid/gid.
-Read the next line, which should be the envelope sender.
-Do basic validation on that. */
-
-if (  Ufgets(big_buffer, big_buffer_size, fp) != NULL
-   && Ufgets(big_buffer, big_buffer_size, fp) != NULL
-   && Ufgets(big_buffer, big_buffer_size, fp) != NULL
-   && (n = Ustrlen(big_buffer)) >= 3
-   && big_buffer[0] == '<' && big_buffer[n-2] == '>'
-   )
-  {
-  yield = store_get(n-2, GET_TAINTED);
-  Ustrncpy(yield, big_buffer+1, n-3);
-  yield[n-3] = 0;
-  }
-fclose(fp);
-return yield;
-}
-#endif  /* COMPILE_UTILITY */
-
 /* vi: aw ai sw=2
 */
 /* End of spool_in.c */
