@@ -2,11 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/*
- * Copyright (c) The Exim Maintainers 2016 - 2025
- * Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
+/* Copyright (c) Tom Kistner <tom@duncanthrax.net> 2003 - 2015
  * License: GPL
- * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (c) The Exim Maintainers 2016 - 2018
  */
 
 /* Code for setting up a MBOX style spool file inside a /scan/<msgid>
@@ -31,31 +29,35 @@ Normally, source_file_override is NULL
 
 FILE *
 spool_mbox(unsigned long *mbox_file_size, const uschar *source_file_override,
-  const uschar ** mbox_fname)
+  uschar ** mbox_fname)
 {
 uschar message_subdir[2];
 uschar buffer[16384];
-const uschar * s, * mbox_path;
-FILE * mbox_file = NULL, * l_data_file = NULL, * yield = NULL;
+uschar *temp_string;
+uschar *mbox_path;
+FILE *mbox_file = NULL;
+FILE *data_file = NULL;
+FILE *yield = NULL;
+header_line *my_headerlist;
 struct stat statbuf;
-int j;
-rmark reset_point;
+int i, j;
+void *reset_point;
 
 mbox_path = string_sprintf("%s/scan/%s/%s.eml",
   spool_directory, message_id, message_id);
 if (mbox_fname) *mbox_fname = mbox_path;
 
-reset_point = store_mark();
+reset_point = store_get(0);
 
 /* Skip creation if already spooled out as mbox file */
 if (!spool_mbox_ok)
   {
   /* create temp directory inside scan dir, directory_make works recursively */
-  s = string_sprintf("scan/%s", message_id);
-  if (!directory_make(spool_directory, s, 0750, FALSE))
+  temp_string = string_sprintf("scan/%s", message_id);
+  if (!directory_make(spool_directory, temp_string, 0750, FALSE))
     {
-    log_write(0, LOG_MAIN|LOG_PANIC, "%s",
-      string_open_failed("scan directory %s/scan/%s", spool_directory, s));
+    log_write(0, LOG_MAIN|LOG_PANIC, "%s", string_open_failed(errno,
+      "scan directory %s/scan/%s", spool_directory, temp_string));
     goto OUT;
     }
 
@@ -63,8 +65,8 @@ if (!spool_mbox_ok)
 
   if (!(mbox_file = modefopen(mbox_path, "wb", SPOOL_MODE)))
     {
-    log_write(0, LOG_MAIN|LOG_PANIC, "%s",
-      string_open_failed("scan file %s", mbox_path));
+    log_write(0, LOG_MAIN|LOG_PANIC, "%s", string_open_failed(errno,
+      "scan file %s", mbox_path));
     goto OUT;
     }
 
@@ -73,13 +75,13 @@ if (!spool_mbox_ok)
   contents of the Received: header line. However, the code below will use it
   if it should become available in future. */
 
-  s = expand_string(
+  temp_string = expand_string(
     US"From ${if def:return_path{$return_path}{MAILER-DAEMON}} ${tod_bsdinbox}\n"
     "${if def:sender_address{X-Envelope-From: <${sender_address}>\n}}"
     "${if def:recipients{X-Envelope-To: ${recipients}\n}}");
 
-  if (s)
-    if (fwrite(s, Ustrlen(s), 1, mbox_file) != 1)
+  if (temp_string)
+    if (fwrite(temp_string, Ustrlen(temp_string), 1, mbox_file) != 1)
       {
       log_write(0, LOG_MAIN|LOG_PANIC, "Error/short write while writing \
 	  mailbox headers to %s", mbox_path);
@@ -88,7 +90,7 @@ if (!spool_mbox_ok)
 
   /* write all non-deleted header lines to mbox file */
 
-  for (header_line * my_headerlist = header_list; my_headerlist;
+  for (my_headerlist = header_list; my_headerlist;
       my_headerlist = my_headerlist->next)
     if (my_headerlist->type != '*')
       if (fwrite(my_headerlist->text, my_headerlist->slen, 1, mbox_file) != 1)
@@ -99,7 +101,6 @@ if (!spool_mbox_ok)
 	}
 
   /* End headers */
-
   if (fwrite("\n", 1, 1, mbox_file) != 1)
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "Error/short write while writing \
@@ -107,23 +108,21 @@ if (!spool_mbox_ok)
     goto OUT;
     }
 
-  /* Copy body file.  If the main receive still has it open then it is holding
-  a lock, and we must not close it (which releases the lock), so just use the
-  global file handle. */
-
-  if (source_file_override)
-    l_data_file = Ufopen(source_file_override, "rb");
-  else if (spool_data_file)
-    l_data_file = spool_data_file;
-  else
-    for (int i = 0; i < 2; i++)
+  /* copy body file */
+  if (!source_file_override)
+    {
+    message_subdir[1] = '\0';
+    for (i = 0; i < 2; i++)
       {
-      set_subdir_str(message_subdir, message_id, i);
-      s = spool_fname(US"input", message_subdir, message_id, US"-D");
-      if ((l_data_file = Ufopen(s, "rb"))) break;
+      message_subdir[0] = split_spool_directory == (i == 0) ? message_id[5] : 0;
+      temp_string = spool_fname(US"input", message_subdir, message_id, US"-D");
+      if ((data_file = Ufopen(temp_string, "rb"))) break;
       }
+    }
+  else
+    data_file = Ufopen(source_file_override, "rb");
 
-  if (!l_data_file)
+  if (!data_file)
     {
     log_write(0, LOG_MAIN|LOG_PANIC, "Could not open datafile for message %s",
       message_id);
@@ -132,7 +131,7 @@ if (!spool_mbox_ok)
 
   /* The code used to use this line, but it doesn't work in Cygwin.
 
-      (void)fread(data_buffer, 1, 18, l_data_file);
+      (void)fread(data_buffer, 1, 18, data_file);
 
      What's happening is that spool_mbox used to use an fread to jump over the
      file header. That fails under Cygwin because the header is locked, but
@@ -140,23 +139,23 @@ if (!spool_mbox_ok)
      explicitly, because the one in the file is parted of the locked area.  */
 
   if (!source_file_override)
-    (void)fseek(l_data_file, spool_data_start_offset(message_id), SEEK_SET);
+    (void)fseek(data_file, SPOOL_DATA_START_OFFSET, SEEK_SET);
 
   do
     {
     uschar * s;
 
-    if (!f.spool_file_wireformat || source_file_override)
-      j = fread(buffer, 1, sizeof(buffer), l_data_file);
+    if (!spool_file_wireformat || source_file_override)
+      j = fread(buffer, 1, sizeof(buffer), data_file);
     else						/* needs CRLF -> NL */
-      if ((s = US fgets(CS buffer, sizeof(buffer), l_data_file)))
+      if ((s = US fgets(CS buffer, sizeof(buffer), data_file)))
 	{
 	uschar * p = s + Ustrlen(s) - 1;
 
 	if (*p == '\n' && p[-1] == '\r')
 	  *--p = '\n';
 	else if (*p == '\r')
-	  ungetc(*p--, l_data_file);
+	  ungetc(*p--, data_file);
 
 	j = p - buffer;
 	}
@@ -185,13 +184,13 @@ if (!spool_mbox_ok)
 if (  !(yield = Ufopen(mbox_path,"rb"))
    || fstat(fileno(yield), &statbuf) != 0
    )
-  log_write(0, LOG_MAIN|LOG_PANIC, "%s",
-    string_open_failed( "scan file %s", mbox_path));
+  log_write(0, LOG_MAIN|LOG_PANIC, "%s", string_open_failed(errno,
+    "scan file %s", mbox_path));
 else
   *mbox_file_size = statbuf.st_size;
 
 OUT:
-if (l_data_file && !spool_data_file) (void)fclose(l_data_file);
+if (data_file) (void)fclose(data_file);
 if (mbox_file) (void)fclose(mbox_file);
 store_reset(reset_point);
 return yield;
@@ -208,38 +207,39 @@ unspool_mbox(void)
 spam_ok = 0;
 malware_ok = 0;
 
-if (spool_mbox_ok && !f.no_mbox_unspool)
+if (spool_mbox_ok && !no_mbox_unspool)
   {
+  uschar *mbox_path;
   uschar *file_path;
+  struct dirent *entry;
   DIR *tempdir;
-  rmark reset_point = store_mark();
-  uschar * mbox_path = string_sprintf("%s/scan/%s", spool_directory, spooled_message_id);
 
-  if (!(tempdir = exim_opendir(mbox_path)))
+  mbox_path = string_sprintf("%s/scan/%s", spool_directory, spooled_message_id);
+
+  if (!(tempdir = opendir(CS mbox_path)))
     {
     debug_printf("Unable to opendir(%s): %s\n", mbox_path, strerror(errno));
     /* Just in case we still can: */
-    (void) rmdir(CS mbox_path);
+    rmdir(CS mbox_path);
     return;
     }
   /* loop thru dir & delete entries */
-  for (struct dirent *entry; entry = readdir(tempdir); )
+  while((entry = readdir(tempdir)))
     {
     uschar *name = US entry->d_name;
+    int dummy;
     if (Ustrcmp(name, US".") == 0 || Ustrcmp(name, US"..") == 0) continue;
 
     file_path = string_sprintf("%s/%s", mbox_path, name);
     debug_printf("unspool_mbox(): unlinking '%s'\n", file_path);
-    if (unlink(CS file_path) != 0)
-      log_write(0, LOG_MAIN|LOG_PANIC, "unlink(%s): %s", file_path, strerror(errno));
+    dummy = unlink(CS file_path); dummy = dummy;	/* compiler quietening */
     }
 
   closedir(tempdir);
 
   /* remove directory */
-  if (rmdir(CS mbox_path) != 0)
-    log_write(0, LOG_MAIN|LOG_PANIC, "rmdir(%s): %s", mbox_path, strerror(errno));
-  store_reset(reset_point);
+  rmdir(CS mbox_path);
+  store_reset(mbox_path);
   }
 spool_mbox_ok = 0;
 }
