@@ -39,7 +39,6 @@ require current GnuTLS, then we'll drop support for the ancient libraries).
 #include <gnutls/x509.h>
 /* man-page is incorrect, gnutls_rnd() is not in gnutls.h: */
 #include <gnutls/crypto.h>
-
 /* needed to disable PKCS11 autoload unless requested */
 #if GNUTLS_VERSION_NUMBER >= 0x020c00
 # include <gnutls/pkcs11.h>
@@ -61,9 +60,6 @@ require current GnuTLS, then we'll drop support for the ancient libraries).
 #if GNUTLS_VERSION_NUMBER >= 0x030014
 # define SUPPORT_SYSDEFAULT_CABUNDLE
 #endif
-#if GNUTLS_VERSION_NUMBER >= 0x030104
-# define GNUTLS_CERT_VFY_STATUS_PRINT
-#endif
 #if GNUTLS_VERSION_NUMBER >= 0x030109
 # define SUPPORT_CORK
 #endif
@@ -128,8 +124,8 @@ typedef struct exim_gnutls_state {
   BOOL			peer_dane_verified;
   BOOL			trigger_sni_changes;
   BOOL			have_set_peerdn;
-  const struct host_item *host;		/* NULL if server */
-  gnutls_x509_crt_t	peercert;
+  const struct host_item *host;
+  gnutls_x509_crt_t 	peercert;
   uschar		*peerdn;
   uschar		*ciphersuite;
   uschar		*received_sni;
@@ -217,7 +213,7 @@ second connection.
 XXX But see gnutls_session_get_ptr()
 */
 
-static exim_gnutls_state_st state_server;
+static exim_gnutls_state_st state_server, state_client;
 
 /* dh_params are initialised once within the lifetime of a process using TLS;
 if we used TLS in a long-lived daemon, we'd have to reconsider this.  But we
@@ -266,7 +262,7 @@ before, for now. */
 
 #define exim_gnutls_err_check(rc, Label) do { \
   if ((rc) != GNUTLS_E_SUCCESS) \
-    return tls_error((Label), US gnutls_strerror(rc), host, errstr); \
+    return tls_error((Label), gnutls_strerror(rc), host, errstr); \
   } while (0)
 
 #define expand_check_tlsvar(Varname, errstr) \
@@ -332,11 +328,11 @@ Returns:    OK/DEFER/FAIL
 */
 
 static int
-tls_error(const uschar *prefix, const uschar *msg, const host_item *host,
+tls_error(const uschar *prefix, const char *msg, const host_item *host,
   uschar ** errstr)
 {
 if (errstr)
-  *errstr = string_sprintf("(%s)%s%s", prefix, msg ? ": " : "", msg ? msg : US"");
+  *errstr = string_sprintf("(%s)%s%s", prefix, msg ? ": " : "", msg ? msg : "");
 return host ? FAIL : DEFER;
 }
 
@@ -361,14 +357,14 @@ Returns:   nothing
 static void
 record_io_error(exim_gnutls_state_st *state, int rc, uschar *when, uschar *text)
 {
-const uschar * msg;
+const char * msg;
 uschar * errstr;
 
 if (rc == GNUTLS_E_FATAL_ALERT_RECEIVED)
-  msg = string_sprintf("%s: %s", US gnutls_strerror(rc),
+  msg = CS string_sprintf("%s: %s", US gnutls_strerror(rc),
     US gnutls_alert_get_name(gnutls_alert_get(state->session)));
 else
-  msg = US gnutls_strerror(rc);
+  msg = gnutls_strerror(rc);
 
 (void) tls_error(when, msg, state->host, &errstr);
 
@@ -452,8 +448,7 @@ gnutls_datum_t channel;
 #endif
 tls_support * tlsp = state->tlsp;
 
-tlsp->active.sock = state->fd_out;
-tlsp->active.tls_ctx = state;
+tlsp->active = state->fd_out;
 
 cipher = gnutls_cipher_get(state->session);
 /* returns size in "bytes" */
@@ -561,7 +556,7 @@ else if (Ustrcmp(exp_tls_dhparam, "none") == 0)
 else if (exp_tls_dhparam[0] != '/')
   {
   if (!(m.data = US std_dh_prime_named(exp_tls_dhparam)))
-    return tls_error(US"No standard prime named", exp_tls_dhparam, NULL, errstr);
+    return tls_error(US"No standard prime named", CS exp_tls_dhparam, NULL, errstr);
   m.size = Ustrlen(m.data);
   }
 else
@@ -624,7 +619,7 @@ if ((fd = Uopen(filename, O_RDONLY, 0)) >= 0)
     {
     saved_errno = errno;
     (void)close(fd);
-    return tls_error(US"TLS cache stat failed", US strerror(saved_errno), NULL, errstr);
+    return tls_error(US"TLS cache stat failed", strerror(saved_errno), NULL, errstr);
     }
   if (!S_ISREG(statbuf.st_mode))
     {
@@ -636,21 +631,21 @@ if ((fd = Uopen(filename, O_RDONLY, 0)) >= 0)
     saved_errno = errno;
     (void)close(fd);
     return tls_error(US"fdopen(TLS cache stat fd) failed",
-        US strerror(saved_errno), NULL, errstr);
+        strerror(saved_errno), NULL, errstr);
     }
 
   m.size = statbuf.st_size;
   if (!(m.data = malloc(m.size)))
     {
     fclose(fp);
-    return tls_error(US"malloc failed", US strerror(errno), NULL, errstr);
+    return tls_error(US"malloc failed", strerror(errno), NULL, errstr);
     }
   if (!(sz = fread(m.data, m.size, 1, fp)))
     {
     saved_errno = errno;
     fclose(fp);
     free(m.data);
-    return tls_error(US"fread failed", US strerror(saved_errno), NULL, errstr);
+    return tls_error(US"fread failed", strerror(saved_errno), NULL, errstr);
     }
   fclose(fp);
 
@@ -686,11 +681,11 @@ if (rc < 0)
 
   if ((PATH_MAX - Ustrlen(filename)) < 10)
     return tls_error(US"Filename too long to generate replacement",
-        filename, NULL, errstr);
+        CS filename, NULL, errstr);
 
-  temp_fn = string_copy(US"%s.XXXXXXX");
+  temp_fn = string_copy(US "%s.XXXXXXX");
   if ((fd = mkstemp(CS temp_fn)) < 0)	/* modifies temp_fn */
-    return tls_error(US"Unable to open temp file", US strerror(errno), NULL, errstr);
+    return tls_error(US"Unable to open temp file", strerror(errno), NULL, errstr);
   (void)fchown(fd, exim_uid, exim_gid);   /* Probably not necessary */
 
   /* GnuTLS overshoots!
@@ -727,7 +722,7 @@ if (rc < 0)
     exim_gnutls_err_check(rc, US"gnutls_dh_params_export_pkcs3(NULL) sizing");
   m.size = sz;
   if (!(m.data = malloc(m.size)))
-    return tls_error(US"memory allocation failed", US strerror(errno), NULL, errstr);
+    return tls_error(US"memory allocation failed", strerror(errno), NULL, errstr);
 
   /* this will return a size 1 less than the allocation size above */
   rc = gnutls_dh_params_export_pkcs3(dh_server_params, GNUTLS_X509_FMT_PEM,
@@ -743,19 +738,19 @@ if (rc < 0)
     {
     free(m.data);
     return tls_error(US"TLS cache write D-H params failed",
-        US strerror(errno), NULL, errstr);
+        strerror(errno), NULL, errstr);
     }
   free(m.data);
   if ((sz = write_to_fd_buf(fd, US"\n", 1)) != 1)
     return tls_error(US"TLS cache write D-H params final newline failed",
-        US strerror(errno), NULL, errstr);
+        strerror(errno), NULL, errstr);
 
   if ((rc = close(fd)))
-    return tls_error(US"TLS cache write close() failed", US strerror(errno), NULL, errstr);
+    return tls_error(US"TLS cache write close() failed", strerror(errno), NULL, errstr);
 
   if (Urename(temp_fn, filename) < 0)
     return tls_error(string_sprintf("failed to rename \"%s\" as \"%s\"",
-          temp_fn, filename), US strerror(errno), NULL, errstr);
+          temp_fn, filename), strerror(errno), NULL, errstr);
 
   DEBUG(D_tls) debug_printf("wrote D-H parameters to file \"%s\"\n", filename);
   }
@@ -787,18 +782,15 @@ if ((rc = gnutls_x509_crt_init(&cert))) goto err;
 where = US"generating pkey";
 if ((rc = gnutls_x509_privkey_generate(pkey, GNUTLS_PK_RSA,
 #ifdef SUPPORT_PARAM_TO_PK_BITS
-# ifndef GNUTLS_SEC_PARAM_MEDIUM
-#  define GNUTLS_SEC_PARAM_MEDIUM GNUTLS_SEC_PARAM_HIGH
-# endif
-	    gnutls_sec_param_to_pk_bits(GNUTLS_PK_RSA, GNUTLS_SEC_PARAM_MEDIUM),
+	    gnutls_sec_param_to_pk_bits(GNUTLS_PK_RSA, GNUTLS_SEC_PARAM_LOW),
 #else
-	    2048,
+	    1024,
 #endif
 	    0)))
   goto err;
 
 where = US"configuring cert";
-now = 1;
+now = 0;
 if (  (rc = gnutls_x509_crt_set_version(cert, 3))
    || (rc = gnutls_x509_crt_set_serial(cert, &now, sizeof(now)))
    || (rc = gnutls_x509_crt_set_activation_time(cert, now = time(NULL)))
@@ -831,7 +823,7 @@ out:
   return rc;
 
 err:
-  rc = tls_error(where, US gnutls_strerror(rc), NULL, errstr);
+  rc = tls_error(where, gnutls_strerror(rc), NULL, errstr);
   goto out;
 }
 
@@ -854,7 +846,7 @@ int rc = gnutls_certificate_set_x509_key_file(state->x509_cred,
 if (rc < 0)
   return tls_error(
     string_sprintf("cert/key setup: cert=%s key=%s", certfile, keyfile),
-    US gnutls_strerror(rc), host, errstr);
+    gnutls_strerror(rc), host, errstr);
 return -rc;
 }
 
@@ -1270,7 +1262,6 @@ tls_init(
     const uschar *crl,
     const uschar *require_ciphers,
     exim_gnutls_state_st **caller_state,
-    tls_support * tlsp,
     uschar ** errstr)
 {
 exim_gnutls_state_st *state;
@@ -1304,7 +1295,7 @@ if (!exim_gnutls_base_init_done)
   DEBUG(D_tls)
     {
     gnutls_global_set_log_function(exim_gnutls_logger_cb);
-    /* arbitrarily chosen level; bump up to 9 for more */
+    /* arbitrarily chosen level; bump upto 9 for more */
     gnutls_global_set_log_level(EXIM_GNUTLS_LIBRARY_LOG_LEVEL);
     }
 #endif
@@ -1319,15 +1310,9 @@ if (!exim_gnutls_base_init_done)
 
 if (host)
   {
-  /* For client-side sessions we allocate a context. This lets us run
-  several in parallel. */
-  int old_pool = store_pool;
-  store_pool = POOL_PERM;
-  state = store_get(sizeof(exim_gnutls_state_st));
-  store_pool = old_pool;
-
+  state = &state_client;
   memcpy(state, &exim_gnutls_state_init, sizeof(exim_gnutls_state_init));
-  state->tlsp = tlsp;
+  state->tlsp = &tls_out;
   DEBUG(D_tls) debug_printf("initialising GnuTLS client session\n");
   rc = gnutls_init(&state->session, GNUTLS_CLIENT);
   }
@@ -1335,7 +1320,7 @@ else
   {
   state = &state_server;
   memcpy(state, &exim_gnutls_state_init, sizeof(exim_gnutls_state_init));
-  state->tlsp = tlsp;
+  state->tlsp = &tls_in;
   DEBUG(D_tls) debug_printf("initialising GnuTLS server session\n");
   rc = gnutls_init(&state->session, GNUTLS_SERVER);
   }
@@ -1525,14 +1510,14 @@ if (cert_list == NULL || cert_list_size == 0)
       cert_list, cert_list_size);
   if (state->verify_requirement >= VERIFY_REQUIRED)
     return tls_error(US"certificate verification failed",
-        US"no certificate received from peer", state->host, errstr);
+        "no certificate received from peer", state->host, errstr);
   return OK;
   }
 
 ct = gnutls_certificate_type_get(state->session);
 if (ct != GNUTLS_CRT_X509)
   {
-  const uschar *ctn = US gnutls_certificate_type_get_name(ct);
+  const char *ctn = gnutls_certificate_type_get_name(ct);
   DEBUG(D_tls)
     debug_printf("TLS: peer cert not X.509 but instead \"%s\"\n", ctn);
   if (state->verify_requirement >= VERIFY_REQUIRED)
@@ -1548,7 +1533,7 @@ if (ct != GNUTLS_CRT_X509)
       DEBUG(D_tls) debug_printf("TLS: peer cert problem: %s: %s\n", \
 	(Label), gnutls_strerror(rc)); \
       if (state->verify_requirement >= VERIFY_REQUIRED) \
-	return tls_error((Label), US gnutls_strerror(rc), state->host, errstr); \
+	return tls_error((Label), gnutls_strerror(rc), state->host, errstr); \
       return OK; \
       } \
     } while (0)
@@ -1605,7 +1590,6 @@ uint verify;
 if (state->verify_requirement == VERIFY_NONE)
   return TRUE;
 
-DEBUG(D_tls) debug_printf("TLS: checking peer certificate\n");
 *errstr = NULL;
 
 if ((rc = peer_status(state, errstr)) != OK)
@@ -1633,7 +1617,8 @@ else
 # ifdef GNUTLS_BROKEN_DANE_VALIDATION
     /* Split the TLSA records into two sets, TA and EE selectors.  Run the
     dane-verification separately so that we know which selector verified;
-    then we know whether to do name-verification (needed for TA but not EE). */
+    then we know whether to do CA-chain-verification and name-verification
+    (needed for TA but not EE). */
 
     if (usage == ((1<<DANESSL_USAGE_DANE_TA) | (1<<DANESSL_USAGE_DANE_EE)))
       {						/* a mixed-usage bundle */
@@ -1715,31 +1700,20 @@ else
       *errstr = US str.data;	/* don't bother to free */
       goto badcert;
       }
+    state->peer_dane_verified = TRUE;
 
 # ifdef GNUTLS_BROKEN_DANE_VALIDATION
     /* If a TA-mode TLSA record was used for verification we must additionally
-    verify the cert name (but not the CA chain).  For EE-mode, skip it. */
+    verify the CA chain and the cert name.  For EE-mode, skip it. */
 
     if (usage & (1 << DANESSL_USAGE_DANE_EE))
 # endif
       {
-      state->peer_dane_verified = state->peer_cert_verified = TRUE;
+      state->peer_cert_verified = TRUE;
       goto goodcert;
       }
-# ifdef GNUTLS_BROKEN_DANE_VALIDATION
-    /* Assume that the name on the A-record is the one that should be matching
-    the cert.  An alternate view is that the domain part of the email address
-    is also permissible. */
-
-    if (gnutls_x509_crt_check_hostname(state->tlsp->peercert,
-	  CS state->host->name))
-      {
-      state->peer_dane_verified = state->peer_cert_verified = TRUE;
-      goto goodcert;
-      }
-# endif
     }
-#endif	/*SUPPORT_DANE*/
+#endif
 
   rc = gnutls_certificate_verify_peers2(state->session, &verify);
   }
@@ -1750,24 +1724,8 @@ if (rc < 0 || verify & (GNUTLS_CERT_INVALID|GNUTLS_CERT_REVOKED))
   {
   state->peer_cert_verified = FALSE;
   if (!*errstr)
-    {
-#ifdef GNUTLS_CERT_VFY_STATUS_PRINT
-    DEBUG(D_tls)
-      {
-      gnutls_datum_t txt;
-
-      if (gnutls_certificate_verification_status_print(verify,
-	    gnutls_certificate_type_get(state->session), &txt, 0)
-	  == GNUTLS_E_SUCCESS)
-	{
-	debug_printf("%s\n", txt.data);
-	gnutls_free(txt.data);
-	}
-      }
-#endif
     *errstr = verify & GNUTLS_CERT_REVOKED
       ? US"certificate revoked" : US"certificate invalid";
-    }
 
   DEBUG(D_tls)
     debug_printf("TLS certificate verification failed (%s): peerdn=\"%s\"\n",
@@ -1781,23 +1739,23 @@ if (rc < 0 || verify & (GNUTLS_CERT_INVALID|GNUTLS_CERT_REVOKED))
 
 else
   {
-  /* Client side, check the server's certificate name versus the name on the
-  A-record for the connection we made.  What to do for server side - what name
-  to use for client?  We document that there is no such checking for server
-  side. */
-
-  if (  state->exp_tls_verify_cert_hostnames
-     && !gnutls_x509_crt_check_hostname(state->tlsp->peercert,
-		CS state->exp_tls_verify_cert_hostnames)
-     )
+  if (state->exp_tls_verify_cert_hostnames)
     {
-    DEBUG(D_tls)
-      debug_printf("TLS certificate verification failed: cert name mismatch\n");
-    if (state->verify_requirement >= VERIFY_REQUIRED)
-      goto badcert;
-    return TRUE;
+    int sep = 0;
+    const uschar * list = state->exp_tls_verify_cert_hostnames;
+    uschar * name;
+    while ((name = string_nextinlist(&list, &sep, NULL, 0)))
+      if (gnutls_x509_crt_check_hostname(state->tlsp->peercert, CS name))
+	break;
+    if (!name)
+      {
+      DEBUG(D_tls)
+	debug_printf("TLS certificate verification failed: cert name mismatch\n");
+      if (state->verify_requirement >= VERIFY_REQUIRED)
+	goto badcert;
+      return TRUE;
+      }
     }
-
   state->peer_cert_verified = TRUE;
   DEBUG(D_tls) debug_printf("TLS certificate verified: peerdn=\"%s\"\n",
       state->peerdn ? state->peerdn : US"<unset>");
@@ -1809,8 +1767,7 @@ goodcert:
 
 #ifdef SUPPORT_DANE
 tlsa_prob:
-  *errstr = string_sprintf("TLSA record problem: %s",
-    rc == DANE_E_REQUESTED_DATA_NOT_AVAILABLE ? "none usable" : dane_strerror(rc));
+  *errstr = string_sprintf("TLSA record problem: %s", dane_strerror(rc));
 #endif
 
 badcert:
@@ -2023,9 +1980,9 @@ int rc;
 exim_gnutls_state_st * state = NULL;
 
 /* Check for previous activation */
-if (tls_in.active.sock >= 0)
+if (tls_in.active >= 0)
   {
-  tls_error(US"STARTTLS received after TLS started", US "", NULL, errstr);
+  tls_error(US"STARTTLS received after TLS started", "", NULL, errstr);
   smtp_printf("554 Already in TLS\r\n", FALSE);
   return FAIL;
   }
@@ -2037,7 +1994,7 @@ DEBUG(D_tls) debug_printf("initialising GnuTLS as a server\n");
 
 if ((rc = tls_init(NULL, tls_certificate, tls_privatekey,
     NULL, tls_verify_certificates, tls_crl,
-    require_ciphers, &state, &tls_in, errstr)) != OK) return rc;
+    require_ciphers, &state, errstr)) != OK) return rc;
 
 /* If this is a host for which certificate verification is mandatory or
 optional, set up appropriately. */
@@ -2092,10 +2049,7 @@ if (!state->tlsp->on_connect)
   }
 
 /* Now negotiate the TLS session. We put our own timer on it, since it seems
-that the GnuTLS library doesn't.
-From 3.1.0 there is gnutls_handshake_set_timeout() - but it requires you
-to set (and clear down afterwards) up a pull-timeout callback function that does
-a select, so we're no better off unless avoiding signals becomes an issue. */
+that the GnuTLS library doesn't. */
 
 gnutls_transport_set_ptr2(state->session,
     (gnutls_transport_ptr_t)(long) fileno(smtp_in),
@@ -2104,11 +2058,11 @@ state->fd_in = fileno(smtp_in);
 state->fd_out = fileno(smtp_out);
 
 sigalrm_seen = FALSE;
-if (smtp_receive_timeout > 0) ALARM(smtp_receive_timeout);
+if (smtp_receive_timeout > 0) alarm(smtp_receive_timeout);
 do
   rc = gnutls_handshake(state->session);
 while (rc == GNUTLS_E_AGAIN ||  rc == GNUTLS_E_INTERRUPTED && !sigalrm_seen);
-ALARM_CLR(0);
+alarm(0);
 
 if (rc != GNUTLS_E_SUCCESS)
   {
@@ -2118,12 +2072,12 @@ if (rc != GNUTLS_E_SUCCESS)
 
   if (sigalrm_seen)
     {
-    tls_error(US"gnutls_handshake", US"timed out", NULL, errstr);
+    tls_error(US"gnutls_handshake", "timed out", NULL, errstr);
     gnutls_db_remove_session(state->session);
     }
   else
     {
-    tls_error(US"gnutls_handshake", US gnutls_strerror(rc), NULL, errstr);
+    tls_error(US"gnutls_handshake", gnutls_strerror(rc), NULL, errstr);
     (void) gnutls_alert_send_appropriate(state->session, rc);
     gnutls_deinit(state->session);
     gnutls_certificate_free_credentials(state->x509_cred);
@@ -2185,7 +2139,7 @@ static void
 tls_client_setup_hostname_checks(host_item * host, exim_gnutls_state_st * state,
   smtp_transport_options_block * ob)
 {
-if (verify_check_given_host(CUSS &ob->tls_verify_cert_hostnames, host) == OK)
+if (verify_check_given_host(&ob->tls_verify_cert_hostnames, host) == OK)
   {
   state->exp_tls_verify_cert_hostnames =
 #ifdef SUPPORT_I18N
@@ -2230,7 +2184,7 @@ dane_data_len = store_get(i * sizeof(int));
 for (rr = dns_next_rr(dnsa, &dnss, RESET_ANSWERS), i = 0;
      rr;
      rr = dns_next_rr(dnsa, &dnss, RESET_NEXT)
-    ) if (rr->type == T_TLSA && rr->size > 3)
+    ) if (rr->type == T_TLSA)
   {
   const uschar * p = rr->data;
   uint8_t usage = p[0], sel = p[1], type = p[2];
@@ -2254,7 +2208,7 @@ for (rr = dns_next_rr(dnsa, &dnss, RESET_ANSWERS), i = 0;
     }
 
   tls_out.tlsa_usage |= 1<<usage;
-  dane_data[i] = CS p;
+  dane_data[i] = p;
   dane_data_len[i++] = rr->size;
   }
 
@@ -2279,7 +2233,7 @@ return TRUE;
 
 Arguments:
   fd                the fd of the connection
-  host              connected host (for messages and option-tests)
+  host              connected host (for messages)
   addr              the first address (not used)
   tb                transport (always smtp)
   tlsa_dnsa	    non-NULL, either request or require dane for this host, and
@@ -2287,33 +2241,31 @@ Arguments:
 		    Which implies cert must be requested and supplied, dane
 		    verify must pass, and cert verify irrelevant (incl.
 		    hostnames), and (caller handled) require_tls
-  tlsp		    record details of channel configuration
   errstr	    error string pointer
 
-Returns:            Pointer to TLS session context, or NULL on error
+Returns:            OK/DEFER/FAIL (because using common functions),
+                    but for a client, DEFER and FAIL have the same meaning
 */
 
-void *
+int
 tls_client_start(int fd, host_item *host,
     address_item *addr ARG_UNUSED,
     transport_instance * tb,
 #ifdef SUPPORT_DANE
     dns_answer * tlsa_dnsa,
 #endif
-    tls_support * tlsp, uschar ** errstr)
+    uschar ** errstr)
 {
-smtp_transport_options_block *ob = tb
-  ? (smtp_transport_options_block *)tb->options_block
-  : &smtp_transport_option_defaults;
+smtp_transport_options_block *ob =
+  (smtp_transport_options_block *)tb->options_block;
 int rc;
 exim_gnutls_state_st * state = NULL;
 uschar *cipher_list = NULL;
-
 #ifndef DISABLE_OCSP
 BOOL require_ocsp =
-  verify_check_given_host(CUSS &ob->hosts_require_ocsp, host) == OK;
+  verify_check_given_host(&ob->hosts_require_ocsp, host) == OK;
 BOOL request_ocsp = require_ocsp ? TRUE
-  : verify_check_given_host(CUSS &ob->hosts_request_ocsp, host) == OK;
+  : verify_check_given_host(&ob->hosts_request_ocsp, host) == OK;
 #endif
 
 DEBUG(D_tls) debug_printf("initialising GnuTLS as a client on fd %d\n", fd);
@@ -2324,7 +2276,7 @@ if (tlsa_dnsa && ob->dane_require_tls_ciphers)
   /* not using expand_check_tlsvar because not yet in state */
   if (!expand_check(ob->dane_require_tls_ciphers, US"dane_require_tls_ciphers",
       &cipher_list, errstr))
-    return NULL;
+    return DEFER;
   cipher_list = cipher_list && *cipher_list
     ? ob->dane_require_tls_ciphers : ob->tls_require_ciphers;
   }
@@ -2333,10 +2285,10 @@ if (tlsa_dnsa && ob->dane_require_tls_ciphers)
 if (!cipher_list)
   cipher_list = ob->tls_require_ciphers;
 
-if (tls_init(host, ob->tls_certificate, ob->tls_privatekey,
+if ((rc = tls_init(host, ob->tls_certificate, ob->tls_privatekey,
     ob->tls_sni, ob->tls_verify_certificates, ob->tls_crl,
-    cipher_list, &state, tlsp, errstr) != OK)
-  return NULL;
+    cipher_list, &state, errstr)) != OK)
+  return rc;
 
   {
   int dh_min_bits = ob->tls_dh_min_bits;
@@ -2373,7 +2325,7 @@ else
 	  && !ob->tls_verify_hosts
 	  && (!ob->tls_try_verify_hosts || !*ob->tls_try_verify_hosts)
 	  )
-	|| verify_check_given_host(CUSS &ob->tls_verify_hosts, host) == OK
+	|| verify_check_given_host(&ob->tls_verify_hosts, host) == OK
        )
   {
   tls_client_setup_hostname_checks(host, state, ob);
@@ -2382,7 +2334,7 @@ else
   state->verify_requirement = VERIFY_REQUIRED;
   gnutls_certificate_server_set_request(state->session, GNUTLS_CERT_REQUIRE);
   }
-else if (verify_check_given_host(CUSS &ob->tls_try_verify_hosts, host) == OK)
+else if (verify_check_given_host(&ob->tls_try_verify_hosts, host) == OK)
   {
   tls_client_setup_hostname_checks(host, state, ob);
   DEBUG(D_tls)
@@ -2405,16 +2357,14 @@ if (request_ocsp)
   DEBUG(D_tls) debug_printf("TLS: will request OCSP stapling\n");
   if ((rc = gnutls_ocsp_status_request_enable_client(state->session,
 		    NULL, 0, NULL)) != OK)
-    {
-    tls_error(US"cert-status-req", US gnutls_strerror(rc), state->host, errstr);
-    return NULL;
-    }
-  tlsp->ocsp = OCSP_NOT_RESP;
+    return tls_error(US"cert-status-req",
+		    gnutls_strerror(rc), state->host, errstr);
+  tls_out.ocsp = OCSP_NOT_RESP;
   }
 #endif
 
 #ifndef DISABLE_EVENT
-if (tb && tb->event_action)
+if (tb->event_action)
   {
   state->event_action = tb->event_action;
   gnutls_session_set_ptr(state->session, state);
@@ -2430,33 +2380,27 @@ DEBUG(D_tls) debug_printf("about to gnutls_handshake\n");
 /* There doesn't seem to be a built-in timeout on connection. */
 
 sigalrm_seen = FALSE;
-ALARM(ob->command_timeout);
+alarm(ob->command_timeout);
 do
   rc = gnutls_handshake(state->session);
 while (rc == GNUTLS_E_AGAIN || rc == GNUTLS_E_INTERRUPTED && !sigalrm_seen);
-ALARM_CLR(0);
+alarm(0);
 
 if (rc != GNUTLS_E_SUCCESS)
-  {
   if (sigalrm_seen)
     {
     gnutls_alert_send(state->session, GNUTLS_AL_FATAL, GNUTLS_A_USER_CANCELED);
-    tls_error(US"gnutls_handshake", US"timed out", state->host, errstr);
+    return tls_error(US"gnutls_handshake", "timed out", state->host, errstr);
     }
   else
-    tls_error(US"gnutls_handshake", US gnutls_strerror(rc), state->host, errstr);
-  return NULL;
-  }
+    return tls_error(US"gnutls_handshake", gnutls_strerror(rc), state->host, errstr);
 
 DEBUG(D_tls) debug_printf("gnutls_handshake was successful\n");
 
 /* Verify late */
 
 if (!verify_certificate(state, errstr))
-  {
-  tls_error(US"certificate verification failed", *errstr, state->host, errstr);
-  return NULL;
-  }
+  return tls_error(US"certificate verification failed", *errstr, state->host, errstr);
 
 #ifndef DISABLE_OCSP
 if (require_ocsp)
@@ -2476,30 +2420,29 @@ if (require_ocsp)
       gnutls_free(printed.data);
       }
     else
-      (void) tls_error(US"ocsp decode", US gnutls_strerror(rc), state->host, errstr);
+      (void) tls_error(US"ocsp decode", gnutls_strerror(rc), state->host, errstr);
     }
 
   if (gnutls_ocsp_status_request_is_checked(state->session, 0) == 0)
     {
-    tlsp->ocsp = OCSP_FAILED;
-    tls_error(US"certificate status check failed", NULL, state->host, errstr);
-    return NULL;
+    tls_out.ocsp = OCSP_FAILED;
+    return tls_error(US"certificate status check failed", NULL, state->host, errstr);
     }
   DEBUG(D_tls) debug_printf("Passed OCSP checking\n");
-  tlsp->ocsp = OCSP_VFIED;
+  tls_out.ocsp = OCSP_VFIED;
   }
 #endif
 
 /* Figure out peer DN, and if authenticated, etc. */
 
-if (peer_status(state, errstr) != OK)
-  return NULL;
+if ((rc = peer_status(state, errstr)) != OK)
+  return rc;
 
 /* Sets various Exim expansion variables; may need to adjust for ACL callouts */
 
 extract_exim_vars_from_tls_state(state);
 
-return state;
+return OK;
 }
 
 
@@ -2514,38 +2457,42 @@ daemon, to shut down the TLS library, without actually doing a shutdown (which
 would tamper with the TLS session in the parent process).
 
 Arguments:
-  ct_ctx	client context pointer, or NULL for the one global server context
   shutdown	1 if TLS close-alert is to be sent,
-		2 if also response to be waited for
+ 		2 if also response to be waited for
 
 Returns:     nothing
 */
 
 void
-tls_close(void * ct_ctx, int shutdown)
+tls_close(BOOL is_server, int shutdown)
 {
-exim_gnutls_state_st * state = ct_ctx ? ct_ctx : &state_server;
+exim_gnutls_state_st *state = is_server ? &state_server : &state_client;
 
-if (!state->tlsp || state->tlsp->active.sock < 0) return;  /* TLS was not active */
+if (!state->tlsp || state->tlsp->active < 0) return;  /* TLS was not active */
 
 if (shutdown)
   {
   DEBUG(D_tls) debug_printf("tls_close(): shutting down TLS%s\n",
     shutdown > 1 ? " (with response-wait)" : "");
 
-  ALARM(2);
+  alarm(2);
   gnutls_bye(state->session, shutdown > 1 ? GNUTLS_SHUT_RDWR : GNUTLS_SHUT_WR);
-  ALARM_CLR(0);
+  alarm(0);
   }
 
 gnutls_deinit(state->session);
 gnutls_certificate_free_credentials(state->x509_cred);
 
 
-state->tlsp->active.sock = -1;
-state->tlsp->active.tls_ctx = NULL;
+state->tlsp->active = -1;
 if (state->xfer_buffer) store_free(state->xfer_buffer);
 memcpy(state, &exim_gnutls_state_init, sizeof(exim_gnutls_state_init));
+
+if (!state_server.session && !state_client.session)
+  {
+  gnutls_global_deinit();
+  exim_gnutls_base_init_done = FALSE;
+  }
 }
 
 
@@ -2561,27 +2508,15 @@ DEBUG(D_tls) debug_printf("Calling gnutls_record_recv(%p, %p, %u)\n",
   state->session, state->xfer_buffer, ssl_xfer_buffer_size);
 
 sigalrm_seen = FALSE;
-if (smtp_receive_timeout > 0) ALARM(smtp_receive_timeout);
+if (smtp_receive_timeout > 0) alarm(smtp_receive_timeout);
+inbytes = gnutls_record_recv(state->session, state->xfer_buffer,
+  MIN(ssl_xfer_buffer_size, lim));
+alarm(0);
 
-do
-  inbytes = gnutls_record_recv(state->session, state->xfer_buffer,
-    MIN(ssl_xfer_buffer_size, lim));
-while (inbytes == GNUTLS_E_AGAIN);
-
-if (smtp_receive_timeout > 0) ALARM_CLR(0);
-
-if (had_command_timeout)		/* set by signal handler */
-  smtp_command_timeout_exit();		/* does not return */
-if (had_command_sigterm)
-  smtp_command_sigterm_exit();
-if (had_data_timeout)
-  smtp_data_timeout_exit();
-if (had_data_sigint)
-  smtp_data_sigint_exit();
-
-/* Timeouts do not get this far.  A zero-byte return appears to mean that the
-TLS session has been closed down, not that the socket itself has been closed
-down. Revert to non-TLS handling. */
+/* Timeouts do not get this far; see command_timeout_handler().
+   A zero-byte return appears to mean that the TLS session has been
+   closed down, not that the socket itself has been closed down. Revert to
+   non-TLS handling. */
 
 if (sigalrm_seen)
   {
@@ -2606,8 +2541,7 @@ else if (inbytes == 0)
   gnutls_certificate_free_credentials(state->x509_cred);
 
   state->session = NULL;
-  state->tlsp->active.sock = -1;
-  state->tlsp->active.tls_ctx = NULL;
+  state->tlsp->active = -1;
   state->tlsp->bits = 0;
   state->tlsp->certificate_verified = FALSE;
   tls_channelbinding_b64 = NULL;
@@ -2622,7 +2556,6 @@ else if (inbytes == 0)
 
 else if (inbytes < 0)
   {
-  DEBUG(D_tls) debug_printf("%s: err from gnutls_record_recv(\n", __FUNCTION__);
   record_io_error(state, (int) inbytes, US"recv", NULL);
   state->xfer_error = TRUE;
   return FALSE;
@@ -2645,7 +2578,7 @@ Only used by the server-side TLS.
 
 This feeds DKIM and should be used for all message-body reads.
 
-Arguments:  lim		Maximum amount to read/buffer
+Arguments:  lim		Maximum amount to read/bufffer
 Returns:    the next character or EOF
 */
 
@@ -2717,18 +2650,17 @@ return state_server.xfer_buffer_lwm < state_server.xfer_buffer_hwm
 then the caller must feed DKIM.
 
 Arguments:
-  ct_ctx    client context pointer, or NULL for the one global server context
   buff      buffer of data
   len       size of buffer
 
 Returns:    the number of bytes read
-            -1 after a failed read, including EOF
+            -1 after a failed read
 */
 
 int
-tls_read(void * ct_ctx, uschar *buff, size_t len)
+tls_read(BOOL is_server, uschar *buff, size_t len)
 {
-exim_gnutls_state_st * state = ct_ctx ? ct_ctx : &state_server;
+exim_gnutls_state_st *state = is_server ? &state_server : &state_client;
 ssize_t inbytes;
 
 if (len > INT_MAX)
@@ -2744,20 +2676,13 @@ DEBUG(D_tls)
   debug_printf("Calling gnutls_record_recv(%p, %p, " SIZE_T_FMT ")\n",
       state->session, buff, len);
 
-do
-  inbytes = gnutls_record_recv(state->session, buff, len);
-while (inbytes == GNUTLS_E_AGAIN);
-
+inbytes = gnutls_record_recv(state->session, buff, len);
 if (inbytes > 0) return inbytes;
 if (inbytes == 0)
   {
   DEBUG(D_tls) debug_printf("Got TLS_EOF\n");
   }
-else
-  {
-  DEBUG(D_tls) debug_printf("%s: err from gnutls_record_recv(\n", __FUNCTION__);
-  record_io_error(state, (int)inbytes, US"recv", NULL);
-  }
+else record_io_error(state, (int)inbytes, US"recv", NULL);
 
 return -1;
 }
@@ -2771,7 +2696,7 @@ return -1;
 
 /*
 Arguments:
-  ct_ctx    client context pointer, or NULL for the one global server context
+  is_server channel specifier
   buff      buffer of data
   len       number of bytes
   more	    more data expected soon
@@ -2781,11 +2706,11 @@ Returns:    the number of bytes after a successful write,
 */
 
 int
-tls_write(void * ct_ctx, const uschar * buff, size_t len, BOOL more)
+tls_write(BOOL is_server, const uschar *buff, size_t len, BOOL more)
 {
 ssize_t outbytes;
 size_t left = len;
-exim_gnutls_state_st * state = ct_ctx ? ct_ctx : &state_server;
+exim_gnutls_state_st *state = is_server ? &state_server : &state_client;
 #ifdef SUPPORT_CORK
 static BOOL corked = FALSE;
 
@@ -2799,15 +2724,11 @@ while (left > 0)
   {
   DEBUG(D_tls) debug_printf("gnutls_record_send(SSL, %p, " SIZE_T_FMT ")\n",
       buff, left);
-
-  do
-    outbytes = gnutls_record_send(state->session, buff, left);
-  while (outbytes == GNUTLS_E_AGAIN);
+  outbytes = gnutls_record_send(state->session, buff, left);
 
   DEBUG(D_tls) debug_printf("outbytes=" SSIZE_T_FMT "\n", outbytes);
   if (outbytes < 0)
     {
-    DEBUG(D_tls) debug_printf("%s: gnutls_record_send err\n", __FUNCTION__);
     record_io_error(state, outbytes, US"send", NULL);
     return -1;
     }
