@@ -2,15 +2,11 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
-
-#ifdef TRANSPORT_AUTOREPLY	/* Remainder of file */
 #include "autoreply.h"
 
 
@@ -20,27 +16,44 @@ order (note that "_" comes before the lower case letters). Those starting
 with "*" are not settable by the user but are used by the option-reading
 software for alternative value types. Some options are publicly visible and so
 are stored in the driver instance block. These are flagged with opt_public. */
-#define LOFF(field) OPT_OFF(autoreply_transport_options_block, field)
 
 optionlist autoreply_transport_options[] = {
-  { "bcc",               opt_stringptr,	LOFF(bcc) },
-  { "cc",                opt_stringptr,	LOFF(cc) },
-  { "file",              opt_stringptr,	LOFF(file) },
-  { "file_expand",     opt_bool,	LOFF(file_expand) },
-  { "file_optional",     opt_bool,	LOFF(file_optional) },
-  { "from",              opt_stringptr,	LOFF(from) },
-  { "headers",           opt_stringptr,	LOFF(headers) },
-  { "log",               opt_stringptr,	LOFF(logfile) },
-  { "mode",              opt_octint,	LOFF(mode) },
-  { "never_mail",        opt_stringptr,	LOFF(never_mail) },
-  { "once",              opt_stringptr,	LOFF(oncelog) },
-  { "once_file_size",    opt_int,	LOFF(once_file_size) },
-  { "once_repeat",       opt_stringptr,	LOFF(once_repeat) },
-  { "reply_to",          opt_stringptr,	LOFF(reply_to) },
-  { "return_message",    opt_bool,	LOFF(return_message) },
-  { "subject",           opt_stringptr,	LOFF(subject) },
-  { "text",              opt_stringptr,	LOFF(text) },
-  { "to",                opt_stringptr,	LOFF(to) },
+  { "bcc",               opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, bcc) },
+  { "cc",                opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, cc) },
+  { "file",              opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, file) },
+  { "file_expand",     opt_bool,
+      (void *)offsetof(autoreply_transport_options_block, file_expand) },
+  { "file_optional",     opt_bool,
+      (void *)offsetof(autoreply_transport_options_block, file_optional) },
+  { "from",              opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, from) },
+  { "headers",           opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, headers) },
+  { "log",               opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, logfile) },
+  { "mode",              opt_octint,
+      (void *)offsetof(autoreply_transport_options_block, mode) },
+  { "never_mail",        opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, never_mail) },
+  { "once",              opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, oncelog) },
+  { "once_file_size",    opt_int,
+      (void *)offsetof(autoreply_transport_options_block, once_file_size) },
+  { "once_repeat",       opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, once_repeat) },
+  { "reply_to",          opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, reply_to) },
+  { "return_message",    opt_bool,
+      (void *)offsetof(autoreply_transport_options_block, return_message) },
+  { "subject",           opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, subject) },
+  { "text",              opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, text) },
+  { "to",                opt_stringptr,
+      (void *)offsetof(autoreply_transport_options_block, to) },
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -54,17 +67,33 @@ int autoreply_transport_options_count =
 
 /* Dummy values */
 autoreply_transport_options_block autoreply_transport_option_defaults = {0};
-void autoreply_transport_init(driver_instance *tblock) {}
+void autoreply_transport_init(transport_instance *tblock) {}
 BOOL autoreply_transport_entry(transport_instance *tblock, address_item *addr) {return FALSE;}
 
 #else   /*!MACRO_PREDEF*/
 
 
-/* Default private options block for the autoreply transport.
-All non-mentioned lements zero/null/false. */
+/* Default private options block for the autoreply transport. */
 
 autoreply_transport_options_block autoreply_transport_option_defaults = {
-  .mode = 0600,
+  NULL,           /* from */
+  NULL,           /* reply_to */
+  NULL,           /* to */
+  NULL,           /* cc */
+  NULL,           /* bcc */
+  NULL,           /* subject */
+  NULL,           /* headers */
+  NULL,           /* text */
+  NULL,           /* file */
+  NULL,           /* logfile */
+  NULL,           /* oncelog */
+  NULL,           /* once_repeat */
+  NULL,           /* never_mail */
+  0600,           /* mode */
+  0,              /* once_file_size */
+  FALSE,          /* file_expand */
+  FALSE,          /* file_optional */
+  FALSE           /* return message */
 };
 
 
@@ -84,9 +113,8 @@ enable consistency checks to be done, or anything else that needs
 to be set up. */
 
 void
-autoreply_transport_init(driver_instance * t)
+autoreply_transport_init(transport_instance *tblock)
 {
-const transport_instance * tblock = (transport_instance *)t;
 /*
 autoreply_transport_options_block *ob =
   (autoreply_transport_options_block *)(tblock->options_block);
@@ -95,8 +123,8 @@ autoreply_transport_options_block *ob =
 /* If a fixed uid field is set, then a gid field must also be set. */
 
 if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
-  log_write_die(0, LOG_CONFIG,
-    "user set without group for the %s transport", t->name);
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    "user set without group for the %s transport", tblock->name);
 }
 
 
@@ -123,21 +151,21 @@ Returns:     expanded string if expansion succeeds;
              NULL otherwise
 */
 
-static const uschar *
-checkexpand(const uschar * s, address_item * addr, const uschar * name,
-  int type)
+static uschar *
+checkexpand(uschar *s, address_item *addr, uschar *name, int type)
 {
-const uschar * ss = expand_string(s);
+uschar *t;
+uschar *ss = expand_string(s);
 
-if (!ss)
+if (ss == NULL)
   {
   addr->transport_return = FAIL;
-  addr->message = string_sprintf("Expansion of %q failed in %s transport: "
+  addr->message = string_sprintf("Expansion of \"%s\" failed in %s transport: "
     "%s", s, name, expand_string_message);
   return NULL;
   }
 
-if (type != cke_text) for (const uschar * t = ss; *t; t++)
+if (type != cke_text) for (t = ss; *t != 0; t++)
   {
   int c = *t;
   const uschar * sp;
@@ -145,7 +173,7 @@ if (type != cke_text) for (const uschar * t = ss; *t; t++)
   if (type == cke_hdr && c == '\n' && (t[1] == ' ' || t[1] == '\t')) continue;
   sp = string_printing(s);
   addr->transport_return = FAIL;
-  addr->message = string_sprintf("Expansion of %q in %s transport "
+  addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
     "contains non-printing character %d", sp, name, c);
   return NULL;
   }
@@ -164,21 +192,18 @@ return ss;
 list. Any that are found are removed.
 
 Arguments:
-  list        list of addresses to be checked
+  listptr     points to the list of addresses
   never_mail  an address list, already expanded
 
-Returns:      edited replacement address list, or NULL, or original
+Returns:      nothing
 */
 
-static const uschar *
-check_never_mail(const uschar * list, const uschar * never_mail)
+static void
+check_never_mail(uschar **listptr, const uschar *never_mail)
 {
-rmark reset_point = store_mark();
-uschar * newlist = string_copy(list);
-uschar * s = newlist;
-BOOL hit = FALSE;
+uschar *s = *listptr;
 
-while (*s)
+while (*s != 0)
   {
   uschar *error, *next;
   uschar *e = parse_find_address_end(s, FALSE);
@@ -195,7 +220,7 @@ while (*s)
   /* If there is some kind of syntax error, just give up on this header
   line. */
 
-  if (!next) break;
+  if (next == NULL) break;
 
   /* See if the address is on the never_mail list */
 
@@ -212,7 +237,6 @@ while (*s)
     {
     DEBUG(D_transport)
       debug_printf("discarding recipient %s (matched never_mail)\n", next);
-    hit = TRUE;
     if (terminator == ',') e++;
     memmove(s, e, Ustrlen(e) + 1);
     }
@@ -223,30 +247,18 @@ while (*s)
     }
   }
 
-/* If no addresses were removed, retrieve the memory used and return
-the original. */
-
-if (!hit)
-  {
-  store_reset(reset_point);
-  return list;
-  }
-
 /* Check to see if we removed the last address, leaving a terminating comma
 that needs to be removed */
 
-s = newlist + Ustrlen(newlist);
-while (s > newlist && (isspace(s[-1]) || s[-1] == ',')) s--;
-*s = '\0';
+s = *listptr + Ustrlen(*listptr);
+while (s > *listptr && (isspace(s[-1]) || s[-1] == ',')) s--;
+*s = 0;
 
-/* Check to see if there any addresses left; if not, return NULL */
+/* Check to see if there any addresses left; if not, set NULL */
 
-s = newlist;
-if (Uskip_whitespace(&s))
-  return newlist;
-
-store_reset(reset_point);
-return NULL;
+s = *listptr;
+while (s != 0 && isspace(*s)) s++;
+if (*s == 0) *listptr = NULL;
 }
 
 
@@ -264,22 +276,27 @@ autoreply_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address we are working on */
 {
-autoreply_transport_options_block * ob = tblock->drinst.options_block;
-const uschar * trname = tblock->drinst.name;
 int fd, pid, rc;
 int cache_fd = -1;
 int cache_size = 0;
 int add_size = 0;
-EXIM_DB * dbm_file = NULL;
+EXIM_DB *dbm_file = NULL;
 BOOL file_expand, return_message;
-const uschar * from, * reply_to, * to, * cc, * bcc, * subject, * headers;
-const uschar * text, * file, * logfile, * oncelog;
-uschar * cache_buff = NULL, * cache_time = NULL, * message_id = NULL;
-header_line * h;
-time_t now = time(NULL), once_repeat_sec = 0;
-FILE * ff = NULL, * fp;
+uschar *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
+uschar *logfile, *oncelog;
+uschar *cache_buff = NULL;
+uschar *cache_time = NULL;
+uschar *message_id = NULL;
+header_line *h;
+time_t now = time(NULL);
+time_t once_repeat_sec = 0;
+FILE *f;
+FILE *ff = NULL;
 
-DEBUG(D_transport) debug_printf("%s transport entered\n", trname);
+autoreply_transport_options_block *ob =
+  (autoreply_transport_options_block *)(tblock->options_block);
+
+DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 
 /* Set up for the good case */
 
@@ -291,7 +308,7 @@ from that block. It has typically been set up by a mail filter processing
 router. Otherwise, the data must be supplied by this transport, and
 it has to be expanded here. */
 
-if (addr->reply)
+if (addr->reply != NULL)
   {
   DEBUG(D_transport) debug_printf("taking data from address\n");
   from = addr->reply->from;
@@ -312,47 +329,49 @@ if (addr->reply)
   }
 else
   {
-  const uschar * oncerepeat;
+  uschar *oncerepeat = ob->once_repeat;
 
   DEBUG(D_transport) debug_printf("taking data from transport\n");
-  GET_OPTION("once_repeat");	oncerepeat = ob->once_repeat;
-  GET_OPTION("from"); 		from = ob->from;
-  GET_OPTION("reply_to");	reply_to = ob->reply_to;
-  GET_OPTION("to");		to = ob->to;
-  GET_OPTION("cc");		cc = ob->cc;
-  GET_OPTION("bcc");		bcc = ob->bcc;
-  GET_OPTION("subject");	subject = ob->subject;
-  GET_OPTION("headers"); 	headers = ob->headers;
-  GET_OPTION("text");		text = ob->text;
-  GET_OPTION("file");		file = ob->file;
-  GET_OPTION("log");		logfile = ob->logfile;
-  GET_OPTION("once");		oncelog = ob->oncelog;
+  from = ob->from;
+  reply_to = ob->reply_to;
+  to = ob->to;
+  cc = ob->cc;
+  bcc = ob->bcc;
+  subject = ob->subject;
+  headers = ob->headers;
+  text = ob->text;
+  file = ob->file;
+  logfile = ob->logfile;
+  oncelog = ob->oncelog;
   file_expand = ob->file_expand;
   return_message = ob->return_message;
 
-  if (  from && !(from = checkexpand(from, addr, trname, cke_hdr))
-     || reply_to && !(reply_to = checkexpand(reply_to, addr, trname, cke_hdr))
-     || to && !(to = checkexpand(to, addr, trname, cke_hdr))
-     || cc && !(cc = checkexpand(cc, addr, trname, cke_hdr))
-     || bcc && !(bcc = checkexpand(bcc, addr, trname, cke_hdr))
-     || subject && !(subject = checkexpand(subject, addr, trname, cke_hdr))
-     || headers && !(headers = checkexpand(headers, addr, trname, cke_text))
-     || text && !(text = checkexpand(text, addr, trname, cke_text))
-     || file && !(file = checkexpand(file, addr, trname, cke_file))
-     || logfile && !(logfile = checkexpand(logfile, addr, trname, cke_file))
-     || oncelog && !(oncelog = checkexpand(oncelog, addr, trname, cke_file))
-     || oncerepeat && !(oncerepeat = checkexpand(oncerepeat, addr, trname, cke_file))
+  if (  from && !(from = checkexpand(from, addr, tblock->name, cke_hdr))
+     || reply_to && !(reply_to = checkexpand(reply_to, addr, tblock->name, cke_hdr))
+     || to && !(to = checkexpand(to, addr, tblock->name, cke_hdr))
+     || cc && !(cc = checkexpand(cc, addr, tblock->name, cke_hdr))
+     || bcc && !(bcc = checkexpand(bcc, addr, tblock->name, cke_hdr))
+     || subject && !(subject = checkexpand(subject, addr, tblock->name, cke_hdr))
+     || headers && !(headers = checkexpand(headers, addr, tblock->name, cke_text))
+     || text && !(text = checkexpand(text, addr, tblock->name, cke_text))
+     || file && !(file = checkexpand(file, addr, tblock->name, cke_file))
+     || logfile && !(logfile = checkexpand(logfile, addr, tblock->name, cke_file))
+     || oncelog && !(oncelog = checkexpand(oncelog, addr, tblock->name, cke_file))
+     || oncerepeat && !(oncerepeat = checkexpand(oncerepeat, addr, tblock->name, cke_file))
      )
     return FALSE;
 
   if (oncerepeat)
-    if ((once_repeat_sec = readconf_readtime(oncerepeat, 0, FALSE)) < 0)
+    {
+    once_repeat_sec = readconf_readtime(oncerepeat, 0, FALSE);
+    if (once_repeat_sec < 0)
       {
       addr->transport_return = FAIL;
-      addr->message = string_sprintf("Invalid time value %q for "
-        "\"once_repeat\" in %s transport", oncerepeat, trname);
+      addr->message = string_sprintf("Invalid time value \"%s\" for "
+        "\"once_repeat\" in %s transport", oncerepeat, tblock->name);
       return FALSE;
       }
+    }
   }
 
 /* If the never_mail option is set, we have to scan all the recipients and
@@ -360,19 +379,19 @@ remove those that match. */
 
 if (ob->never_mail)
   {
-  const uschar * never_mail = expand_string(ob->never_mail);
+  const uschar *never_mail = expand_string(ob->never_mail);
 
   if (!never_mail)
     {
     addr->transport_return = FAIL;
-    addr->message = string_sprintf("Failed to expand %q for "
-      "\"never_mail\" in %s transport", ob->never_mail, trname);
+    addr->message = string_sprintf("Failed to expand \"%s\" for "
+      "\"never_mail\" in %s transport", ob->never_mail, tblock->name);
     return FALSE;
     }
 
-  if (to) to = check_never_mail(to, never_mail);
-  if (cc) cc = check_never_mail(cc, never_mail);
-  if (bcc) bcc = check_never_mail(bcc, never_mail);
+  if (to) check_never_mail(&to, never_mail);
+  if (cc) check_never_mail(&cc, never_mail);
+  if (bcc) check_never_mail(&bcc, never_mail);
 
   if (!to && !cc && !bcc)
     {
@@ -384,11 +403,11 @@ if (ob->never_mail)
 
 /* If the -N option is set, can't do any more. */
 
-if (f.dont_deliver)
+if (dont_deliver)
   {
   DEBUG(D_transport)
     debug_printf("*** delivery by %s transport bypassed by -N option\n",
-      trname);
+      tblock->name);
   return FALSE;
   }
 
@@ -400,34 +419,25 @@ recipient, the effect might not be quite as envisaged. If once_file_size is
 set, instead of a dbm file, we use a regular file containing a circular buffer
 recipient cache. */
 
-if (oncelog && *oncelog && to)
+if (oncelog && *oncelog != 0 && to)
   {
   time_t then = 0;
 
-  if (is_tainted(oncelog))
-    {
-    addr->transport_return = DEFER;
-    addr->basic_errno = EACCES;
-    addr->message = string_sprintf("Tainted '%s' (once file for %s transport)"
-      " not permitted", oncelog, trname);
-    goto END_OFF;
-    }
-
   /* Handle fixed-size cache file. */
 
   if (ob->once_file_size > 0)
     {
-    uschar * nextp;
+    uschar * p, * nextp;
     struct stat statbuf;
-
     cache_fd = Uopen(oncelog, O_CREAT|O_RDWR, ob->mode);
+
     if (cache_fd < 0 || fstat(cache_fd, &statbuf) != 0)
       {
       addr->transport_return = DEFER;
-      addr->basic_errno = errno;
       addr->message = string_sprintf("Failed to %s \"once\" file %s when "
         "sending message from %s transport: %s",
-        cache_fd < 0 ? "open" : "stat", oncelog, trname, strerror(errno));
+        (cache_fd < 0)? "open" : "stat", oncelog, tblock->name,
+          strerror(errno));
       goto END_OFF;
       }
 
@@ -437,7 +447,7 @@ if (oncelog && *oncelog && to)
 
     cache_size = statbuf.st_size;
     add_size = sizeof(time_t) + Ustrlen(to) + 1;
-    cache_buff = store_get(cache_size + add_size, oncelog);
+    cache_buff = store_get(cache_size + add_size);
 
     if (read(cache_fd, cache_buff, cache_size) != cache_size)
       {
@@ -454,7 +464,7 @@ if (oncelog && *oncelog && to)
     zero. If we find a match, put the time into "then", and the place where it
     was found into "cache_time". Otherwise, "then" is left at zero. */
 
-    for (uschar * p = cache_buff; p < cache_buff + cache_size; p = nextp)
+    for (p = cache_buff; p < cache_buff + cache_size; p = nextp)
       {
       uschar *s = p + sizeof(time_t);
       nextp = s + Ustrlen(s) + 1;
@@ -472,26 +482,39 @@ if (oncelog && *oncelog && to)
   else
     {
     EXIM_DATUM key_datum, result_datum;
-    const uschar * s = Ustrrchr(oncelog, '/');
-    const uschar * dirname = s ? string_copyn(oncelog, s - oncelog) : NULL;
+    uschar * dirname = string_copy(oncelog);
+    uschar * s;
 
-    if (!(dbm_file = exim_dbopen(oncelog, dirname, O_RDWR|O_CREAT, ob->mode)))
+    if ((s = Ustrrchr(dirname, '/'))) *s = '\0';
+    EXIM_DBOPEN(oncelog, dirname, O_RDWR|O_CREAT, ob->mode, &dbm_file);
+    if (!dbm_file)
       {
       addr->transport_return = DEFER;
-      addr->basic_errno = errno;
       addr->message = string_sprintf("Failed to open %s file %s when sending "
-        "message from %s transport: %s", EXIM_DBTYPE, oncelog, trname,
+        "message from %s transport: %s", EXIM_DBTYPE, oncelog, tblock->name,
         strerror(errno));
       goto END_OFF;
       }
 
-    exim_datum_init(&key_datum);        /* Some DBM libraries need datums */
-    exim_datum_init(&result_datum);     /* to be cleared */
-    exim_datum_data_set(&key_datum, (void *) to);
-    exim_datum_size_set(&key_datum, Ustrlen(to) + 1);
+    EXIM_DATUM_INIT(key_datum);        /* Some DBM libraries need datums */
+    EXIM_DATUM_INIT(result_datum);     /* to be cleared */
+    EXIM_DATUM_DATA(key_datum) = CS to;
+    EXIM_DATUM_SIZE(key_datum) = Ustrlen(to) + 1;
 
-    if (exim_dbget(dbm_file, &key_datum, &result_datum))
-      memcpy(&then, exim_datum_data_get(&result_datum), sizeof(time_t));
+    if (EXIM_DBGET(dbm_file, key_datum, result_datum))
+      {
+      /* If the datum size is that of a binary time, we are in the new world
+      where messages are sent periodically. Otherwise the file is an old one,
+      where the datum was filled with a tod_log time, which is assumed to be
+      different in size. For that, only one message is ever sent. This change
+      introduced at Exim 3.00. In a couple of years' time the test on the size
+      can be abolished. */
+
+      if (EXIM_DATUM_SIZE(result_datum) == sizeof(time_t))
+        memcpy(&then, EXIM_DATUM_DATA(result_datum), sizeof(time_t));
+      else
+        then = now;
+      }
     }
 
   /* Either "then" is set zero, if no message has yet been sent, or it
@@ -500,15 +523,6 @@ if (oncelog && *oncelog && to)
   if (then != 0 && (once_repeat_sec <= 0 || now - then < once_repeat_sec))
     {
     int log_fd;
-    if (is_tainted(logfile))
-      {
-      addr->transport_return = DEFER;
-      addr->basic_errno = EACCES;
-      addr->message = string_sprintf("Tainted '%s' (logfile for %s transport)"
-	" not permitted", logfile, trname);
-      goto END_OFF;
-      }
-
     DEBUG(D_transport) debug_printf("message previously sent to %s%s\n", to,
       (once_repeat_sec > 0)? " and repeat time not reached" : "");
     log_fd = logfile ? Uopen(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode) : -1;
@@ -520,7 +534,7 @@ if (oncelog && *oncelog && to)
       if(write(log_fd, log_buffer, ptr - log_buffer) != ptr-log_buffer
         || close(log_fd))
         DEBUG(D_transport) debug_printf("Problem writing log file %s for %s "
-          "transport\n", logfile, trname);
+          "transport\n", logfile, tblock->name);
       }
     goto END_OFF;
     }
@@ -530,38 +544,32 @@ if (oncelog && *oncelog && to)
   }
 
 /* We are going to send a message. Ensure any requested file is available. */
+
 if (file)
   {
-  if (is_tainted(file))
+  ff = Ufopen(file, "rb");
+  if (!ff && !ob->file_optional)
     {
     addr->transport_return = DEFER;
-    addr->basic_errno = EACCES;
-    addr->message = string_sprintf("Tainted '%s' (file for %s transport)"
-      " not permitted", file, trname);
-    return FALSE;
-    }
-  if (!(ff = Ufopen(file, "rb")) && !ob->file_optional)
-    {
-    addr->transport_return = DEFER;
-    addr->basic_errno = errno;
     addr->message = string_sprintf("Failed to open file %s when sending "
-      "message from %s transport: %s", file, trname, strerror(errno));
+      "message from %s transport: %s", file, tblock->name, strerror(errno));
     return FALSE;
     }
   }
 
 /* Make a subprocess to send the message */
 
-if ((pid = child_open_exim(&fd, US"autoreply")) < 0)
-  {
-  /* Creation of child failed; defer this delivery. */
+pid = child_open_exim(&fd);
 
+/* Creation of child failed; defer this delivery. */
+
+if (pid < 0)
+  {
   addr->transport_return = DEFER;
-  addr->basic_errno = errno;
   addr->message = string_sprintf("Failed to create child process to send "
-    "message from %s transport: %s", trname, strerror(errno));
+    "message from %s transport: %s", tblock->name, strerror(errno));
   DEBUG(D_transport) debug_printf("%s\n", addr->message);
-  if (dbm_file) exim_dbclose(dbm_file);
+  if (dbm_file) EXIM_DBCLOSE(dbm_file);
   return FALSE;
   }
 
@@ -569,14 +577,14 @@ if ((pid = child_open_exim(&fd, US"autoreply")) < 0)
 as the -t option is used. The "headers" stuff *must* be last in case there
 are newlines in it which might, if placed earlier, screw up other headers. */
 
-fp = fdopen(fd, "wb");
+f = fdopen(fd, "wb");
 
-if (from) fprintf(fp, "From: %s\n", from);
-if (reply_to) fprintf(fp, "Reply-To: %s\n", reply_to);
-if (to) fprintf(fp, "To: %s\n", to);
-if (cc) fprintf(fp, "Cc: %s\n", cc);
-if (bcc) fprintf(fp, "Bcc: %s\n", bcc);
-if (subject) fprintf(fp, "Subject: %s\n", subject);
+if (from) fprintf(f, "From: %s\n", from);
+if (reply_to) fprintf(f, "Reply-To: %s\n", reply_to);
+if (to) fprintf(f, "To: %s\n", to);
+if (cc) fprintf(f, "Cc: %s\n", cc);
+if (bcc) fprintf(f, "Bcc: %s\n", bcc);
+if (subject) fprintf(f, "Subject: %s\n", subject);
 
 /* Generate In-Reply-To from the message_id header; there should
 always be one, but code defensively. */
@@ -587,41 +595,90 @@ for (h = header_list; h; h = h->next)
 if (h)
   {
   message_id = Ustrchr(h->text, ':') + 1;
-  Uskip_whitespace(&message_id);
-  fprintf(fp, "In-Reply-To: %s", message_id);
+  while (isspace(*message_id)) message_id++;
+  fprintf(f, "In-Reply-To: %s", message_id);
   }
 
-moan_write_references(fp, message_id);
+/* Generate a References header if there is at least one of Message-ID:,
+References:, or In-Reply-To: (see RFC 2822). */
+
+for (h = header_list; h; h = h->next)
+  if (h->type != htype_old && strncmpic(US"References:", h->text, 11) == 0)
+    break;
+
+if (!h)
+  for (h = header_list; h; h = h->next)
+    if (h->type != htype_old && strncmpic(US"In-Reply-To:", h->text, 12) == 0)
+      break;
+
+/* We limit the total length of references.  Although there is no fixed
+limit, some systems do not like headers growing beyond recognition.
+Keep the first message ID for the thread root and the last few for
+the position inside the thread, up to a maximum of 12 altogether. */
+
+if (h || message_id)
+  {
+  fprintf(f, "References:");
+  if (h)
+    {
+    uschar *s, *id, *error;
+    uschar *referenced_ids[12];
+    int reference_count = 0;
+    int i;
+
+    s = Ustrchr(h->text, ':') + 1;
+    parse_allow_group = FALSE;
+    while (*s != 0 && (s = parse_message_id(s, &id, &error)) != NULL)
+      {
+      if (reference_count == nelem(referenced_ids))
+        {
+        memmove(referenced_ids + 1, referenced_ids + 2,
+           sizeof(referenced_ids) - 2*sizeof(uschar *));
+        referenced_ids[reference_count - 1] = id;
+        }
+      else referenced_ids[reference_count++] = id;
+      }
+    for (i = 0; i < reference_count; ++i) fprintf(f, " %s", referenced_ids[i]);
+    }
+
+  /* The message id will have a newline on the end of it. */
+
+  if (message_id) fprintf(f, " %s", message_id);
+  else fprintf(f, "\n");
+  }
 
 /* Add an Auto-Submitted: header */
 
-fprintf(fp, "Auto-Submitted: auto-replied\n");
+fprintf(f, "Auto-Submitted: auto-replied\n");
 
 /* Add any specially requested headers */
 
-if (headers) fprintf(fp, "%s\n", headers);
-fprintf(fp, "\n");
+if (headers) fprintf(f, "%s\n", headers);
+fprintf(f, "\n");
 
 if (text)
   {
-  fprintf(fp, "%s", CS text);
-  if (text[Ustrlen(text)-1] != '\n') fprintf(fp, "\n");
+  fprintf(f, "%s", CS text);
+  if (text[Ustrlen(text)-1] != '\n') fprintf(f, "\n");
   }
 
 if (ff)
   {
   while (Ufgets(big_buffer, big_buffer_size, ff) != NULL)
+    {
     if (file_expand)
       {
-      const uschar * s = expand_string(big_buffer);
-      if (!s) DEBUG(D_transport)
-	debug_printf("error while expanding line from file:\n  %s\n  %s\n",
-	  big_buffer, expand_string_message);
-      fprintf(fp, "%s", s ? CS s : CS big_buffer);
+      uschar *s = expand_string(big_buffer);
+      DEBUG(D_transport)
+        {
+        if (!s)
+          debug_printf("error while expanding line from file:\n  %s\n  %s\n",
+            big_buffer, expand_string_message);
+        }
+      fprintf(f, "%s", s ? CS s : CS big_buffer);
       }
-    else
-      fprintf(fp, "%s", CS big_buffer);
-
+    else fprintf(f, "%s", CS big_buffer);
+    }
   (void) fclose(ff);
   }
 
@@ -630,13 +687,14 @@ limit if we are returning the body. */
 
 if (return_message)
   {
-  const uschar * rubric = tblock->headers_only
-    ? US"------ This is a copy of the message's header lines.\n"
-    : tblock->body_only
-    ? US"------ This is a copy of the body of the message, without the headers.\n"
-    : US"------ This is a copy of the message, including all the headers.\n";
+  uschar *rubric = (tblock->headers_only)?
+    US"------ This is a copy of the message's header lines.\n"
+    : (tblock->body_only)?
+    US"------ This is a copy of the body of the message, without the headers.\n"
+    :
+    US"------ This is a copy of the message, including all the headers.\n";
   transport_ctx tctx = {
-    .u = {.fd = fileno(fp)},
+    .u = {.fd = fileno(f)},
     .tblock = tblock,
     .addr = addr,
     .check_string = NULL,
@@ -656,23 +714,23 @@ if (return_message)
       DELIVER_IN_BUFFER_SIZE;
     if (fstat(deliver_datafile, &statbuf) == 0 && statbuf.st_size > max)
       {
-      fprintf(fp, "\n%s"
+      fprintf(f, "\n%s"
 "------ The body of the message is " OFF_T_FMT " characters long; only the first\n"
 "------ %d or so are included here.\n\n", rubric, statbuf.st_size,
         (max/1000)*1000);
       }
-    else fprintf(fp, "\n%s\n", rubric);
+    else fprintf(f, "\n%s\n", rubric);
     }
-  else fprintf(fp, "\n%s\n", rubric);
+  else fprintf(f, "\n%s\n", rubric);
 
-  fflush(fp);
+  fflush(f);
   transport_count = 0;
   transport_write_message(&tctx, bounce_return_size_limit);
   }
 
 /* End the message and wait for the child process to end; no timeout. */
 
-(void)fclose(fp);
+(void)fclose(f);
 rc = child_close(pid, 0);
 
 /* Update the "sent to" log whatever the yield. This errs on the side of
@@ -689,11 +747,11 @@ between multiple simultaneous deliveries. */
 
 if (cache_fd >= 0)
   {
+  uschar *from = cache_buff;
+  int size = cache_size;
+
   if (lseek(cache_fd, 0, SEEK_SET) == 0)
     {
-    uschar * from = cache_buff;
-    int size = cache_size;
-
     if (!cache_time)
       {
       cache_time = from + size;
@@ -710,7 +768,7 @@ if (cache_fd >= 0)
     memcpy(cache_time, &now, sizeof(time_t));
     if(write(cache_fd, from, size) != size)
       DEBUG(D_transport) debug_printf("Problem writing cache file %s for %s "
-	"transport\n", oncelog, trname);
+	"transport\n", oncelog, tblock->name);
     }
   }
 
@@ -719,18 +777,18 @@ if (cache_fd >= 0)
 else if (dbm_file)
   {
   EXIM_DATUM key_datum, value_datum;
-  exim_datum_init(&key_datum);          /* Some DBM libraries need to have */
-  exim_datum_init(&value_datum);        /* cleared datums. */
-  exim_datum_data_set(&key_datum, US to);   /*XXX rely on dbput not modifying */
-  exim_datum_size_set(&key_datum, Ustrlen(to) + 1);
+  EXIM_DATUM_INIT(key_datum);          /* Some DBM libraries need to have */
+  EXIM_DATUM_INIT(value_datum);        /* cleared datums. */
+  EXIM_DATUM_DATA(key_datum) = CS to;
+  EXIM_DATUM_SIZE(key_datum) = Ustrlen(to) + 1;
 
   /* Many OS define the datum value, sensibly, as a void *. However, there
   are some which still have char *. By casting this address to a char * we
   can avoid warning messages from the char * systems. */
 
-  exim_datum_data_set(&value_datum, &now);
-  exim_datum_size_set(&value_datum, sizeof(time_t));
-  exim_dbput(dbm_file, &key_datum, &value_datum);
+  EXIM_DATUM_DATA(value_datum) = CS (&now);
+  EXIM_DATUM_SIZE(value_datum) = (int)sizeof(time_t);
+  EXIM_DBPUT(dbm_file, key_datum, value_datum);
   }
 
 /* If sending failed, defer to try again - but if once is set the next
@@ -741,13 +799,13 @@ if (rc != 0)
   if (rc == EXIT_NORECIPIENTS)
     {
     DEBUG(D_any) debug_printf("%s transport: message contained no recipients\n",
-      trname);
+      tblock->name);
     }
   else
     {
     addr->transport_return = DEFER;
     addr->message = string_sprintf("Failed to send message from %s "
-      "transport (%d)", trname, rc);
+      "transport (%d)", tblock->name, rc);
     goto END_OFF;
     }
 
@@ -765,69 +823,63 @@ if (logfile)
   int log_fd = Uopen(logfile, O_WRONLY|O_APPEND|O_CREAT, ob->mode);
   if (log_fd >= 0)
     {
-    gstring gs = { .size = LOG_BUFFER_SIZE, .ptr = 0, .s = log_buffer }, *g = &gs;
-
-    /* Use taint-unchecked routines for writing into log_buffer, trusting
-    that we'll never expand it. */
-
+    uschar *ptr = log_buffer;
     DEBUG(D_transport) debug_printf("logging message details\n");
-    g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "%s\n", tod_stamp(tod_log));
+    sprintf(CS ptr, "%s\n", tod_stamp(tod_log));
+    while(*ptr) ptr++;
     if (from)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  From: %s\n", from);
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  From: %s\n", from);
+      while(*ptr) ptr++;
+      }
     if (to)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  To: %s\n", to);
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  To: %s\n", to);
+      while(*ptr) ptr++;
+      }
     if (cc)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  Cc: %s\n", cc);
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Cc: %s\n", cc);
+      while(*ptr) ptr++;
+      }
     if (bcc)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  Bcc: %s\n", bcc);
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Bcc: %s\n", bcc);
+      while(*ptr) ptr++;
+      }
     if (subject)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  Subject: %s\n", subject);
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  Subject: %s\n", subject);
+      while(*ptr) ptr++;
+      }
     if (headers)
-      g = string_fmt_append_f(g, SVFMT_TAINT_NOCHK, "  %s\n", headers);
-    if(write(log_fd, g->s, g->ptr) != g->ptr || close(log_fd))
+      {
+      (void)string_format(ptr, LOG_BUFFER_SIZE - (ptr-log_buffer),
+        "  %s\n", headers);
+      while(*ptr) ptr++;
+      }
+    if(write(log_fd, log_buffer, ptr - log_buffer) != ptr-log_buffer
+      || close(log_fd))
       DEBUG(D_transport) debug_printf("Problem writing log file %s for %s "
-        "transport\n", logfile, trname);
+        "transport\n", logfile, tblock->name);
     }
   else DEBUG(D_transport) debug_printf("Failed to open log file %s for %s "
-    "transport: %s\n", logfile, trname, strerror(errno));
+    "transport: %s\n", logfile, tblock->name, strerror(errno));
   }
 
 END_OFF:
-if (dbm_file) exim_dbclose(dbm_file);
+if (dbm_file) EXIM_DBCLOSE(dbm_file);
 if (cache_fd > 0) (void)close(cache_fd);
 
-DEBUG(D_transport) debug_printf("%s transport succeeded\n", trname);
+DEBUG(D_transport) debug_printf("%s transport succeeded\n", tblock->name);
 
 return FALSE;
 }
 
-
-
-
-# ifdef DYNLOOKUP
-#  define autoreply_transport_info _transport_info
-# endif
 #endif	/*!MACRO_PREDEF*/
-
-transport_info autoreply_transport_info = {
-.drinfo = {
-  .driver_name =	US"autoreply",
-  .options =		autoreply_transport_options,
-  .options_count =	&autoreply_transport_options_count,
-  .options_block =	&autoreply_transport_option_defaults,
-  .options_len =	sizeof(autoreply_transport_options_block),
-  .init =		autoreply_transport_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		TRANSPORT_MAGIC,
-# endif
-  },
-.code =		autoreply_transport_entry,
-.tidyup =	NULL,
-.closedown =	NULL,
-.local =	TRUE
-};
-
-#endif	/*TRANSPORT_AUTOREPLY*/
 /* End of transport/autoreply.c */
-/* vi: aw ai sw=2
-*/
