@@ -2,15 +2,11 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim maintainers 2020 - 2025 */
 /* Copyright (c) University of Cambridge 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 
 #include "../exim.h"
-
-#ifdef TRANSPORT_PIPE	/* Remainder of file */
 #include "pipe.h"
 
 #ifdef HAVE_SETCLASSRESOURCES
@@ -25,50 +21,72 @@ with "*" are not settable by the user but are used by the option-reading
 software for alternative value types. Some options are stored in the transport
 instance block so as to be publicly visible; these are flagged with opt_public.
 */
-#define LOFF(field) OPT_OFF(pipe_transport_options_block, field)
 
 optionlist pipe_transport_options[] = {
-  { "allow_commands",    opt_stringptr,	LOFF(allow_commands) },
+  { "allow_commands",    opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, allow_commands) },
   { "batch_id",          opt_stringptr | opt_public,
-      OPT_OFF(transport_instance, batch_id) },
+      (void *)offsetof(transport_instance, batch_id) },
   { "batch_max",         opt_int | opt_public,
-      OPT_OFF(transport_instance, batch_max) },
-  { "check_string",      opt_stringptr,	LOFF(check_string) },
-  { "command",           opt_stringptr,	LOFF(cmd) },
-  { "environment",       opt_stringptr,	LOFF(environment) },
-  { "escape_string",     opt_stringptr,	LOFF(escape_string) },
-  { "force_command",         opt_bool,	LOFF(force_command) },
-  { "freeze_exec_fail",  opt_bool,	LOFF(freeze_exec_fail) },
-  { "freeze_signal",     opt_bool,	LOFF(freeze_signal) },
-  { "ignore_status",     opt_bool,	LOFF(ignore_status) },
+      (void *)offsetof(transport_instance, batch_max) },
+  { "check_string",      opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, check_string) },
+  { "command",           opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, cmd) },
+  { "environment",       opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, environment) },
+  { "escape_string",     opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, escape_string) },
+  { "force_command",         opt_bool,
+      (void *)offsetof(pipe_transport_options_block, force_command) },
+  { "freeze_exec_fail",  opt_bool,
+      (void *)offsetof(pipe_transport_options_block, freeze_exec_fail) },
+  { "freeze_signal",     opt_bool,
+      (void *)offsetof(pipe_transport_options_block, freeze_signal) },
+  { "ignore_status",     opt_bool,
+      (void *)offsetof(pipe_transport_options_block, ignore_status) },
   { "log_defer_output",  opt_bool | opt_public,
-      OPT_OFF(transport_instance, log_defer_output) },
+      (void *)offsetof(transport_instance, log_defer_output) },
   { "log_fail_output",   opt_bool | opt_public,
-      OPT_OFF(transport_instance, log_fail_output) },
+      (void *)offsetof(transport_instance, log_fail_output) },
   { "log_output",        opt_bool | opt_public,
-      OPT_OFF(transport_instance, log_output) },
-  { "max_output",        opt_mkint,	LOFF(max_output) },
-  { "message_prefix",    opt_stringptr,	LOFF(message_prefix) },
-  { "message_suffix",    opt_stringptr,	LOFF(message_suffix) },
-  { "path",              opt_stringptr,	LOFF(path) },
-  { "permit_coredump",   opt_bool,	LOFF(permit_coredump) },
+      (void *)offsetof(transport_instance, log_output) },
+  { "max_output",        opt_mkint,
+      (void *)offsetof(pipe_transport_options_block, max_output) },
+  { "message_prefix",    opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, message_prefix) },
+  { "message_suffix",    opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, message_suffix) },
+  { "path",              opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, path) },
+  { "permit_coredump",   opt_bool,
+      (void *)offsetof(pipe_transport_options_block, permit_coredump) },
   { "pipe_as_creator",   opt_bool | opt_public,
-      OPT_OFF(transport_instance, deliver_as_creator) },
-  { "restrict_to_path",  opt_bool,	LOFF(restrict_to_path) },
+      (void *)offsetof(transport_instance, deliver_as_creator) },
+  { "restrict_to_path",  opt_bool,
+      (void *)offsetof(pipe_transport_options_block, restrict_to_path) },
   { "return_fail_output",opt_bool | opt_public,
-      OPT_OFF(transport_instance, return_fail_output) },
+      (void *)offsetof(transport_instance, return_fail_output) },
   { "return_output",     opt_bool | opt_public,
-      OPT_OFF(transport_instance, return_output) },
-  { "temp_errors",       opt_stringptr,	LOFF(temp_errors) },
-  { "timeout",           opt_time,	LOFF(timeout) },
-  { "timeout_defer",     opt_bool,	LOFF(timeout_defer) },
-  { "umask",             opt_octint,	LOFF(umask) },
-  { "use_bsmtp",         opt_bool,	LOFF(use_bsmtp) },
+      (void *)offsetof(transport_instance, return_output) },
+  { "temp_errors",       opt_stringptr,
+      (void *)offsetof(pipe_transport_options_block, temp_errors) },
+  { "timeout",           opt_time,
+      (void *)offsetof(pipe_transport_options_block, timeout) },
+  { "timeout_defer",     opt_bool,
+      (void *)offsetof(pipe_transport_options_block, timeout_defer) },
+  { "umask",             opt_octint,
+      (void *)offsetof(pipe_transport_options_block, umask) },
+  { "use_bsmtp",         opt_bool,
+      (void *)offsetof(pipe_transport_options_block, use_bsmtp) },
   #ifdef HAVE_SETCLASSRESOURCES
-  { "use_classresources", opt_bool,	LOFF(use_classresources) },
+  { "use_classresources", opt_bool,
+      (void *)offsetof(pipe_transport_options_block, use_classresources) },
   #endif
-  { "use_crlf",          opt_bool,	LOFF(use_crlf) },
-  { "use_shell",         opt_bool,	LOFF(use_shell) },
+  { "use_crlf",          opt_bool,
+      (void *)offsetof(pipe_transport_options_block, use_crlf) },
+  { "use_shell",         opt_bool,
+      (void *)offsetof(pipe_transport_options_block, use_shell) },
 };
 
 /* Size of the options list. An extern variable has to be used so that its
@@ -82,7 +100,7 @@ int pipe_transport_options_count =
 
 /* Dummy values */
 pipe_transport_options_block pipe_transport_option_defaults = {0};
-void pipe_transport_init(driver_instance *tblock) {}
+void pipe_transport_init(transport_instance *tblock) {}
 BOOL pipe_transport_entry(transport_instance *tblock, address_item *addr) {return FALSE;}
 
 #else   /*!MACRO_PREDEF*/
@@ -91,13 +109,31 @@ BOOL pipe_transport_entry(transport_instance *tblock, address_item *addr) {retur
 /* Default private options block for the pipe transport. */
 
 pipe_transport_options_block pipe_transport_option_defaults = {
-  .path =	US"/bin:/usr/bin",
-  .temp_errors = US mac_expanded_string(EX_TEMPFAIL) ":"
-		   mac_expanded_string(EX_CANTCREAT),
-  .umask =	022,
-  .max_output = 20480,
-  .timeout =	60*60,
-  /* all others null/zero/false */
+  NULL,           /* cmd */
+  NULL,           /* allow_commands */
+  NULL,           /* environment */
+  US"/bin:/usr/bin",  /* path */
+  NULL,           /* message_prefix (reset in init if not bsmtp) */
+  NULL,           /* message_suffix (ditto) */
+  US mac_expanded_string(EX_TEMPFAIL) ":"    /* temp_errors */
+     mac_expanded_string(EX_CANTCREAT),
+  NULL,           /* check_string */
+  NULL,           /* escape_string */
+  022,            /* umask */
+  20480,          /* max_output */
+  60*60,          /* timeout */
+  0,              /* options */
+  FALSE,          /* force_command */
+  FALSE,          /* freeze_exec_fail */
+  FALSE,          /* freeze_signal */
+  FALSE,          /* ignore_status */
+  FALSE,          /* permit_coredump */
+  FALSE,          /* restrict_to_path */
+  FALSE,          /* timeout_defer */
+  FALSE,          /* use_shell */
+  FALSE,          /* use_bsmtp */
+  FALSE,          /* use_classresources */
+  FALSE           /* use_crlf */
 };
 
 
@@ -123,10 +159,18 @@ Returns:     OK, FAIL, or DEFER
 */
 
 static int
-pipe_transport_setup(transport_instance * tblock, address_item * addrlist,
-  transport_feedback * dummy, uid_t uid, gid_t gid, uschar ** errmsg)
+pipe_transport_setup(transport_instance *tblock, address_item *addrlist,
+  transport_feedback *dummy, uid_t uid, gid_t gid, uschar **errmsg)
 {
-const pipe_transport_options_block * ob = tblock->drinst.options_block;
+pipe_transport_options_block *ob =
+  (pipe_transport_options_block *)(tblock->options_block);
+
+addrlist = addrlist;  /* Keep compiler happy */
+dummy = dummy;
+uid = uid;
+gid = gid;
+errmsg = errmsg;
+ob = ob;
 
 #ifdef HAVE_SETCLASSRESOURCES
 if (ob->use_classresources)
@@ -176,11 +220,10 @@ enable consistency checks to be done, or anything else that needs
 to be set up. */
 
 void
-pipe_transport_init(driver_instance * t)
+pipe_transport_init(transport_instance *tblock)
 {
-transport_instance * tblock = (transport_instance *)t;
-const uschar * trname = t->name;
-pipe_transport_options_block * ob = t->options_block;
+pipe_transport_options_block *ob =
+  (pipe_transport_options_block *)(tblock->options_block);
 
 /* Set up the setup entry point, to be called in the privileged state */
 
@@ -190,15 +233,15 @@ tblock->setup = pipe_transport_setup;
 
 if (tblock->deliver_as_creator && (tblock->uid_set || tblock->gid_set ||
   tblock->expand_uid != NULL || tblock->expand_gid != NULL))
-    log_write_die(0, LOG_CONFIG,
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
       "both pipe_as_creator and an explicit uid/gid are set for the %s "
-        "transport", trname);
+        "transport", tblock->name);
 
 /* If a fixed uid field is set, then a gid field must also be set. */
 
 if (tblock->uid_set && !tblock->gid_set && tblock->expand_gid == NULL)
-  log_write_die(0, LOG_CONFIG,
-    "user set without group for the %s transport", trname);
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
+    "user set without group for the %s transport", tblock->name);
 
 /* Temp_errors must consist only of digits and colons, but there can be
 spaces round the colons, so allow them too. */
@@ -207,23 +250,23 @@ if (ob->temp_errors != NULL && Ustrcmp(ob->temp_errors, "*") != 0)
   {
   size_t p = Ustrspn(ob->temp_errors, "0123456789: ");
   if (ob->temp_errors[p] != 0)
-    log_write_die(0, LOG_CONFIG,
+    log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
       "temp_errors must be a list of numbers or an asterisk for the %s "
-      "transport", trname);
+      "transport", tblock->name);
   }
 
 /* Only one of return_output/return_fail_output or log_output/log_fail_output
 should be set. */
 
 if (tblock->return_output && tblock->return_fail_output)
-  log_write_die(0, LOG_CONFIG,
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "both return_output and return_fail_output set for %s transport",
-    trname);
+    tblock->name);
 
 if (tblock->log_output && tblock->log_fail_output)
-  log_write_die(0, LOG_CONFIG,
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "both log_output and log_fail_output set for the %s transport",
-    trname);
+    tblock->name);
 
 /* If batch SMTP is set, force the check and escape strings, and arrange that
 headers are also escaped. */
@@ -248,27 +291,27 @@ else
 /* The restrict_to_path  and use_shell options are incompatible */
 
 if (ob->restrict_to_path && ob->use_shell)
-  log_write_die(0, LOG_CONFIG,
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "both restrict_to_path and use_shell set for %s transport",
-    trname);
+    tblock->name);
 
 /* The allow_commands and use_shell options are incompatible */
 
 if (ob->allow_commands && ob->use_shell)
-  log_write_die(0, LOG_CONFIG,
+  log_write(0, LOG_PANIC_DIE|LOG_CONFIG,
     "both allow_commands and use_shell set for %s transport",
-    trname);
+    tblock->name);
 
 /* Set up the bitwise options for transport_write_message from the various
 driver options. Only one of body_only and headers_only can be set. */
 
 ob->options |=
-    (tblock->body_only ? topt_no_headers : 0)
-  | (tblock->headers_only ? topt_no_body : 0)
-  | (tblock->return_path_add ? topt_add_return_path : 0)
-  | (tblock->delivery_date_add ? topt_add_delivery_date : 0)
-  | (tblock->envelope_to_add ? topt_add_envelope_to : 0)
-  | (ob->use_crlf ? topt_use_crlf : 0);
+  (tblock->body_only? topt_no_headers : 0) |
+  (tblock->headers_only? topt_no_body : 0) |
+  (tblock->return_path_add? topt_add_return_path : 0) |
+  (tblock->delivery_date_add? topt_add_delivery_date : 0) |
+  (tblock->envelope_to_add? topt_add_envelope_to : 0) |
+  (ob->use_crlf? topt_use_crlf : 0);
 }
 
 
@@ -294,9 +337,9 @@ Returns:             TRUE if all went well; otherwise an error will be
 */
 
 static BOOL
-set_up_direct_command(const uschar *** argvptr, const uschar * cmd,
-  BOOL expand_arguments, int expand_fail, address_item * addr,
-  const uschar * tname, pipe_transport_options_block * ob)
+set_up_direct_command(const uschar ***argvptr, uschar *cmd,
+  BOOL expand_arguments, int expand_fail, address_item *addr, uschar *tname,
+  pipe_transport_options_block *ob)
 {
 BOOL permitted = FALSE;
 const uschar **argv;
@@ -306,9 +349,8 @@ call the common function for creating an argument list and expanding
 the items if necessary. If it fails, this function fails (error information
 is in the addresses). */
 
-if (!transport_set_up_command(argvptr, cmd,
-      expand_arguments ? TSUC_EXPAND_ARGS : 0,
-      expand_fail, addr, string_sprintf("%.50s transport", tname), NULL))
+if (!transport_set_up_command(argvptr, cmd, expand_arguments, expand_fail,
+      addr, string_sprintf("%.50s transport", tname), NULL))
   return FALSE;
 
 /* Point to the set-up arguments. */
@@ -317,16 +359,16 @@ argv = *argvptr;
 
 /* If allow_commands is set, see if the command is in the permitted list. */
 
-GET_OPTION("allow_commands");
 if (ob->allow_commands)
   {
   int sep = 0;
-  const uschar * s, *p;
+  const uschar *s;
+  uschar *p;
 
   if (!(s = expand_string(ob->allow_commands)))
     {
     addr->transport_return = DEFER;
-    addr->message = string_sprintf("failed to expand string %q "
+    addr->message = string_sprintf("failed to expand string \"%s\" "
       "for %s transport: %s", ob->allow_commands, tname, expand_string_message);
     return FALSE;
     }
@@ -349,7 +391,7 @@ if (!permitted)
     if (Ustrchr(argv[0], '/') != NULL)
       {
       addr->transport_return = FAIL;
-      addr->message = string_sprintf("\"/\" found in %q (command for %s "
+      addr->message = string_sprintf("\"/\" found in \"%s\" (command for %s "
         "transport) - failed for security reasons", cmd, tname);
       return FALSE;
       }
@@ -358,7 +400,7 @@ if (!permitted)
   else if (ob->allow_commands)
     {
     addr->transport_return = FAIL;
-    addr->message = string_sprintf("%q command not permitted by %s "
+    addr->message = string_sprintf("\"%s\" command not permitted by %s "
       "transport", argv[0], tname);
     return FALSE;
     }
@@ -370,11 +412,10 @@ for it. */
 if (argv[0][0] != '/')
   {
   int sep = 0;
-  const uschar * p;
+  uschar *p;
+  const uschar *listptr = expand_string(ob->path);
 
-  GET_OPTION("path");
-  for (const uschar * listptr = expand_string(ob->path);
-      p = string_nextinlist(&listptr, &sep, NULL, 0); )
+  while ((p = string_nextinlist(&listptr, &sep, NULL, 0)))
     {
     struct stat statbuf;
     sprintf(CS big_buffer, "%.256s/%.256s", p, argv[0]);
@@ -387,7 +428,7 @@ if (argv[0][0] != '/')
   if (!p)
     {
     addr->transport_return = FAIL;
-    addr->message = string_sprintf("%q command not found for %s transport",
+    addr->message = string_sprintf("\"%s\" command not found for %s transport",
       argv[0], tname);
     return FALSE;
     }
@@ -417,13 +458,12 @@ Returns:             TRUE if all went well; otherwise an error will be
 */
 
 static BOOL
-set_up_shell_command(const uschar *** argvptr, const uschar * cmd,
-  BOOL expand_arguments, int expand_fail, address_item * addr,
-  const uschar * tname)
+set_up_shell_command(const uschar ***argvptr, uschar *cmd,
+  BOOL expand_arguments, int expand_fail, address_item *addr, uschar *tname)
 {
 const uschar **argv;
 
-*argvptr = argv = store_get((4)*sizeof(uschar *), GET_UNTAINTED);
+*argvptr = argv = store_get((4)*sizeof(uschar *));
 
 argv[0] = US"/bin/sh";
 argv[1] = US"-c";
@@ -441,13 +481,14 @@ if (expand_arguments)
 
   /* Allow $recipients in the expansion iff it comes from a system filter */
 
-  f.enable_dollar_recipients = addr && addr->parent &&
+  enable_dollar_recipients = addr && addr->parent &&
     Ustrcmp(addr->parent->address, "system-filter") == 0;
 
   if (p != NULL && (
          (p > cmd && p[-1] == '$') ||
          (p > cmd + 1 && p[-2] == '$' && p[-1] == '{' && p[14] == '}')))
     {
+    address_item *ad;
     uschar *q = p + 14;
 
     if (p[-1] == '{') { q++; p--; }
@@ -455,11 +496,8 @@ if (expand_arguments)
     g = string_get(Ustrlen(cmd) + 64);
     g = string_catn(g, cmd, p - cmd - 1);
 
-    for (address_item * ad = addr; ad; ad = ad->next)
+    for (ad = addr; ad; ad = ad->next)
       {
-      DEBUG(D_transport) if (is_tainted(ad->address))
-	debug_printf("tainted element '%s' from $pipe_addresses\n", ad->address);
-
       /*XXX string_append_listele() ? */
       if (ad != addr) g = string_catn(g, US" ", 1);
       g = string_cat(g, ad->address);
@@ -471,12 +509,12 @@ if (expand_arguments)
   else
     argv[2] = expand_string(cmd);
 
-  f.enable_dollar_recipients = FALSE;
+  enable_dollar_recipients = FALSE;
 
   if (!argv[2])
     {
-    addr->transport_return = f.search_find_defer ? DEFER : expand_fail;
-    addr->message = string_sprintf("Expansion of command %q "
+    addr->transport_return = search_find_defer ? DEFER : expand_fail;
+    addr->message = string_sprintf("Expansion of command \"%s\" "
       "in %s transport failed: %s",
       cmd, tname, expand_string_message);
     return FALSE;
@@ -512,16 +550,21 @@ pipe_transport_entry(
   transport_instance *tblock,      /* data for this instantiation */
   address_item *addr)              /* address(es) we are working on */
 {
-pipe_transport_options_block * ob = tblock->drinst.options_block;
-const uschar * trname = tblock->drinst.name;
 pid_t pid, outpid;
 int fd_in, fd_out, rc;
-int envcount = 0, envsep = 0, expand_fail, timeout = ob->timeout;
-BOOL written_ok = FALSE, expand_arguments;
-const uschar ** argv;
-uschar * envp[50];
-const uschar * envlist = ob->environment, * cmd, * ss;
-uschar * eol = ob->use_crlf ? US"\r\n" : US"\n";
+int envcount = 0;
+int envsep = 0;
+int expand_fail;
+pipe_transport_options_block *ob =
+  (pipe_transport_options_block *)(tblock->options_block);
+int timeout = ob->timeout;
+BOOL written_ok = FALSE;
+BOOL expand_arguments;
+const uschar **argv;
+uschar *envp[50];
+const uschar *envlist = ob->environment;
+uschar *cmd, *ss;
+uschar *eol = ob->use_crlf ? US"\r\n" : US"\n";
 transport_ctx tctx = {
   .tblock = tblock,
   .addr = addr,
@@ -530,7 +573,7 @@ transport_ctx tctx = {
   ob->options | topt_not_socket /* set at initialization time */
 };
 
-DEBUG(D_transport) debug_printf("%s transport entered\n", trname);
+DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 
 /* Set up for the good case */
 
@@ -544,11 +587,11 @@ symbol. In other cases, the command is supplied as one of the pipe transport's
 options. */
 
 if (testflag(addr, af_pfr) && addr->local_part[0] == '|')
+  {
   if (ob->force_command)
     {
     /* Enables expansion of $address_pipe into separate arguments */
     setflag(addr, af_force_command);
-    GET_OPTION("commsnd");
     cmd = ob->cmd;
     expand_arguments = TRUE;
     expand_fail = PANIC;
@@ -556,35 +599,28 @@ if (testflag(addr, af_pfr) && addr->local_part[0] == '|')
   else
     {
     cmd = addr->local_part + 1;
-    Uskip_whitespace(&cmd);
+    while (isspace(*cmd)) cmd++;
     expand_arguments = testflag(addr, af_expand_pipe);
     expand_fail = FAIL;
     }
+  }
 else
   {
-  GET_OPTION("commsnd");
   cmd = ob->cmd;
   expand_arguments = TRUE;
   expand_fail = PANIC;
   }
 
 /* If no command has been supplied, we are in trouble.
-We also check for an empty string since it may be
-coming from addr->local_part[0] == '|' */
+ * We also check for an empty string since it may be
+ * coming from addr->local_part[0] == '|'
+ */
 
-if (!cmd || !*cmd)
+if (cmd == NULL || *cmd == '\0')
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf("no command specified for %s transport",
-    trname);
-  return FALSE;
-  }
-if (is_tainted(cmd))
-  {
-  DEBUG(D_transport) debug_printf("cmd '%s' is tainted\n", cmd);
-  addr->message = string_sprintf("Tainted '%s' (command "
-    "for %s transport) not permitted", cmd, trname);
-  addr->transport_return = PANIC;
+    tblock->name);
   return FALSE;
   }
 
@@ -594,10 +630,10 @@ addr->pipe_expandn for use here. */
 
 if (expand_arguments && addr->pipe_expandn)
   {
-  uschar ** ss = addr->pipe_expandn;
+  uschar **ss = addr->pipe_expandn;
   expand_nmax = -1;
-  if (*ss) filter_thisaddress = *ss++;
-  while (*ss)
+  if (*ss != NULL) filter_thisaddress = *ss++;
+  while (*ss != NULL)
     {
     expand_nstring[++expand_nmax] = *ss;
     expand_nlength[expand_nmax] = Ustrlen(*ss++);
@@ -612,10 +648,10 @@ there is an option to do that. */
 if (ob->use_shell)
   {
   if (!set_up_shell_command(&argv, cmd, expand_arguments, expand_fail, addr,
-    trname)) return FALSE;
+    tblock->name)) return FALSE;
   }
 else if (!set_up_direct_command(&argv, cmd, expand_arguments, expand_fail, addr,
-  trname, ob)) return FALSE;
+  tblock->name, ob)) return FALSE;
 
 expand_nmax = -1;           /* Reset */
 filter_thisaddress = NULL;
@@ -640,35 +676,35 @@ envp[envcount++] = string_sprintf("QUALIFY_DOMAIN=%s", qualify_domain_sender);
 envp[envcount++] = string_sprintf("SENDER=%s", sender_address);
 envp[envcount++] = US"SHELL=/bin/sh";
 
-if (addr->host_list)
+if (addr->host_list != NULL)
   envp[envcount++] = string_sprintf("HOST=%s", addr->host_list->name);
 
-if (f.timestamps_utc)
-  envp[envcount++] = US"TZ=UTC";
-else if (timezone_string && timezone_string[0])
+if (timestamps_utc) envp[envcount++] = US"TZ=UTC";
+else if (timezone_string != NULL && timezone_string[0] != 0)
   envp[envcount++] = string_sprintf("TZ=%s", timezone_string);
 
 /* Add any requested items */
 
-GET_OPTION("environment");
 if (envlist)
-  if (!(envlist = expand_string(envlist)))
+  {
+  envlist = expand_cstring(envlist);
+  if (envlist == NULL)
     {
     addr->transport_return = DEFER;
-    addr->message = string_sprintf("failed to expand string %q "
-      "for %s transport: %s", ob->environment, trname,
+    addr->message = string_sprintf("failed to expand string \"%s\" "
+      "for %s transport: %s", ob->environment, tblock->name,
       expand_string_message);
     return FALSE;
     }
+  }
 
-while ((ss = string_nextinlist(&envlist, &envsep, NULL, 0)))
+while ((ss = string_nextinlist(&envlist, &envsep, big_buffer, big_buffer_size)))
    {
    if (envcount > nelem(envp) - 2)
      {
      addr->transport_return = DEFER;
-     addr->basic_errno = E2BIG;
      addr->message = string_sprintf("too many environment settings for "
-       "%s transport", trname);
+       "%s transport", tblock->name);
      return FALSE;
      }
    envp[envcount++] = string_copy(ss);
@@ -678,11 +714,11 @@ envp[envcount] = NULL;
 
 /* If the -N option is set, can't do any more. */
 
-if (f.dont_deliver)
+if (dont_deliver)
   {
   DEBUG(D_transport)
     debug_printf("*** delivery by %s transport bypassed by -N option",
-      trname);
+      tblock->name);
   return FALSE;
   }
 
@@ -708,12 +744,11 @@ reading of the output pipe. */
 uid/gid and current directory. Request that the new process be a process group
 leader, so we can kill it and all its children on a timeout. */
 
-if ((pid = child_open(USS argv, envp, ob->umask, &fd_in, &fd_out, TRUE,
-			US"pipe-tpt-cmd")) < 0)
+if ((pid = child_open(USS argv, envp, ob->umask, &fd_in, &fd_out, TRUE)) < 0)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
-    "Failed to create child process for %s transport: %s", trname,
+    "Failed to create child process for %s transport: %s", tblock->name,
       strerror(errno));
   return FALSE;
   }
@@ -721,13 +756,13 @@ tctx.u.fd = fd_in;
 
 /* Now fork a process to handle the output that comes down the pipe. */
 
-if ((outpid = exim_fork(US"pipe-tpt-output")) < 0)
+if ((outpid = fork()) < 0)
   {
   addr->basic_errno = errno;
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
     "Failed to create process for handling output in %s transport",
-      trname);
+      tblock->name);
   (void)close(fd_in);
   (void)close(fd_out);
   return FALSE;
@@ -779,7 +814,7 @@ bit here to let the sub-process get going, but it may still not complete. So we
 ignore all writing errors. (When in the test harness, we do do a short sleep so
 any debugging output is likely to be in the same order.) */
 
-testharness_pause_ms(500);
+if (running_in_test_harness) millisleep(500);
 
 DEBUG(D_transport) debug_printf("Writing message to pipe\n");
 
@@ -797,15 +832,14 @@ transport_count = 0;
 
 /* First write any configured prefix information */
 
-GET_OPTION("message_prefix");
-if (ob->message_prefix)
+if (ob->message_prefix != NULL)
   {
-  uschar * prefix = expand_string(ob->message_prefix);
-  if (!prefix)
+  uschar *prefix = expand_string(ob->message_prefix);
+  if (prefix == NULL)
     {
-    addr->transport_return = f.search_find_defer? DEFER : PANIC;
-    addr->message = string_sprintf("Expansion of %q (prefix for %s "
-      "transport) failed: %s", ob->message_prefix, trname,
+    addr->transport_return = search_find_defer? DEFER : PANIC;
+    addr->message = string_sprintf("Expansion of \"%s\" (prefix for %s "
+      "transport) failed: %s", ob->message_prefix, tblock->name,
       expand_string_message);
     return FALSE;
     }
@@ -820,10 +854,12 @@ than one address available here, all must be included. Force SMTP dot-handling.
 
 if (ob->use_bsmtp)
   {
+  address_item *a;
+
   if (!transport_write_string(fd_in, "MAIL FROM:<%s>%s", return_path, eol))
     goto END_WRITE;
 
-  for (address_item * a = addr; a; a = a->next)
+  for (a = addr; a; a = a->next)
     if (!transport_write_string(fd_in,
         "RCPT TO:<%s>%s",
         transport_rcpt_address(a, tblock->rcpt_include_affixes),
@@ -840,15 +876,14 @@ if (!transport_write_message(&tctx, 0))
 
 /* Now any configured suffix */
 
-GET_OPTION("message_suffix");
 if (ob->message_suffix)
   {
-  uschar * suffix = expand_string(ob->message_suffix);
+  uschar *suffix = expand_string(ob->message_suffix);
   if (!suffix)
     {
-    addr->transport_return = f.search_find_defer? DEFER : PANIC;
-    addr->message = string_sprintf("Expansion of %q (suffix for %s "
-      "transport) failed: %s", ob->message_suffix, trname,
+    addr->transport_return = search_find_defer? DEFER : PANIC;
+    addr->message = string_sprintf("Expansion of \"%s\" (suffix for %s "
+      "transport) failed: %s", ob->message_suffix, tblock->name,
       expand_string_message);
     return FALSE;
     }
@@ -885,7 +920,7 @@ if (!written_ok)
   if (errno == ETIMEDOUT)
     {
     addr->message = string_sprintf("%stimeout while writing to pipe",
-      f.transport_filter_timed_out ? "transport filter " : "");
+      transport_filter_timed_out? "transport filter " : "");
     addr->transport_return = ob->timeout_defer? DEFER : FAIL;
     timeout = 1;
     }
@@ -906,7 +941,7 @@ if (!written_ok)
       addr->more_errno,
       (addr->more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
     else if (errno == ERRNO_WRITEINCOMPLETE)
-      addr->message = US"Failed repeatedly to write data";
+      addr->message = string_sprintf("Failed repeatedly to write data");
     else
       addr->message = string_sprintf("Error %d", errno);
     return FALSE;
@@ -919,8 +954,8 @@ above timed out. */
 
 if ((rc = child_close(pid, timeout)) != 0)
   {
-  uschar * tmsg = addr->message
-    ? string_sprintf(" (preceded by %s)", addr->message) : US"";
+  uschar *tmsg = (addr->message == NULL)? US"" :
+    string_sprintf(" (preceded by %s)", addr->message);
 
   /* The process did not complete in time; kill its process group and fail
   the delivery. It appears to be necessary to kill the output process too, as
@@ -942,7 +977,7 @@ if ((rc = child_close(pid, timeout)) != 0)
     {
     addr->transport_return = PANIC;
     addr->message = string_sprintf("Wait() failed for child process of %s "
-      "transport: %s%s", trname, strerror(errno), tmsg);
+      "transport: %s%s", tblock->name, strerror(errno), tmsg);
     }
 
   /* Since the transport_filter timed out we assume it has sent the child process
@@ -951,7 +986,7 @@ if ((rc = child_close(pid, timeout)) != 0)
   This prevents the transport_filter timeout message from getting overwritten
   by the exit error which is not the cause of the problem. */
 
-  else if (f.transport_filter_timed_out)
+  else if (transport_filter_timed_out)
     {
     killpg(pid, SIGKILL);
     kill(outpid, SIGKILL);
@@ -970,14 +1005,14 @@ if ((rc = child_close(pid, timeout)) != 0)
       addr->transport_return = DEFER;
       addr->special_action = SPECIAL_FREEZE;
       addr->message = string_sprintf("Child process of %s transport (running "
-        "command %q) was terminated by signal %d (%s)%s", trname, cmd,
+        "command \"%s\") was terminated by signal %d (%s)%s", tblock->name, cmd,
         -rc, os_strsignal(-rc), tmsg);
       }
     else if (!ob->ignore_status)
       {
       addr->transport_return = FAIL;
       addr->message = string_sprintf("Child process of %s transport (running "
-        "command %q) was terminated by signal %d (%s)%s", trname, cmd,
+        "command \"%s\") was terminated by signal %d (%s)%s", tblock->name, cmd,
         -rc, os_strsignal(-rc), tmsg);
       }
     }
@@ -1026,11 +1061,11 @@ if ((rc = child_close(pid, timeout)) != 0)
     {
     /* Always handle execve() failure specially if requested to */
 
-    if (ob->freeze_exec_fail  &&  rc == EX_EXECFAILED)
+    if (ob->freeze_exec_fail && (rc == EX_EXECFAILED))
       {
       addr->transport_return = DEFER;
       addr->special_action = SPECIAL_FREEZE;
-      addr->message = string_sprintf("pipe process failed to exec %q%s",
+      addr->message = string_sprintf("pipe process failed to exec \"%s\"%s",
         cmd, tmsg);
       }
 
@@ -1038,8 +1073,9 @@ if ((rc = child_close(pid, timeout)) != 0)
 
     else if (!ob->ignore_status)
       {
-      const uschar * ss;
+      uschar *ss;
       gstring * g;
+      int i;
 
       /* If temp_errors is "*" all codes are temporary. Initialization checks
       that it's either "*" or a list of numbers. If not "*", scan the list of
@@ -1063,18 +1099,18 @@ if ((rc = child_close(pid, timeout)) != 0)
       doesn't have to be brilliantly efficient - it is an error situation. */
 
       addr->message = string_sprintf("Child process of %s transport returned "
-        "%d", trname, rc);
+        "%d", tblock->name, rc);
       g = string_cat(NULL, addr->message);
 
       /* If the return code is > 128, it often means that a shell command
       was terminated by a signal. */
 
-      ss = rc > 128
-        ? string_sprintf("(could mean shell command ended by signal %d (%s))",
-			  rc-128, os_strsignal(rc-128))
-        : US os_strexit(rc);
+      ss = (rc > 128)?
+        string_sprintf("(could mean shell command ended by signal %d (%s))",
+          rc-128, os_strsignal(rc-128)) :
+        US os_strexit(rc);
 
-      if (*ss)
+      if (*ss != 0)
         {
         g = string_catn(g, US" ", 1);
         g = string_cat (g, ss);
@@ -1084,7 +1120,7 @@ if ((rc = child_close(pid, timeout)) != 0)
 
       g = string_catn(g, US" from command:", 14);
 
-      for (int i = 0; i < sizeof(argv)/sizeof(int *) && argv[i] != NULL; i++)
+      for (i = 0; i < sizeof(argv)/sizeof(int *) && argv[i] != NULL; i++)
         {
         BOOL quote = FALSE;
         g = string_catn(g, US" ", 1);
@@ -1113,7 +1149,7 @@ are complete before we pass this point. */
 
 while (wait(&rc) >= 0);
 
-DEBUG(D_transport) debug_printf("%s transport yielded %d\n", trname,
+DEBUG(D_transport) debug_printf("%s transport yielded %d\n", tblock->name,
   addr->transport_return);
 
 /* If there has been a problem, the message in addr->message contains details
@@ -1126,31 +1162,5 @@ if (addr->transport_return != OK)
 return FALSE;
 }
 
-
-
-
-# ifdef DYNLOOKUP
-#  define pipe_transport_info _transport_info
-# endif
 #endif	/*!MACRO_PREDEF*/
-
-transport_info pipe_transport_info = {
-.drinfo = {
-  .driver_name =	US"pipe",
-  .options =		pipe_transport_options,
-  .options_count =	&pipe_transport_options_count,
-  .options_block =	&pipe_transport_option_defaults,
-  .options_len =	sizeof(pipe_transport_options_block),
-  .init =		pipe_transport_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		TRANSPORT_MAGIC,
-# endif
-  },
-.code =		pipe_transport_entry,
-.tidyup =	NULL,
-.closedown =	NULL,
-.local =	TRUE
-};
-
-#endif	/*TRASPORT_PIPE*/
 /* End of transport/pipe.c */
