@@ -2,23 +2,15 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 1995 - 2025 */
 /* Copyright (c) Andrew Colin Kissa <andrew@topdog.za.net> 2016 */
 /* Copyright (c) University of Cambridge 2016 */
+/* Copyright (c) The Exim Maintainers 1995 - 2018 */
 /* See the file NOTICE for conditions of use and distribution. */
-/* SPDX-License-Identifier: GPL-2.0-or-later */
-
 
 
 #include "../exim.h"
-
-#ifdef EXPERIMENTAL_QUEUEFILE	/* whole file */
 #include "queuefile.h"
 
-#ifndef EXIM_HAVE_OPENAT
-# error queuefile transport reqires openat() support
-#endif
-
 /* Options specific to the appendfile transport. They must be in alphabetic
 order (note that "_" comes before the lower case letters). Some of them are
 stored in the publicly visible instance block - these are flagged with the
@@ -26,7 +18,7 @@ opt_public flag. */
 
 optionlist queuefile_transport_options[] = {
   { "directory", opt_stringptr,
-    OPT_OFF(queuefile_transport_options_block, dirname) },
+    (void *)offsetof(queuefile_transport_options_block, dirname) },
 };
 
 
@@ -41,7 +33,7 @@ int queuefile_transport_options_count =
 
 /* Dummy values */
 queuefile_transport_options_block queuefile_transport_option_defaults = {0};
-void queuefile_transport_init(driver_instance *tblock) {}
+void queuefile_transport_init(transport_instance *tblock) {}
 BOOL queuefile_transport_entry(transport_instance *tblock, address_item *addr) {return FALSE;}
 
 #else   /*!MACRO_PREDEF*/
@@ -58,13 +50,14 @@ queuefile_transport_options_block queuefile_transport_option_defaults = {
 *          Initialization entry point            *
 *************************************************/
 
-void queuefile_transport_init(driver_instance * t)
+void queuefile_transport_init(transport_instance *tblock)
 {
-queuefile_transport_options_block * ob = t->options_block;
+queuefile_transport_options_block *ob =
+  (queuefile_transport_options_block *) tblock->options_block;
 
 if (!ob->dirname)
-  log_write_die(0, LOG_CONFIG,
-    "directory must be set for the %s transport", t->name);
+  log_write(0, LOG_PANIC_DIE | LOG_CONFIG,
+    "directory must be set for the %s transport", tblock->name);
 }
 
 /* This function will copy from a file to another
@@ -81,13 +74,14 @@ copy_spool_file(int dst, int src)
 {
 int i, j;
 uschar buffer[16384];
+uschar * s;
 
 if (lseek(src, 0, SEEK_SET) != 0)
   return FALSE;
 
 do
   if ((j = read(src, buffer, sizeof(buffer))) > 0)
-    for (uschar * s = buffer; (i = write(dst, s, j)) != j; s += i, j -= i)
+    for (s = buffer; (i = write(dst, s, j)) != j; s += i, j -= i)
       if (i < 0)
 	return FALSE;
   else if (j < 0)
@@ -115,7 +109,6 @@ static BOOL
 copy_spool_files(transport_instance * tb, address_item * addr,
   const uschar * dstpath, int sdfd, int ddfd, BOOL link_file, int srcfd)
 {
-const uschar * trname = tb->drinst.name;
 BOOL is_hdr_file = srcfd < 0;
 const uschar * suffix = srcfd < 0 ? US"H" : US"D";
 int dstfd;
@@ -128,7 +121,7 @@ dstpath = string_sprintf("%s/%s-%s", dstpath, message_id, suffix);
 if (link_file)
   {
   DEBUG(D_transport) debug_printf("%s transport, linking %s => %s\n",
-    trname, srcpath, dstpath);
+    tb->name, srcpath, dstpath);
 
   if (linkat(sdfd, CCS filename, ddfd, CCS filename, 0) >= 0)
     return TRUE;
@@ -139,14 +132,14 @@ if (link_file)
 else					/* use data copy */
   {
   DEBUG(D_transport) debug_printf("%s transport, copying %s => %s\n",
-    trname, srcpath, dstpath);
+    tb->name, srcpath, dstpath);
 
   if (  (s = dstpath,
-	 (dstfd = exim_openat4(ddfd, CCS filename, O_RDWR|O_CREAT|O_EXCL, SPOOL_MODE))
+	 (dstfd = openat(ddfd, CCS filename, O_RDWR|O_CREAT|O_EXCL, SPOOL_MODE))
 	 < 0
 	)
      ||    is_hdr_file
-	&& (s = srcpath, (srcfd = exim_openat(sdfd, CCS filename, O_RDONLY)) < 0)
+	&& (s = srcpath, (srcfd = openat(sdfd, CCS filename, O_RDONLY)) < 0)
      )
     op = US"opening";
 
@@ -162,7 +155,7 @@ else					/* use data copy */
 
 addr->basic_errno = errno;
 addr->message = string_sprintf("%s transport %s file: %s failed with error: %s",
-  trname, op, s, strerror(errno));
+  tb->name, op, s, strerror(errno));
 addr->transport_return = DEFER;
 return FALSE;
 }
@@ -177,8 +170,8 @@ the first address is the status for all addresses in a batch. */
 BOOL
 queuefile_transport_entry(transport_instance * tblock, address_item * addr)
 {
-queuefile_transport_options_block * ob = tblock->drinst.options_block;
-const uschar * trname = tblock->drinst.name;
+queuefile_transport_options_block * ob =
+  (queuefile_transport_options_block *) tblock->options_block;
 BOOL can_link;
 uschar * sourcedir = spool_dname(US"input", message_subdir);
 uschar * s, * dstdir;
@@ -186,7 +179,7 @@ struct stat dstatbuf, sstatbuf;
 int ddfd = -1, sdfd = -1;
 
 DEBUG(D_transport)
-  debug_printf("%s transport entered\n", trname);
+  debug_printf("%s transport entered\n", tblock->name);
 
 #ifndef O_DIRECTORY
 # define O_DIRECTORY 0
@@ -195,11 +188,10 @@ DEBUG(D_transport)
 # define O_NOFOLLOW 0
 #endif
 
-GET_OPTION("directory");
 if (!(dstdir = expand_string(ob->dirname)))
   {
   addr->message = string_sprintf("%s transport: failed to expand dirname option",
-    trname);
+    tblock->name);
   addr->transport_return = DEFER;
   return FALSE;
   }
@@ -207,7 +199,7 @@ if (*dstdir != '/')
   {
   addr->transport_return = PANIC;
   addr->message = string_sprintf("%s transport directory: "
-    "%s is not absolute", trname, dstdir);
+    "%s is not absolute", tblock->name, dstdir);
   return FALSE;
   }
 
@@ -223,7 +215,7 @@ if (  (s = dstdir,
   addr->transport_return = PANIC;
   addr->basic_errno = errno;
   addr->message = string_sprintf("%s transport accessing directory: %s "
-    "failed with error: %s", trname, s, strerror(errno));
+    "failed with error: %s", tblock->name, s, strerror(errno));
   if (ddfd >= 0) (void) close(ddfd);
   return FALSE;
   }
@@ -235,16 +227,16 @@ if (  (s = dstdir,    fstat(ddfd, &dstatbuf) < 0)
   addr->transport_return = PANIC;
   addr->basic_errno = errno;
   addr->message = string_sprintf("%s transport fstat on directory fd: "
-    "%s failed with error: %s", trname, s, strerror(errno));
+    "%s failed with error: %s", tblock->name, s, strerror(errno));
   goto RETURN;
   }
 can_link = (dstatbuf.st_dev == sstatbuf.st_dev);
 
-if (f.dont_deliver)
+if (dont_deliver)
   {
   DEBUG(D_transport)
     debug_printf("*** delivery by %s transport bypassed by -N option\n",
-      trname);
+      tblock->name);
   addr->transport_return = OK;
   goto RETURN;
   }
@@ -252,26 +244,26 @@ if (f.dont_deliver)
 /* Link or copy the header and data spool files */
 
 DEBUG(D_transport)
-  debug_printf("%s transport, copying header file\n", trname);
+  debug_printf("%s transport, copying header file\n", tblock->name);
 
 if (!copy_spool_files(tblock, addr, dstdir, sdfd, ddfd, can_link, -1))
   goto RETURN;
 
 DEBUG(D_transport)
-  debug_printf("%s transport, copying data file\n", trname);
+  debug_printf("%s transport, copying data file\n", tblock->name);
 
 if (!copy_spool_files(tblock, addr, dstdir, sdfd, ddfd, can_link,
 	deliver_datafile))
   {
   DEBUG(D_transport)
     debug_printf("%s transport, copying data file failed, "
-      "unlinking the header file\n", trname);
+      "unlinking the header file\n", tblock->name);
   Uunlink(string_sprintf("%s/%s-H", dstdir, message_id));
   goto RETURN;
   }
 
 DEBUG(D_transport)
-  debug_printf("%s transport succeeded\n", trname);
+  debug_printf("%s transport succeeded\n", tblock->name);
 
 addr->transport_return = OK;
 
@@ -284,30 +276,4 @@ put in the first address of a batch. */
 return FALSE;
 }
 
-
-
-
-# ifdef DYNLOOKUP
-#  define queuefile_transport_info _transport_info
-# endif
 #endif /*!MACRO_PREDEF*/
-
-transport_info queuefile_transport_info = {
-.drinfo = {
-  .driver_name =	US"queuefile",
-  .options =		queuefile_transport_options,
-  .options_count =	&queuefile_transport_options_count,
-  .options_block =	&queuefile_transport_option_defaults,
-  .options_len =	sizeof(queuefile_transport_options_block),
-  .init =		queuefile_transport_init,
-# ifdef DYNLOOKUP
-  .dyn_magic =		TRANSPORT_MAGIC,
-# endif
-  },
-.code =		queuefile_transport_entry,
-.tidyup =	NULL,
-.closedown =	NULL,
-.local =	TRUE
-};
-
-#endif /*EXPERIMENTAL_QUEUEFILE*/
